@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright 2022-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -61,10 +61,11 @@ from debusine.client.models import (
     WorkRequestResponse,
     WorkflowTemplateRequest,
     WorkflowTemplateResponse,
+    model_to_json_serializable_dict,
 )
 
 if TYPE_CHECKING:
-    from debusine.tasks.models import LookupMultiple, LookupSingle
+    from debusine.tasks.models import LookupMultiple, LookupSingle, OutputData
 
 
 class _MessageProcessor(Protocol):
@@ -160,12 +161,17 @@ class Debusine:
         return f"/work-request/{work_request_id}/completed/"
 
     def work_request_completed_update(
-        self, work_request_id: int, result: str
+        self, work_request_id: int, result: str, output_data: "OutputData"
     ) -> None:
         """Update work_request_id as completed with result."""
         return self._debusine_http_client.put(
             path=self.work_request_completed_path(work_request_id),
-            data={"result": result},
+            data={
+                "result": result,
+                "output_data": model_to_json_serializable_dict(
+                    output_data, exclude_unset=True
+                ),
+            },
             expected_class=None,
         )
 
