@@ -1,4 +1,4 @@
-# Copyright 2022-2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -67,7 +67,7 @@ from debusine.client.models import (
 )
 from debusine.client.tests import server
 from debusine.client.tests.server import DebusineAioHTTPTestCase
-from debusine.tasks.models import LookupMultiple
+from debusine.tasks.models import LookupMultiple, OutputData
 from debusine.test import TestCase
 from debusine.test.utils import (
     create_artifact_response,
@@ -257,7 +257,9 @@ class DebusineTests(TestCase):
             "",
         )
 
-        self.debusine.work_request_completed_update(work_request_id, "success")
+        self.debusine.work_request_completed_update(
+            work_request_id, "success", OutputData()
+        )
 
     @responses.activate
     def test_work_request_create_no_workspace_success(self) -> None:
