@@ -1,4 +1,4 @@
-# Copyright 2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -1763,3 +1763,197 @@ class MigrationTests(TestCase):
                 pk=work_request.pk
             ).workflow_runtime_status
         )
+
+    def test_autopkgtest_migrate_to_extra_repositories(self) -> None:
+        """`autopkgtest` tasks are migrated to extra_repositories."""
+        migrate_from = [("db", "0111_workrequest_workflow_runtime_status")]
+        migrate_to = [("db", "0112_workrequest_autopkgtest_extra_repositories")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        workspace = old_apps.get_model("db", "Workspace").objects.get(
+            scope__name=settings.DEBUSINE_DEFAULT_SCOPE,
+            name=DEFAULT_WORKSPACE_NAME,
+        )
+
+        old_work_requests = [
+            (
+                {
+                    "input": {"source_artifact": 1},
+                    "extra_apt_sources": [
+                        "deb http://example.com/ foo/",
+                        "deb http://example.net/ foo bar baz",
+                        # Will lose the options in the migration:
+                        (
+                            "deb [signed-by=/dev/null] http://example.org/ "
+                            "foo bar baz"
+                        ),
+                    ],
+                },
+            ),
+            (
+                {
+                    "input": {"source_artifact": 2},
+                },
+            ),
+            (
+                {
+                    "input": {"source_artifact": 3},
+                    # Will all be skipped on migration:
+                    "extra_apt_sources": [
+                        "deb-src http://example.com/deb-src foo bar",
+                        "some nonsense",
+                    ],
+                },
+            ),
+        ]
+
+        new_work_requests = [
+            (
+                {
+                    "input": {"source_artifact": 1},
+                    "extra_repositories": [
+                        {
+                            "url": "http://example.com/",
+                            "suite": "foo/",
+                        },
+                        {
+                            "url": "http://example.net/",
+                            "suite": "foo",
+                            "components": ["bar", "baz"],
+                        },
+                        {
+                            "url": "http://example.org/",
+                            "suite": "foo",
+                            "components": ["bar", "baz"],
+                        },
+                    ],
+                },
+            ),
+            (
+                {
+                    "input": {"source_artifact": 2},
+                },
+            ),
+            (
+                {
+                    "input": {"source_artifact": 3},
+                },
+            ),
+        ]
+
+        new_old_work_requests = [
+            (
+                {
+                    "input": {"source_artifact": 1},
+                    "extra_apt_sources": [
+                        "deb http://example.com/ foo/",
+                        "deb http://example.net/ foo bar baz",
+                        "deb http://example.org/ foo bar baz",
+                    ],
+                },
+            ),
+            (
+                {
+                    "input": {"source_artifact": 2},
+                },
+            ),
+            (
+                {
+                    "input": {"source_artifact": 3},
+                },
+            ),
+        ]
+
+        self.maxDiff = None
+        for (task_data,) in old_work_requests:
+            old_apps.get_model("db", "WorkRequest").objects.create(
+                workspace=workspace,
+                created_by_id=self.get_test_user_for_apps(old_apps).id,
+                task_type=TaskTypes.WORKER,
+                task_name="autopkgtest",
+                task_data=task_data,
+            )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+
+        self.assertQuerySetEqual(
+            new_apps.get_model("db", "WorkRequest")
+            .objects.order_by("id")
+            .values_list("task_data"),
+            new_work_requests,
+        )
+
+        w = new_apps.get_model("db", "WorkRequest").objects.get(
+            task_data__input__source_artifact=1
+        )
+        # Will be lost
+        w.task_data["extra_repositories"][0]["signing_key"] = "public key"
+        w.save()
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+
+        self.assertQuerySetEqual(
+            old_apps.get_model("db", "WorkRequest")
+            .objects.order_by("id")
+            .values_list("task_data"),
+            new_old_work_requests,
+        )
+
+    def test_workrequest_workflow_runtime_status_use_value(self) -> None:
+        """Test WorkRequest workflow_runtime_status uses value not UI string."""
+        migrate_from = [("db", "0114_user_manager_with_permissions")]
+        migrate_to = [
+            ("db", "0115_workrequest_workflow_runtime_status_use_value")
+        ]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+
+        old_work_request = old_apps.get_model(
+            "db", "WorkRequest"
+        ).objects.create(
+            created_by_id=self.get_test_user_for_apps(old_apps).id,
+            task_name="foo",
+            workspace=old_apps.get_model("db", "Workspace").objects.get(
+                name=DEFAULT_WORKSPACE_NAME
+            ),
+            workflow_runtime_status="Needs Input",
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+
+        NewWorkRequest = new_apps.get_model("db", "WorkRequest")
+
+        self.assertEqual(
+            NewWorkRequest.objects.get(
+                pk=old_work_request.pk
+            ).workflow_runtime_status,
+            "needs_input",
+        )
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+
+        OldWorkRequest = old_apps.get_model("db", "WorkRequest")
+
+        self.assertEqual(
+            OldWorkRequest.objects.get(
+                pk=old_work_request.pk
+            ).workflow_runtime_status,
+            "Needs Input",
+        )
