@@ -1,4 +1,4 @@
-# Copyright 2019, 2021-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -411,10 +411,10 @@ class FileInArtifactTests(TestCase):
     ) -> FileInArtifact:
         """Return FileInArtifact to be used in tests."""
         if fileobj is None:
-            fileobj = self.create_file()
+            fileobj = self.playground.create_file()
 
         if artifact is None:
-            artifact, _ = self.create_artifact()
+            artifact, _ = self.playground.create_artifact()
 
         if path is None:
             path = "/usr/bin/test"
@@ -426,10 +426,10 @@ class FileInArtifactTests(TestCase):
     @context.disable_permission_checks()
     def test_artifact_path_unique_constraint(self) -> None:
         """Test two FileInArtifact cannot have the same artifact and path."""
-        artifact, _ = self.create_artifact()
+        artifact, _ = self.playground.create_artifact()
 
-        file1 = self.create_file(b"contents1")
-        file2 = self.create_file(b"contents2")
+        file1 = self.playground.create_file(b"contents1")
+        file2 = self.playground.create_file(b"contents2")
 
         self.create_file_in_artifact(file1, artifact, "/usr/bin/test")
 
@@ -479,7 +479,7 @@ class FileUploadTests(TestCase):
     @context.disable_permission_checks()
     def setUp(self) -> None:
         """Set up basic objects for the tests."""
-        self.file_upload = self.create_file_upload()
+        self.file_upload = self.playground.create_file_upload()
         self.artifact = self.file_upload.file_in_artifact.artifact
         self.workspace = self.artifact.workspace
         self.file_path = self.file_upload.file_in_artifact.path
@@ -504,7 +504,9 @@ class FileUploadTests(TestCase):
             category="test", workspace=self.workspace
         )
         file_in_artifact = FileInArtifact.objects.create(
-            artifact=artifact, path="something", file=self.create_file()
+            artifact=artifact,
+            path="something",
+            file=self.playground.create_file(),
         )
 
         with self.assertRaisesRegex(
@@ -529,7 +531,7 @@ class FileUploadTests(TestCase):
     def test_no_two_fileobj_to_same_path(self) -> None:
         """Test cannot create two FileUpload with same path."""
         data = next(data_generator(self.file_size))
-        file = self.create_file(data)
+        file = self.playground.create_file(data)
         new_file_in_artifact = FileInArtifact.objects.create(
             artifact=self.artifact, path="README-2", file=file
         )
@@ -625,7 +627,7 @@ class ArtifactRelationTests(TestCase):
     @context.disable_permission_checks()
     def setUp(self) -> None:
         """Initialize test object."""
-        self.artifact_1, _ = self.create_artifact()
+        self.artifact_1, _ = self.playground.create_artifact()
         self.artifact_2 = Artifact.objects.create(
             workspace=self.artifact_1.workspace,
             category=self.artifact_1.category,
