@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -275,7 +275,7 @@ class CollectionTests(TestCase):
             created_by_user=self.scenario.user,
         )
 
-        artifact, _ = self.create_artifact()
+        artifact, _ = self.playground.create_artifact()
         artifact_item_collection_1 = CollectionItem.objects.create(
             name="test-2",
             category="test-2",
@@ -469,7 +469,7 @@ class CollectionItemManagerTests(TestCase):
     def test_create_from_artifact(self) -> None:
         """Verify create_from_artifact method."""
         data = {"a": "b"}
-        artifact, _ = self.create_artifact(data=data)
+        artifact, _ = self.playground.create_artifact(data=data)
 
         name = "some-name"
         collection_item = CollectionItem.objects.create_from_artifact(
@@ -526,7 +526,7 @@ class CollectionItemManagerTests(TestCase):
     def test_drop_full_history(self) -> None:
         """Verify drop_full_history method."""
         data = {"a": "b"}
-        artifact, _ = self.create_artifact(data=data)
+        artifact, _ = self.playground.create_artifact(data=data)
 
         self.collection.full_history_retention_period = timedelta(days=2)
         self.collection.save()
@@ -578,7 +578,7 @@ class CollectionItemManagerTests(TestCase):
     def test_drop_full_history_no_retention_period(self) -> None:
         """Verify drop_full_history method without retention_period."""
         data = {"a": "b"}
-        artifact, _ = self.create_artifact(data=data)
+        artifact, _ = self.playground.create_artifact(data=data)
 
         collection_item_removed = CollectionItem.objects.create(
             parent_collection=self.collection,
@@ -627,7 +627,7 @@ class CollectionItemManagerTests(TestCase):
     def test_drop_metadata(self) -> None:
         """Verify drop_metadata method."""
         data = {"a": "b"}
-        artifact, _ = self.create_artifact(data=data)
+        artifact, _ = self.playground.create_artifact(data=data)
 
         self.collection.full_history_retention_period = timedelta(days=1)
         self.collection.metadata_only_retention_period = timedelta(days=1)
@@ -680,7 +680,7 @@ class CollectionItemManagerTests(TestCase):
     def test_drop_metadata_no_retention_period(self) -> None:
         """Verify drop_metadata method."""
         data = {"a": "b"}
-        artifact, _ = self.create_artifact(data=data)
+        artifact, _ = self.playground.create_artifact(data=data)
 
         collection_item_removed = CollectionItem.objects.create(
             parent_collection=self.collection,
@@ -732,7 +732,7 @@ class CollectionItemTests(TestCase):
     @context.disable_permission_checks()
     def setUp(self) -> None:
         """Create objects for the tests."""
-        self.artifact, _ = self.create_artifact()
+        self.artifact, _ = self.playground.create_artifact()
 
         self.workspace = self.playground.create_workspace(name="System")
         self.collection = Collection.objects.create(
@@ -1008,7 +1008,7 @@ class CollectionItemTests(TestCase):
         category = ArtifactCategory.SYSTEM_TARBALL
         data = {"codename": "bookworm", "architecture": "amd64"}
 
-        artifact_1, _ = self.create_artifact(category=category)
+        artifact_1, _ = self.playground.create_artifact(category=category)
         CollectionItem.objects.create(
             name="bookworm-amd64",
             category=category,
@@ -1021,7 +1021,7 @@ class CollectionItemTests(TestCase):
         )
 
         # Can be created because the previous one is removed
-        artifact_2, _ = self.create_artifact(category=category)
+        artifact_2, _ = self.playground.create_artifact(category=category)
         CollectionItem.objects.create(
             name="bookworm-amd64-2",
             category=category,
@@ -1033,7 +1033,7 @@ class CollectionItemTests(TestCase):
         )
 
         # Can be created because it's different variant
-        artifact_3, _ = self.create_artifact(category=category)
+        artifact_3, _ = self.playground.create_artifact(category=category)
         CollectionItem.objects.create(
             name="bookworm-amd64-3",
             category=category,
@@ -1045,7 +1045,7 @@ class CollectionItemTests(TestCase):
         )
 
         # Can be created because it's different backend
-        artifact_4, _ = self.create_artifact(category=category)
+        artifact_4, _ = self.playground.create_artifact(category=category)
         CollectionItem.objects.create(
             name="bookworm-amd64-4",
             category=category,
@@ -1057,7 +1057,7 @@ class CollectionItemTests(TestCase):
         )
 
         # Can be created: duplicated but in a different collection
-        artifact_5, _ = self.create_artifact(category=category)
+        artifact_5, _ = self.playground.create_artifact(category=category)
         CollectionItem.objects.create(
             name="bookworm-amd64-5",
             category=category,
@@ -1074,7 +1074,7 @@ class CollectionItemTests(TestCase):
 
         # Cannot be created because already one active artifact
         # with the same data in the same collection (from artifact_3).
-        artifact_6, _ = self.create_artifact(category=category)
+        artifact_6, _ = self.playground.create_artifact(category=category)
         msg = "db_collectionitem_unique_debian_environment"
         with self.assertRaisesRegex(IntegrityError, msg):
             CollectionItem.objects.create(
