@@ -1,4 +1,4 @@
-# Copyright 2019, 2021-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -64,7 +64,7 @@ class WorkspaceManagerTests(TestCase):
         """Test the get_absolute_url method."""
         self.assertEqual(
             self.scenario.workspace.get_absolute_url(),
-            f"/{settings.DEBUSINE_DEFAULT_SCOPE}/System/view/",
+            f"/{settings.DEBUSINE_DEFAULT_SCOPE}/System/",
         )
 
     def test_get_roles_model(self) -> None:
@@ -574,7 +574,7 @@ class WorkspaceTests(TestCase):
 
         The file is added in the default FileStore.
         """
-        fileobj = self.create_file()
+        fileobj = self.playground.create_file()
         workspace = default_workspace()
         self.assertQuerySetEqual(workspace.file_stores(fileobj), [])
         self.assertFalse(workspace.is_file_in_workspace(fileobj))
@@ -591,7 +591,9 @@ class WorkspaceTests(TestCase):
 
         # Add file to an artifact in another workspace
         other_workspace = self.playground.create_workspace(name="other")
-        other_artifact, _ = self.create_artifact(workspace=other_workspace)
+        other_artifact, _ = self.playground.create_artifact(
+            workspace=other_workspace
+        )
         FileInArtifact.objects.create(
             artifact=other_artifact, path="test", file=fileobj
         )
@@ -602,11 +604,11 @@ class WorkspaceTests(TestCase):
         self.assertFalse(workspace.is_file_in_workspace(fileobj))
 
         # Add different but complete file to an artifact in this workspace
-        artifact, _ = self.create_artifact(workspace=workspace)
+        artifact, _ = self.playground.create_artifact(workspace=workspace)
         FileInArtifact.objects.create(
             artifact=artifact,
             path="other",
-            file=self.create_file(b"other"),
+            file=self.playground.create_file(b"other"),
             complete=True,
         )
 
@@ -641,7 +643,7 @@ class WorkspaceTests(TestCase):
 
         The file is added in a FileStore that is the non-default.
         """
-        fileobj = self.create_file()
+        fileobj = self.playground.create_file()
         workspace = default_workspace()
         self.assertQuerySetEqual(workspace.file_stores(fileobj), [])
         self.assertFalse(workspace.is_file_in_workspace(fileobj))
@@ -663,7 +665,7 @@ class WorkspaceTests(TestCase):
         self.assertFalse(workspace.is_file_in_workspace(fileobj))
 
         # Add file to an artifact in this workspace
-        artifact, _ = self.create_artifact(workspace=workspace)
+        artifact, _ = self.playground.create_artifact(workspace=workspace)
         FileInArtifact.objects.create(
             artifact=artifact, path="test", file=fileobj, complete=True
         )
