@@ -1,4 +1,4 @@
-# Copyright 2019, 2021-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -15,6 +15,7 @@ from collections.abc import Generator
 from contextlib import contextmanager
 from datetime import datetime, timedelta
 from functools import cached_property
+from itertools import chain
 from typing import (
     Any,
     Generic,
@@ -66,6 +67,7 @@ from debusine.tasks.models import (
     LookupMultiple,
     LookupSingle,
     NotificationDataEmail,
+    OutputData,
     TaskTypes,
 )
 from debusine.tasks.server import TaskDatabaseInterface
@@ -114,7 +116,7 @@ class WorkflowTemplateQuerySet(QuerySet["WorkflowTemplate", A], Generic[A]):
     def can_display(
         self, user: PermissionUser
     ) -> "WorkflowTemplateQuerySet[A]":
-        """Keep only WorkRequests that can be displayed."""
+        """Keep only WorkflowTemplates that can be displayed."""
         # Delegate to workspace can_display check
         return self.filter(workspace__in=Workspace.objects.can_display(user))
 
@@ -188,6 +190,39 @@ class WorkflowTemplate(models.Model):
         # Delegate to workspace can_display check
         return self.workspace.can_display(user)
 
+    def completed_workflows(self) -> int:
+        """Count completed workflows for this template in context.workspace."""
+        return (
+            self.workrequest_set.filter(workspace=context.workspace)
+            .filter(status=WorkRequest.Statuses.COMPLETED)
+            .count()
+        )
+
+    def running_workflows(self) -> int:
+        """Count completed workflows for this template in context.workspace."""
+        return (
+            self.workrequest_set.in_current_workspace()
+            .filter(status=WorkRequest.Statuses.RUNNING)
+            .count()
+        )
+
+    def needs_input_workflows(self) -> int:
+        """Count completed workflows for this template in context.workspace."""
+        return (
+            self.workrequest_set.filter(workspace=context.workspace)
+            .filter(
+                workflow_runtime_status=WorkRequest.RuntimeStatuses.NEEDS_INPUT
+            )
+            .count()
+        )
+
+    def get_absolute_url(self) -> str:
+        """Return the canonical display URL."""
+        return reverse(
+            "workspaces:workflow_templates:detail",
+            kwargs={"wname": self.workspace.name, "name": self.name},
+        )
+
 
 class _WorkRequestStatuses(models.TextChoices):
     """Choices for WorkRequest.status."""
@@ -458,13 +493,13 @@ class WorkRequestManager(models.Manager["WorkRequest"]):
 
 
 class _RuntimeStatuses(models.TextChoices):
-    NEEDS_INPUT = "Needs_Input", "Needs Input"
-    RUNNING = "Running", "Running"
-    WAITING = "Waiting", "Waiting"
-    PENDING = "Pending", "Pending"
-    ABORTED = "Aborted", "Aborted"
-    COMPLETED = "Completed", "Completed"
-    BLOCKED = "Blocked", "Blocked"
+    NEEDS_INPUT = "needs_input", "Needs Input"
+    RUNNING = "running", "Running"
+    WAITING = "waiting", "Waiting"
+    PENDING = "pending", "Pending"
+    ABORTED = "aborted", "Aborted"
+    COMPLETED = "completed", "Completed"
+    BLOCKED = "blocked", "Blocked"
 
 
 class WorkRequest(models.Model):
@@ -605,6 +640,12 @@ class WorkRequest(models.Model):
             "Administrator adjustment to the priority of this work request"
         ),
     )
+    output_data_json = JSONField(
+        null=True,
+        blank=True,
+        db_column="output_data",
+        encoder=DjangoJSONEncoder,
+    )
 
     # Workflows
     unblock_strategy = models.CharField(
@@ -754,6 +795,20 @@ class WorkRequest(models.Model):
                 assert_never(unreachable)
         return WorkRequest.objects.can_unblock(user).filter(pk=self.pk).exists()
 
+    @property
+    def output_data(self) -> OutputData | None:
+        """Access output_data_json as a pydantic structure."""
+        return (
+            None
+            if self.output_data_json is None
+            else OutputData(**self.output_data_json)
+        )
+
+    @output_data.setter
+    def output_data(self, value: OutputData | None) -> None:
+        """Set output_data_json from a pydantic structure."""
+        self.output_data_json = None if value is None else value.dict()
+
     @property
     def workflow_data(self) -> "WorkRequestWorkflowData":
         """Access workflow_data_json as a python structure."""
@@ -1204,7 +1259,11 @@ class WorkRequest(models.Model):
         logger.debug("Marked WorkRequest %s as running", self.pk)
         return True
 
-    def mark_completed(self, result: "WorkRequest.Results") -> bool:
+    def mark_completed(
+        self,
+        result: "WorkRequest.Results",
+        output_data: OutputData | None = None,
+    ) -> bool:
         """Worker has finished executing the task."""
         if self.status not in (self.Statuses.PENDING, self.Statuses.RUNNING):
             logger.debug(
@@ -1219,6 +1278,7 @@ class WorkRequest(models.Model):
         self.result = result
         self.completed_at = timezone.now()
         self.status = self.Statuses.COMPLETED
+        self.output_data = output_data
         self.save()
 
         if self.parent is not None:
@@ -1275,7 +1335,15 @@ class WorkRequest(models.Model):
         actions: dict[str, list[EventReaction]] = {
             action: [] for action in ActionTypes
         }
-        for action in getattr(self.event_reactions, event_name):
+        try:
+            task = self.get_task()
+        except Exception:
+            task_event_reactions = []
+        else:
+            task_event_reactions = task.get_event_reactions(event_name)
+        for action in chain(
+            task_event_reactions, getattr(self.event_reactions, event_name)
+        ):
             action_type = action.action
             actions[action_type].append(action)
         return actions
