@@ -1,4 +1,4 @@
-# Copyright 2022-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -10,9 +10,10 @@
 """Infrastructure to create test scenarios in the database."""
 
 import tempfile
+from dataclasses import dataclass
 from datetime import timedelta
 from pathlib import Path
-from typing import Any, TYPE_CHECKING, assert_never
+from typing import Any, Literal, TYPE_CHECKING, assert_never, overload
 
 from django.conf import settings
 from django.contrib.auth.models import Permission
@@ -22,6 +23,7 @@ from django.test import override_settings
 from django.utils import timezone
 
 from debusine.artifacts import LocalArtifact
+from debusine.artifacts.local_artifact import BinaryPackage, SourcePackage
 from debusine.artifacts.models import (
     ArtifactCategory,
     ArtifactData,
@@ -78,6 +80,39 @@ if TYPE_CHECKING:
     from debusine.db.playground.scenarios import Scenario
 
 
+@dataclass(kw_only=True)
+class UploadArtifacts:
+    """The return value for Playground.create_upload_artifacts()."""
+
+    upload: Artifact
+    source: Artifact | None
+    binaries: list[Artifact] | None
+
+
+@dataclass(kw_only=True)
+class SourceUploadArtifacts(UploadArtifacts):
+    """UploadArtifacts for a source-only upload."""
+
+    source: Artifact
+    binaries: None
+
+
+@dataclass(kw_only=True)
+class BinaryUploadArtifacts(UploadArtifacts):
+    """UploadArtifacts for a binary-only upload."""
+
+    source: None
+    binaries: list[Artifact]
+
+
+@dataclass(kw_only=True)
+class MixedUploadArtifacts(UploadArtifacts):
+    """UploadArtifacts for a mixed upload."""
+
+    source: Artifact
+    binaries: list[Artifact]
+
+
 class Playground(ArtifactPlayground):
     """Generate data scenarios for the database."""
 
@@ -88,6 +123,8 @@ class Playground(ArtifactPlayground):
         default_username: str = "playground",
         default_user_password: str | None = None,
         default_user_email: str = "playground@example.org",
+        default_user_first_name: str = "Play",
+        default_user_last_name: str = "Ground",
     ):
         """Set default values."""
         super().__init__()
@@ -96,6 +133,8 @@ class Playground(ArtifactPlayground):
         self.default_username = default_username
         self.default_user_password = default_user_password
         self.default_user_email = default_user_email
+        self.default_user_first_name = default_user_first_name
+        self.default_user_last_name = default_user_last_name
         # Registry of scenarios created by this playground.
         #
         # This can be used when instantiating multiple scenarios, allowing one
@@ -111,6 +150,8 @@ class Playground(ArtifactPlayground):
             user = User.objects.create_user(
                 username=self.default_username,
                 email=self.default_user_email,
+                first_name=self.default_user_first_name,
+                last_name=self.default_user_last_name,
             )
             if self.default_user_password is not None:
                 user.set_password(self.default_user_password)
@@ -703,10 +744,10 @@ class Playground(ArtifactPlayground):
     @context.disable_permission_checks()
     def create_minimal_binary_packages_artifact(
         self,
-        srcpkg_name: str,
-        srcpkg_version: str,
-        version: str,
-        architecture: str,
+        srcpkg_name: str = "hello",
+        srcpkg_version: str = "1.0-1",
+        version: str = "1.0-1",
+        architecture: str = "all",
     ) -> Artifact:
         """Create a minimal `debian:binary-packages` artifact."""
         artifact, _ = self.create_artifact(
@@ -721,9 +762,49 @@ class Playground(ArtifactPlayground):
         )
         return artifact
 
-    @context.disable_permission_checks()
-    def create_upload_artifact(
+    @overload
+    def create_upload_artifacts(
         self,
+        *,
+        src_name: str = ...,
+        version: str = ...,
+        source: Literal[True] = True,
+        binary: Literal[False],
+        binaries: list[str] | None = ...,
+        workspace: Workspace | None = ...,
+        created_by: User | None = ...,
+    ) -> SourceUploadArtifacts: ...
+
+    @overload
+    def create_upload_artifacts(
+        self,
+        *,
+        src_name: str = ...,
+        version: str = ...,
+        source: Literal[False],
+        binary: Literal[True] = True,
+        binaries: list[str] | None = ...,
+        workspace: Workspace | None = ...,
+        created_by: User | None = ...,
+    ) -> BinaryUploadArtifacts: ...
+
+    @overload
+    def create_upload_artifacts(
+        self,
+        *,
+        src_name: str = ...,
+        version: str = ...,
+        source: Literal[True] = True,
+        binary: Literal[True] = True,
+        binaries: list[str] | None = ...,
+        workspace: Workspace | None = ...,
+        created_by: User | None = ...,
+    ) -> MixedUploadArtifacts: ...
+
+    @context.disable_permission_checks()
+    def create_upload_artifacts(
+        self,
+        *,
         src_name: str = "hello",
         version: str = "1.0-1",
         source: bool = True,
@@ -732,11 +813,18 @@ class Playground(ArtifactPlayground):
         workspace: Workspace | None = None,
         created_by: User | None = None,
         create_files: bool = False,
-    ) -> Artifact:
-        """Create an artifact for an upload package."""
+    ) -> UploadArtifacts:
+        """Create a set of artifacts for a package upload."""
+        if binaries is None:
+            binaries = [src_name]
+        local_artifacts: dict[str, list[LocalArtifact[Any]]] = {
+            "source": [],
+            "binaries": [],
+        }
+        artifacts: dict[str, list[Artifact]] = {}
         with tempfile.TemporaryDirectory() as tempdir:
             workdir = Path(tempdir)
-            source_package = self.create_upload(
+            upload = self.create_upload(
                 workdir,
                 src_name=src_name,
                 version=version,
@@ -744,12 +832,55 @@ class Playground(ArtifactPlayground):
                 binary=binary,
                 binaries=binaries,
             )
-            return self.create_artifact_from_local(
-                source_package,
-                workspace=workspace,
-                created_by=created_by,
-                create_files=create_files,
+            local_artifacts["upload"] = [upload]
+            if source:
+                source_package = SourcePackage.create(
+                    name=src_name,
+                    version=version,
+                    files=[
+                        file
+                        for file in upload.files.values()
+                        if file.name.endswith((".dsc", ".tar.gz", ".tar.xz"))
+                    ],
+                )
+                local_artifacts["source"] = [source_package]
+            if binary:
+                binary_packages: list[LocalArtifact[Any]] = [
+                    BinaryPackage.create(file=file)
+                    for file in upload.files.values()
+                    if file.suffix == ".deb"
+                ]
+                local_artifacts["binaries"] = binary_packages
+
+            for key, values in local_artifacts.items():
+                artifacts[key] = [
+                    self.create_artifact_from_local(
+                        local_artifact,
+                        workspace=workspace,
+                        created_by=created_by,
+                        create_files=create_files,
+                    )
+                    for local_artifact in values
+                ]
+
+        if source:
+            self.create_artifact_relation(
+                artifact=artifacts["upload"][0],
+                target=artifacts["source"][0],
+                relation_type=ArtifactRelation.Relations.EXTENDS,
             )
+        for binary_artifact in artifacts["binaries"]:
+            self.create_artifact_relation(
+                artifact=artifacts["upload"][0],
+                target=binary_artifact,
+                relation_type=ArtifactRelation.Relations.EXTENDS,
+            )
+
+        return UploadArtifacts(
+            upload=artifacts["upload"][0],
+            source=artifacts["source"][0] if source else None,
+            binaries=artifacts["binaries"] or None,
+        )
 
     @context.disable_permission_checks()
     def create_build_log_artifact(
