@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -135,6 +135,19 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         )
         self.assertEqual(ws2.default_file_store, custom_store)
 
+    def test_create_file_in_backend(self) -> None:
+        """create_file_in_backend() returns a File and writes contents to it."""
+        playground = self.enterContext(Playground(memory_file_store=False))
+        file_backend = FileStore.default().get_backend_object()
+        contents = b"some-test-data"
+
+        fileobj = playground.create_file_in_backend(file_backend, contents)
+
+        self.assertIsInstance(fileobj.id, int)
+        local_path = file_backend.get_local_path(fileobj)
+        assert local_path is not None
+        self.assertEqual(local_path.read_bytes(), contents)
+
     def test_compute_dynamic_data_noop(self) -> None:
         """Test compute_dynamic_data with empty result."""
         playground = Playground()
@@ -217,10 +230,11 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(files[1].path, "hello_1.0-1.dsc")
         self.assertEqual(files[2].path, "hello_1.0.orig.tar.gz")
 
-    def test_create_upload_artifact(self) -> None:
-        """Test creating a upload artifact."""
+    def test_create_upload_artifacts(self) -> None:
+        """Test creating a set of upload artifacts."""
         playground = Playground()
-        upload = playground.create_upload_artifact()
+        upload_artifacts = playground.create_upload_artifacts(binaries=["bin1"])
+        upload = upload_artifacts.upload
         self.assertEqual(upload.category, ArtifactCategory.UPLOAD)
         self.assertEqual(upload.workspace, playground.get_default_workspace())
         self.assertEqual(upload.files.count(), 0)
@@ -229,6 +243,104 @@ class PlaygroundTest(django.test.TestCase, TestCase):
 
         DebianUpload(**upload.data)
 
+        source = upload_artifacts.source
+        self.assertIsNotNone(source)
+        self.assertEqual(source.category, ArtifactCategory.SOURCE_PACKAGE)
+        self.assertEqual(source.workspace, playground.get_default_workspace())
+        self.assertEqual(source.files.count(), 0)
+        self.assertEqual(source.created_by, playground.get_default_user())
+        self.assertIsNone(source.created_by_work_request)
+        self.assertTrue(
+            ArtifactRelation.objects.filter(
+                artifact=upload,
+                target=source,
+                type=ArtifactRelation.Relations.EXTENDS,
+            ).exists()
+        )
+
+        binaries = upload_artifacts.binaries
+        self.assertIsNotNone(binaries)
+        self.assertEqual(len(binaries), 1)
+        binary = binaries[0]
+        self.assertEqual(binary.category, ArtifactCategory.BINARY_PACKAGE)
+        self.assertEqual(binary.workspace, playground.get_default_workspace())
+        self.assertEqual(binary.files.count(), 0)
+        self.assertEqual(binary.created_by, playground.get_default_user())
+        self.assertIsNone(binary.created_by_work_request)
+
+        self.assertTrue(
+            ArtifactRelation.objects.filter(
+                artifact=upload,
+                target=binary,
+                type=ArtifactRelation.Relations.EXTENDS,
+            ).exists()
+        )
+
+    def test_create_upload_artifacts_no_binaries(self) -> None:
+        """Test creating a set of upload source artifacts."""
+        playground = Playground()
+        upload_artifacts = playground.create_upload_artifacts(binary=False)
+        upload = upload_artifacts.upload
+        self.assertEqual(upload.category, ArtifactCategory.UPLOAD)
+        self.assertEqual(upload.workspace, playground.get_default_workspace())
+        self.assertEqual(upload.files.count(), 0)
+        self.assertEqual(upload.created_by, playground.get_default_user())
+        self.assertIsNone(upload.created_by_work_request)
+
+        DebianUpload(**upload.data)
+
+        source = upload_artifacts.source
+        self.assertIsNotNone(source)
+        self.assertEqual(source.category, ArtifactCategory.SOURCE_PACKAGE)
+        self.assertEqual(source.workspace, playground.get_default_workspace())
+        self.assertEqual(source.files.count(), 0)
+        self.assertEqual(source.created_by, playground.get_default_user())
+        self.assertIsNone(source.created_by_work_request)
+        self.assertTrue(
+            ArtifactRelation.objects.filter(
+                artifact=upload,
+                target=source,
+                type=ArtifactRelation.Relations.EXTENDS,
+            ).exists()
+        )
+
+        self.assertIsNone(upload_artifacts.binaries)
+
+    def test_create_upload_artifacts_binary_only(self) -> None:
+        """Test creating a set of upload binary artifacts."""
+        playground = Playground()
+        upload_artifacts = playground.create_upload_artifacts(
+            source=False, binaries=["foo"]
+        )
+        upload = upload_artifacts.upload
+        self.assertEqual(upload.category, ArtifactCategory.UPLOAD)
+        self.assertEqual(upload.workspace, playground.get_default_workspace())
+        self.assertEqual(upload.files.count(), 0)
+        self.assertEqual(upload.created_by, playground.get_default_user())
+        self.assertIsNone(upload.created_by_work_request)
+
+        DebianUpload(**upload.data)
+
+        self.assertIsNone(upload_artifacts.source)
+
+        binaries = upload_artifacts.binaries
+        self.assertIsNotNone(binaries)
+        self.assertEqual(len(binaries), 1)
+        binary = binaries[0]
+        self.assertEqual(binary.category, ArtifactCategory.BINARY_PACKAGE)
+        self.assertEqual(binary.workspace, playground.get_default_workspace())
+        self.assertEqual(binary.files.count(), 0)
+        self.assertEqual(binary.created_by, playground.get_default_user())
+        self.assertIsNone(binary.created_by_work_request)
+
+        self.assertTrue(
+            ArtifactRelation.objects.filter(
+                artifact=upload,
+                target=binary,
+                type=ArtifactRelation.Relations.EXTENDS,
+            ).exists()
+        )
+
     def test_create_build_log_artifact(self) -> None:
         """Test creating a build log artifact."""
         playground = Playground()
