@@ -1,4 +1,4 @@
-# Copyright 2019, 2021-2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -36,6 +36,7 @@ from django.contrib.auth import views as auth_views
 from django.urls import URLPattern, URLResolver, include, path, re_path
 from django.views.generic import RedirectView
 
+from debusine.web.views.auth import LogoutView
 from debusine.web.views.task_status import TaskStatusView
 
 # from django.conf import settings
@@ -62,6 +63,14 @@ service_urlpatterns: list[URLPattern | URLResolver] = [
         ),
         name="login",
     ),
+    path(
+        "logout/",
+        LogoutView.as_view(
+            template_name="account/logged_out.html",
+            extra_context={"is_logout_view": True},
+        ),
+        name="logout",
+    ),
     path(
         "signon/",
         include("debusine.web.urls.signon", namespace="signon"),
