@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -78,7 +78,10 @@ class LookupMixin(TestCase):
             name=name,
             child_type=CollectionItem.Types.ARTIFACT,
             category=category,
-            artifact=artifact or self.create_artifact(category=category)[0],
+            artifact=(
+                artifact
+                or self.playground.create_artifact(category=category)[0]
+            ),
             data=data or {},
             created_by_user=self.user,
         )
@@ -107,7 +110,7 @@ class LookupMixin(TestCase):
 
     def create_source_package(self, name: str, version: str) -> Artifact:
         """Create a minimal `debian:source-package` artifact."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.SOURCE_PACKAGE,
             data={
                 "name": name,
@@ -122,7 +125,7 @@ class LookupMixin(TestCase):
         self, name: str, version: str, architecture: str
     ) -> Artifact:
         """Create a minimal `debian:binary-package` artifact."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.BINARY_PACKAGE,
             data={
                 "srcpkg_name": name,
@@ -216,7 +219,7 @@ class LookupSingleTests(LookupMixin, TestCase):
     @context.disable_permission_checks()
     def test_specific_artifact(self) -> None:
         """`nnn@artifacts` finds a specific artifact."""
-        artifacts = [self.create_artifact()[0] for _ in range(3)]
+        artifacts = [self.playground.create_artifact()[0] for _ in range(3)]
 
         for i, artifact in enumerate(artifacts):
             lookup = f"{artifact.id}@artifacts"
@@ -226,7 +229,7 @@ class LookupSingleTests(LookupMixin, TestCase):
     @context.disable_permission_checks()
     def test_specific_artifact_integer(self) -> None:
         """Integer lookups find a specific artifact."""
-        artifacts = [self.create_artifact()[0] for _ in range(3)]
+        artifacts = [self.playground.create_artifact()[0] for _ in range(3)]
 
         for i, artifact in enumerate(artifacts):
             with self.subTest(artifact=artifact):
@@ -256,7 +259,7 @@ class LookupSingleTests(LookupMixin, TestCase):
                 name="public", public=True
             )
             artifacts = [
-                self.create_artifact(workspace=workspace)[0]
+                self.playground.create_artifact(workspace=workspace)[0]
                 for workspace in private_workspaces + [public_workspace]
             ]
 
@@ -681,7 +684,7 @@ class LookupMultipleTests(LookupMixin, TestCase):
     @context.disable_permission_checks()
     def test_not_collection(self) -> None:
         """The `collection` key does not identify a collection."""
-        artifact = self.create_artifact()[0]
+        artifact = self.playground.create_artifact()[0]
         lookup = {"collection": f"{artifact.id}@artifacts"}
 
         with self.assertRaisesRegex(
@@ -1108,7 +1111,9 @@ class LookupMultipleTests(LookupMixin, TestCase):
                 ("hello_1.0", {"package": "hello", "version": "1.0"}),
             )
         ]
-        loose_artifacts = [self.create_artifact()[0] for _ in range(3)]
+        loose_artifacts = [
+            self.playground.create_artifact()[0] for _ in range(3)
+        ]
 
         self.assert_lookup_equal(
             [
@@ -1138,7 +1143,7 @@ class LookupMultipleTests(LookupMixin, TestCase):
     @context.disable_permission_checks()
     def test_artifact_expired(self) -> None:
         """If one artifact has expired, `lookup_multiple` raises LookupError."""
-        artifacts = [self.create_artifact()[0] for _ in range(3)]
+        artifacts = [self.playground.create_artifact()[0] for _ in range(3)]
         artifact_ids = [artifact.id for artifact in artifacts]
         artifacts[0].delete()
 
@@ -1237,7 +1242,9 @@ class ReconstructLookupTests(LookupMixin, TestCase):
     @context.disable_permission_checks()
     def test_artifact(self) -> None:
         """A result with an artifact returns a lookup for that artifact."""
-        artifact, _ = self.create_artifact(category=ArtifactCategory.TEST)
+        artifact, _ = self.playground.create_artifact(
+            category=ArtifactCategory.TEST
+        )
 
         self.assertEqual(
             reconstruct_lookup(self.make_result(artifact)),
