@@ -1,4 +1,4 @@
-# Copyright 2021-2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -64,10 +64,10 @@ class DeleteExpiredCommandTests(TestCase):
     def setUpTestData(cls) -> None:
         """Set up common data for tests."""
         super().setUpTestData()
-        artifact_1, _ = cls.create_artifact(expiration_delay=1)
+        artifact_1, _ = cls.playground.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
         artifact_1.save()
-        artifact_2, _ = cls.create_artifact(expiration_delay=1)
+        artifact_2, _ = cls.playground.create_artifact(expiration_delay=1)
         artifact_2.created_at = timezone.now() - timedelta(days=2)
         artifact_2.save()
 
@@ -118,7 +118,7 @@ class DeleteExpiredCommandTests(TestCase):
             artifact_id_1, artifact_id_2 = Artifact.objects.all()
 
             # Add an artifact with a file
-            artifact_id_3, _ = self.create_artifact(
+            artifact_id_3, _ = self.playground.create_artifact(
                 paths=["README"], create_files=True, expiration_delay=1
             )
             artifact_id_3.created_at = timezone.now() - timedelta(days=2)
@@ -239,21 +239,21 @@ class DeleteExpiredArtifactsTests(TestCase):
     @context.disable_permission_checks()
     def test_mark_to_keep_artifact_no_expiration_date(self) -> None:
         """One artifact, not expired: keep it."""
-        artifact, _ = self.create_artifact(expiration_delay=0)
+        artifact, _ = self.playground.create_artifact(expiration_delay=0)
 
         self.assertEqual(self.delete_expired._mark_to_keep(), {artifact.id})
 
     @context.disable_permission_checks()
     def test_mark_to_keep_artifact_expires_tomorrow(self) -> None:
         """One artifact, not expired (expires tomorrow): keep it."""
-        artifact, _ = self.create_artifact(expiration_delay=1)
+        artifact, _ = self.playground.create_artifact(expiration_delay=1)
 
         self.assertEqual(self.delete_expired._mark_to_keep(), {artifact.id})
 
     @context.disable_permission_checks()
     def test_mark_to_keep_only_artifact_is_expired(self) -> None:
         """One artifact, expired yesterday. No artifacts are kept."""
-        artifact_1, _ = self.create_artifact(expiration_delay=1)
+        artifact_1, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
         artifact_1.save()
 
@@ -277,7 +277,7 @@ class DeleteExpiredArtifactsTests(TestCase):
     @context.disable_permission_checks()
     def test_mark_to_keep_only_artifact_is_retained(self) -> None:
         """One artifact, expired yesterday. retains_artifacts is set."""
-        artifact_1, _ = self.create_artifact(expiration_delay=1)
+        artifact_1, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
         artifact_1.save()
 
@@ -319,7 +319,9 @@ class DeleteExpiredArtifactsTests(TestCase):
                 Collection.RetainsArtifacts.WORKFLOW
             )
             collections[status].save()
-            artifacts[status], _ = self.create_artifact(expiration_delay=1)
+            artifacts[status], _ = self.playground.create_artifact(
+                expiration_delay=1
+            )
             artifacts[status].created_at = timezone.now() - timedelta(days=2)
             artifacts[status].save()
             collections[status].manager.add_artifact(
@@ -340,12 +342,18 @@ class DeleteExpiredArtifactsTests(TestCase):
     @context.disable_permission_checks()
     def test_mark_to_keep_all_artifacts_non_expired_or_targeted(self) -> None:
         """Keep both artifacts: the expired is targeted by the not-expired."""
-        artifact_expired, _ = self.create_artifact(expiration_delay=1)
+        artifact_expired, _ = self.playground.create_artifact(
+            expiration_delay=1
+        )
         artifact_expired.created_at = timezone.now() - timedelta(days=2)
         artifact_expired.save()
-        artifact_not_expired, _ = self.create_artifact(expiration_delay=1)
+        artifact_not_expired, _ = self.playground.create_artifact(
+            expiration_delay=1
+        )
 
-        self.create_artifact_relation(artifact_not_expired, artifact_expired)
+        self.playground.create_artifact_relation(
+            artifact_not_expired, artifact_expired
+        )
 
         self.assertEqual(
             self.delete_expired._mark_to_keep(),
@@ -355,48 +363,54 @@ class DeleteExpiredArtifactsTests(TestCase):
     @context.disable_permission_checks()
     def test_mark_to_keep_no_artifact_all_expired_two(self) -> None:
         """Two artifacts (related), both expired."""
-        artifact_expired_1, _ = self.create_artifact(expiration_delay=1)
+        artifact_expired_1, _ = self.playground.create_artifact(
+            expiration_delay=1
+        )
         artifact_expired_1.created_at = timezone.now() - timedelta(days=2)
         artifact_expired_1.save()
-        artifact_expired_2, _ = self.create_artifact(expiration_delay=1)
+        artifact_expired_2, _ = self.playground.create_artifact(
+            expiration_delay=1
+        )
         artifact_expired_2.created_at = timezone.now() - timedelta(days=2)
         artifact_expired_2.save()
 
-        self.create_artifact_relation(artifact_expired_1, artifact_expired_2)
+        self.playground.create_artifact_relation(
+            artifact_expired_1, artifact_expired_2
+        )
 
         self.assertEqual(self.delete_expired._mark_to_keep(), set())
 
     @context.disable_permission_checks()
     def test_mark_to_keep_no_artifact_all_expired_three(self) -> None:
         """All artifacts are expired."""
-        artifact_1, _ = self.create_artifact(expiration_delay=1)
+        artifact_1, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
         artifact_1.save()
-        artifact_2, _ = self.create_artifact(expiration_delay=1)
+        artifact_2, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_2.created_at = timezone.now() - timedelta(days=2)
         artifact_2.save()
-        artifact_3, _ = self.create_artifact(expiration_delay=1)
+        artifact_3, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_3.created_at = timezone.now() - timedelta(days=2)
         artifact_3.save()
 
-        self.create_artifact_relation(artifact_2, artifact_1)
-        self.create_artifact_relation(artifact_3, artifact_2)
+        self.playground.create_artifact_relation(artifact_2, artifact_1)
+        self.playground.create_artifact_relation(artifact_3, artifact_2)
 
         self.assertEqual(self.delete_expired._mark_to_keep(), set())
 
     @context.disable_permission_checks()
     def test_mark_to_keep_two_artifacts_non_expired_and_targeted(self) -> None:
         """Three artifacts, keep two (non-expired and target of non-expired)."""
-        artifact_1, _ = self.create_artifact(expiration_delay=1)
+        artifact_1, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
         artifact_1.save()
-        artifact_2, _ = self.create_artifact(expiration_delay=0)
-        artifact_3, _ = self.create_artifact(expiration_delay=1)
+        artifact_2, _ = self.playground.create_artifact(expiration_delay=0)
+        artifact_3, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_3.created_at = timezone.now() - timedelta(days=2)
         artifact_3.save()
 
-        self.create_artifact_relation(artifact_2, artifact_1)
-        self.create_artifact_relation(artifact_3, artifact_2)
+        self.playground.create_artifact_relation(artifact_2, artifact_1)
+        self.playground.create_artifact_relation(artifact_3, artifact_2)
 
         self.assertEqual(
             self.delete_expired._mark_to_keep(),
@@ -406,27 +420,27 @@ class DeleteExpiredArtifactsTests(TestCase):
     @context.disable_permission_checks()
     def test_mark_to_keep_isolated(self) -> None:
         """Two expired and one non-expired isolated."""
-        artifact_1, _ = self.create_artifact(expiration_delay=1)
+        artifact_1, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
         artifact_1.save()
-        artifact_2, _ = self.create_artifact(expiration_delay=1)
+        artifact_2, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_2.created_at = timezone.now() - timedelta(days=2)
         artifact_2.save()
-        artifact_3, _ = self.create_artifact(expiration_delay=0)
+        artifact_3, _ = self.playground.create_artifact(expiration_delay=0)
 
-        self.create_artifact_relation(artifact_2, artifact_1)
-        self.create_artifact_relation(artifact_1, artifact_2)
+        self.playground.create_artifact_relation(artifact_2, artifact_1)
+        self.playground.create_artifact_relation(artifact_1, artifact_2)
 
         self.assertEqual(self.delete_expired._mark_to_keep(), {artifact_3.id})
 
     @context.disable_permission_checks()
     def test_mark_to_keep_artifacts_circular_dependency(self) -> None:
         """Artifacts are not expired (keep), have a circular dependency."""
-        artifact_1, _ = self.create_artifact(expiration_delay=0)
-        artifact_2, _ = self.create_artifact(expiration_delay=0)
+        artifact_1, _ = self.playground.create_artifact(expiration_delay=0)
+        artifact_2, _ = self.playground.create_artifact(expiration_delay=0)
 
-        self.create_artifact_relation(artifact_2, artifact_1)
-        self.create_artifact_relation(artifact_1, artifact_2)
+        self.playground.create_artifact_relation(artifact_2, artifact_1)
+        self.playground.create_artifact_relation(artifact_1, artifact_2)
 
         self.assertEqual(
             self.delete_expired._mark_to_keep(),
@@ -436,22 +450,22 @@ class DeleteExpiredArtifactsTests(TestCase):
     @context.disable_permission_checks()
     def test_sweep_delete_artifacts(self) -> None:
         """Sweep() delete the artifacts and print progress."""
-        artifact_1, _ = self.create_artifact(expiration_delay=1)
+        artifact_1, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
         artifact_1.save()
-        artifact_2, _ = self.create_artifact(expiration_delay=1)
+        artifact_2, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_2.created_at = timezone.now() - timedelta(days=2)
         artifact_2.save()
 
-        self.create_artifact_relation(artifact_2, artifact_1)
-        self.create_artifact_relation(artifact_1, artifact_2)
+        self.playground.create_artifact_relation(artifact_2, artifact_1)
+        self.playground.create_artifact_relation(artifact_1, artifact_2)
 
-        artifact_3, _ = self.create_artifact(expiration_delay=1)
+        artifact_3, _ = self.playground.create_artifact(expiration_delay=1)
         artifact_3.created_at = timezone.now() - timedelta(days=2)
         artifact_3.save()
-        artifact_4, _ = self.create_artifact(expiration_delay=0)
+        artifact_4, _ = self.playground.create_artifact(expiration_delay=0)
 
-        self.create_artifact_relation(artifact_4, artifact_3)
+        self.playground.create_artifact_relation(artifact_4, artifact_3)
 
         artifacts_to_keep = self.delete_expired._mark_to_keep()
 
@@ -469,7 +483,7 @@ class DeleteExpiredArtifactsTests(TestCase):
     def test_sweep_delete_artifacts_no_retains(self) -> None:
         """Sweep() delete the artifacts from the collections."""
         with context.disable_permission_checks():
-            artifact, _ = self.create_artifact(
+            artifact, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 data={
                     "codename": "bookworm",
@@ -512,7 +526,7 @@ class DeleteExpiredArtifactsTests(TestCase):
     def test_delete_artifact_all_related_files(self) -> None:
         """_delete_artifact() deletes all the artifact's related models."""
         path_in_artifact = "README"
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             paths=[path_in_artifact], create_files=True
         )
 
@@ -559,10 +573,10 @@ class DeleteExpiredArtifactsTests(TestCase):
         )
 
         # Create the artifacts
-        artifact_to_delete, _ = self.create_artifact(
+        artifact_to_delete, _ = self.playground.create_artifact(
             paths=[file_to_keep_name, file_to_delete_name], create_files=True
         )
-        artifact_to_keep, _ = self.create_artifact()
+        artifact_to_keep, _ = self.playground.create_artifact()
 
         # Get both files
         file_to_keep = FileInArtifact.objects.get(
@@ -607,7 +621,7 @@ class DeleteExpiredArtifactsTests(TestCase):
     @context.disable_permission_checks()
     def test_delete_artifact_dry_run(self) -> None:
         """_delete_artifact() does nothing: running in dry-run."""
-        artifact, _ = self.create_artifact(expiration_delay=1)
+        artifact, _ = self.playground.create_artifact(expiration_delay=1)
         artifact.created_at = timezone.now() - timedelta(days=2)
         artifact.save()
 
@@ -624,7 +638,7 @@ class DeleteExpiredArtifactsTests(TestCase):
 
     def test_delete_files_from_stores(self) -> None:
         """_delete_files_from_stores delete the file."""
-        fileobj = self.create_file()
+        fileobj = self.playground.create_file()
         FileInStore.objects.create(store=default_file_store(), file=fileobj)
 
         self.delete_expired._delete_files_from_stores({fileobj})
@@ -633,7 +647,7 @@ class DeleteExpiredArtifactsTests(TestCase):
 
     def test_delete_files_from_multiple_stores(self) -> None:
         """_delete_files_from_stores delete files from more than one store."""
-        fileobj = self.create_file()
+        fileobj = self.playground.create_file()
 
         # Add the file in the default_file_store()
         FileInStore.objects.create(store=default_file_store(), file=fileobj)
@@ -659,10 +673,10 @@ class DeleteExpiredArtifactsTests(TestCase):
 
         The file exist in another Artifact: cannot be deleted.
         """
-        fileobj = self.create_file()
+        fileobj = self.playground.create_file()
         files_to_delete = {fileobj}
 
-        artifact, _ = self.create_artifact()
+        artifact, _ = self.playground.create_artifact()
         FileInArtifact.objects.create(artifact=artifact, file=fileobj)
 
         self.delete_expired._delete_files_from_stores(files_to_delete)
@@ -1141,6 +1155,37 @@ class DeleteExpiredWorkRequestsTests(TestCase):
         )
         self.assert_dry_run_does_not_delete()
 
+    def test_delete_expired_with_internal_collection(self) -> None:
+        """Delete a work request referenced by an internal collection item."""
+        collection = self.playground.create_collection(
+            "internal", CollectionCategory.WORKFLOW_INTERNAL
+        )
+
+        work_request = self.playground.create_work_request(
+            created_at=timezone.now() - timedelta(days=365),
+            expiration_delay=timedelta(days=1),
+            internal_collection=collection,
+        )
+
+        collection.manager.add_bare_data(
+            name="Item",
+            user=self.playground.get_default_user(),
+            category=BareDataCategory.TEST,
+            workflow=work_request,
+        )
+
+        # Work request is now expired but referenced by an item in an internal
+        # collection: it should be deleted
+        with self.mock_perform_deletions():
+            self.delete_expired.run()
+        self.assertEqual(
+            self.delete_expired.work_requests,
+            {work_request},
+        )
+
+        # Check that it is actually deleted
+        self.delete_expired.run()
+
 
 def _format_deleted_work_requests(
     work_requests: list[WorkRequest], collections: list[Collection]
