@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -75,7 +75,7 @@ class MonthlyCleanupCommandTests(TestCase):
         Also test that is not finding a file part of an artifact.
         """
         with context.disable_permission_checks():
-            artifact, _ = self.create_artifact(
+            artifact, _ = self.playground.create_artifact(
                 paths=["README"], create_files=True
             )
 
@@ -121,7 +121,7 @@ class MonthlyCleanupCommandTests(TestCase):
         """
         with context.disable_permission_checks():
             path_in_artifact = "README"
-            artifact, _ = self.create_artifact(
+            artifact, _ = self.playground.create_artifact(
                 paths=[path_in_artifact], create_files=True
             )
 
@@ -160,7 +160,7 @@ class MonthlyCleanupCommandTests(TestCase):
                 directory=self.debusine_store_directory
             )
             self._set_modified_time_two_days_ago(empty_directory)
-            self.create_artifact(paths=["README"], create_files=True)
+            self.playground.create_artifact(paths=["README"], create_files=True)
 
         stdout, stderr, exitcode = call_command(
             "monthly_cleanup", "--dry-run", verbosity=2
@@ -190,7 +190,7 @@ class MonthlyCleanupCommandTests(TestCase):
             )
             self._set_modified_time_two_days_ago(empty_dir)
             path_in_artifact = "README"
-            artifact, _ = self.create_artifact(
+            artifact, _ = self.playground.create_artifact(
                 paths=[path_in_artifact],
                 create_files=True,
                 skip_add_files_in_store=False,
@@ -242,7 +242,7 @@ class MonthlyCleanupCommandTests(TestCase):
     def test_delete_unfinished_uploads_no_delete_dry_run(self) -> None:
         """Unfinished, old FileUpload is not deleted: dry run."""
         with context.disable_permission_checks():
-            artifact, _ = self.create_artifact(
+            artifact, _ = self.playground.create_artifact(
                 paths=["README"], create_files=True
             )
             file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
@@ -282,7 +282,7 @@ class MonthlyCleanupCommandTests(TestCase):
     def test_delete_unfinished_upload_deleted(self) -> None:
         """Unfinished and old FileUpload and its file is deleted."""
         with context.disable_permission_checks():
-            artifact, _ = self.create_artifact(
+            artifact, _ = self.playground.create_artifact(
                 paths=["README"], create_files=True
             )
             file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
@@ -312,7 +312,7 @@ class MonthlyCleanupCommandTests(TestCase):
     def test_delete_unfinished_upload_not_deleted_recent_activity(self) -> None:
         """Unfinished FileUpload is not deleted: recent activity."""
         with context.disable_permission_checks():
-            artifact, _ = self.create_artifact(
+            artifact, _ = self.playground.create_artifact(
                 paths=["README"], create_files=True
             )
             file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
@@ -386,7 +386,7 @@ class MonthlyCleanupCommandTests(TestCase):
         """File is referenced by the DB but not in disk in the store."""
         with context.disable_permission_checks():
             path_in_artifact = "README"
-            artifact, _ = self.create_artifact(
+            artifact, _ = self.playground.create_artifact(
                 paths=[path_in_artifact], create_files=True
             )
 
@@ -402,7 +402,7 @@ class MonthlyCleanupCommandTests(TestCase):
     def test_file_in_upload_db_missing_from_disk(self) -> None:
         """File is referenced by the DB but not in disk for uploads."""
         with context.disable_permission_checks():
-            artifact, _ = self.create_artifact(
+            artifact, _ = self.playground.create_artifact(
                 paths=["README"], create_files=True
             )
             file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
