@@ -1,4 +1,4 @@
-# Copyright 2021-2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -47,7 +47,7 @@ from debusine.server.exceptions import (
 )
 from debusine.server.workflows import Workflow
 from debusine.server.workflows.models import WorkRequestManualUnblockAction
-from debusine.tasks.models import EventReactions, WorkerType
+from debusine.tasks.models import EventReactions, OutputData, WorkerType
 
 if TYPE_CHECKING:
     SerializerMixinBase: TypeAlias = serializers.Serializer[Any]
@@ -313,10 +313,19 @@ class WorkRequestCompletedSerializer(serializers.ModelSerializer[WorkRequest]):
             if choice != WorkRequest.Results.NONE
         ]
     )
+    output_data = serializers.JSONField(required=False)
+
+    def validate_output_data(self, value: Any) -> Any:
+        """Validate output_data."""
+        try:
+            OutputData.parse_obj(value)
+        except pydantic.ValidationError as e:
+            raise serializers.ValidationError(f"Invalid output_data: {e}")
+        return value
 
     class Meta:
         model = WorkRequest
-        fields = ['result']
+        fields = ["result", "output_data"]
 
 
 class OnWorkRequestCompleted(serializers.Serializer[Never]):
