@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -172,7 +172,7 @@ class BaseServerTaskTests(TestCase):
 
         The relation is from the debug logs artifact to the source_artifact_id.
         """
-        source_artifact = self.create_artifact()[0]
+        source_artifact = self.playground.create_artifact()[0]
         expected_files = self.setup_upload_work_request_debug_logs(
             [source_artifact]
         )
