@@ -1,4 +1,4 @@
-# Copyright 2021-2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -176,7 +176,7 @@ class TaskDatabaseTests(TestCase):
         collection = self.playground.create_collection(
             "debian", CollectionCategory.ENVIRONMENTS, workspace=self.workspace
         )
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.SYSTEM_TARBALL,
             data={"codename": "bookworm", "architecture": "amd64"},
         )
@@ -213,7 +213,7 @@ class TaskDatabaseTests(TestCase):
         )
         artifacts: list[Artifact] = []
         for codename in "bookworm", "trixie":
-            artifact, _ = self.create_artifact(
+            artifact, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 data={"codename": codename, "architecture": "amd64"},
             )
