@@ -1,4 +1,4 @@
-# Copyright 2021-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -9,7 +9,7 @@
 
 """Views for the server application: base infrastructure."""
 import logging
-from collections.abc import Callable, Iterable, Sequence
+from collections.abc import Callable, Iterable
 from typing import (
     Any,
     Protocol,
@@ -19,10 +19,8 @@ from typing import (
     runtime_checkable,
 )
 
-from django.core.exceptions import PermissionDenied
 from django.db.models import Model, QuerySet
-from django.http import HttpRequest, HttpResponse, JsonResponse
-from django.views import View
+from django.http import JsonResponse
 from rest_framework import status
 from rest_framework.exceptions import PermissionDenied as DRFPermissionDenied
 from rest_framework.filters import BaseFilterBackend
@@ -181,16 +179,6 @@ class IsTokenUserAuthenticatedDjangoModelPermissions(DjangoModelPermissions):
         return True
 
 
-class IsUserAuthenticated(BasePermission):
-    """Allow access to requests with an authenticated user."""
-
-    def has_permission(
-        self, request: Request, view: APIView  # noqa: U100
-    ) -> bool:
-        """Return True if the request has an authenticated user."""
-        return request.user.is_authenticated
-
-
 class IsWorkerAuthenticated(IsTokenAuthenticated):
     """Allow access to requests with a token assigned to a worker."""
 
@@ -223,55 +211,6 @@ class IsGet(BasePermission):
         return request.method == "GET"
 
 
-class ValidatePermissionsMixin:
-    """
-    Mixin implementing _check_permissions() for the Views.
-
-    To use it:
-        class YourView(ValidatePermissionsMixin, View):
-            permission_denied_message = "Message explaining the problem"
-            permission_classes = [IsUserAuthenticated | IsTokenAuthenticated]
-
-    It overrides dispatch() and raise PermissionDenied if the request cannot
-    proceed.
-    """
-
-    permission_classes: Sequence["_PermissionClass"]
-    permission_denied_message: str | None = None
-
-    def _check_permissions(self, request: HttpRequest) -> None:
-        """
-        Raise PermissionDenied() if one permission does not allow access.
-
-        Same approach as Django REST ApiView methods: if no permissions:
-        it is allowed. If any class denies access raises PermissionDenied().
-
-        See rest_framework/views.py ApiView.check_permissions.
-        """
-        for permission_class in self.permission_classes:
-            permission = permission_class()
-            # TODO: This arg-type ignore is cheating slightly, because we
-            # also use this mixin in a few places under debusine.web where
-            # it's mixed in with View rather than APIView.  However, it
-            # isn't obvious how to fix that, and it doesn't cause a
-            # practical problem right now.
-            if not permission.has_permission(
-                request, self  # type: ignore[arg-type]
-            ):
-                raise PermissionDenied(self.permission_denied_message)
-
-    def dispatch(
-        self, request: HttpRequest, *args: Any, **kwargs: Any
-    ) -> HttpResponse:
-        """Check permissions for any request method (GET, POST, PUT, etc.)."""
-        self._check_permissions(request)
-
-        assert isinstance(self, View)
-        # The mixin confuses mypy, but it works as long as it's mixed into a
-        # View.
-        return super().dispatch(request, *args, **kwargs)  # type: ignore
-
-
 class BaseAPIView(APIView):
     """Common base for API views."""
 
