@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -73,10 +73,10 @@ class AutopkgtestWorkflow(
         """Create an autopkgtest work request for a single architecture."""
         assert self.work_request is not None
 
-        filtered_binary_artifacts = self.filter_artifact_lookup(
+        filtered_binary_artifacts = self.filter_artifact_lookup_by_arch(
             self.data.binary_artifacts, (architecture, "all")
         )
-        filtered_context_artifacts = self.filter_artifact_lookup(
+        filtered_context_artifacts = self.filter_artifact_lookup_by_arch(
             self.data.context_artifacts, (architecture, "all")
         )
         task_data = AutopkgtestData(
@@ -92,6 +92,7 @@ class AutopkgtestWorkflow(
             backend=self.data.backend,
             include_tests=self.data.include_tests,
             exclude_tests=self.data.exclude_tests,
+            extra_repositories=self.data.extra_repositories,
             debug_level=self.data.debug_level,
             extra_environment=self.data.extra_environment,
             needs_internet=self.data.needs_internet,
