@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -13,6 +13,7 @@ See :ref:`explanation-workflows` for a high level explanation of concepts used
 here.
 """
 import logging
+import typing
 from abc import ABCMeta, abstractmethod
 from collections.abc import Iterable
 from typing import Any, TypeVar, TypedDict, cast
@@ -32,6 +33,8 @@ from debusine.client.models import (
     model_to_json_serializable_dict,
 )
 from debusine.db.models import (
+    Artifact,
+    ArtifactRelation,
     Collection,
     CollectionItem,
     TaskDatabase,
@@ -380,11 +383,9 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
     @staticmethod
     def lookup_result_architecture(result: LookupResult) -> str:
         """Get architecture from result of looking up an artifact."""
-        if result.collection_item is not None:
-            architecture = result.collection_item.data.get("architecture")
-            if isinstance(architecture, str):
-                return architecture
-        elif result.artifact is not None:
+        architecture: str | None
+
+        if result.artifact is not None:
             artifact_data = result.artifact.create_data()
             match artifact_data:
                 case DebianBinaryPackages():
@@ -401,6 +402,11 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
             assert isinstance(architecture, str)
             return architecture
 
+        elif result.collection_item is not None:
+            architecture = result.collection_item.data.get("architecture")
+            if isinstance(architecture, str):
+                return architecture
+
         else:
             raise ValueError(
                 "Unexpected result: must have collection_item or artifact"
@@ -410,7 +416,7 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
             f"Cannot determine architecture for lookup result: {result}"
         )
 
-    def filter_artifact_lookup(
+    def filter_artifact_lookup_by_arch(
         self, lookup: LookupMultiple, architectures: Iterable[str]
     ) -> LookupMultiple:
         """Filter an artifact lookup by architecture."""
@@ -465,6 +471,91 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
         """Unused abstract method from BaseTask: populate() is used instead."""
         raise NotImplementedError()
 
+    @staticmethod
+    def ensure_artifact_categories(
+        artifact: Artifact, categories: typing.Collection[ArtifactCategory]
+    ) -> None:
+        """Ensure that artifact has a category in categories."""
+        if artifact.category not in categories:
+            raise TaskConfigError(
+                f'Category of "{artifact}" is not one of '
+                f"{[category.value for category in categories]}"
+            )
+
+    @staticmethod
+    def follow_artifact_relation(
+        artifact: Artifact,
+        relation_type: ArtifactRelation.Relations,
+        category: ArtifactCategory,
+    ) -> Artifact:
+        """Follow relations from artifact to find an artifact of category."""
+        try:
+            relation = artifact.relations.get(
+                type=relation_type, target__category=category
+            )
+        except ArtifactRelation.DoesNotExist:
+            raise TaskConfigError(
+                f"Unable to find an artifact of category {category.value} with "
+                f'a relationship of type {relation_type} from "{artifact}"'
+            )
+        except ArtifactRelation.MultipleObjectsReturned:
+            raise TaskConfigError(
+                f"Multiple artifacts of category {category.value} with "
+                f'a relationship of type {relation_type} from "{artifact}" '
+                f"found"
+            )
+        return relation.target
+
+    @classmethod
+    def locate_debian_source_package(cls, artifact: Artifact) -> Artifact:
+        """
+        Accept a debian:upload or debian:source-package in a workflow.
+
+        Resolve to the debian:source-package.
+        """
+        cls.ensure_artifact_categories(
+            artifact,
+            [ArtifactCategory.SOURCE_PACKAGE, ArtifactCategory.UPLOAD],
+        )
+        match artifact.category:
+            case ArtifactCategory.SOURCE_PACKAGE:
+                return artifact
+            case ArtifactCategory.UPLOAD:
+                return cls.follow_artifact_relation(
+                    artifact,
+                    ArtifactRelation.Relations.EXTENDS,
+                    ArtifactCategory.SOURCE_PACKAGE,
+                )
+            case _ as unreachable:  # pragma: no cover
+                raise AssertionError(
+                    f"Unexpected artifact category: {unreachable}"
+                )
+
+    def locate_debian_source_package_lookup(
+        self, lookup: LookupSingle
+    ) -> LookupSingle:
+        """
+        Return a lookup to a debian:source-packge.
+
+        If the specified lookup returns a debian:source-package, return it.
+        If it returns a debian:upload, find the related debian:source-package
+        and return a lookup to it.
+        """
+        artifact = lookup_single(
+            lookup,
+            self.workspace,
+            user=self.work_request.created_by,
+            workflow_root=self.work_request.get_workflow_root(),
+            expect_type=LookupChildType.ARTIFACT,
+        ).artifact
+        if artifact.category == ArtifactCategory.UPLOAD:
+            source_package = self.locate_debian_source_package(artifact)
+            return f"{source_package.id}@artifacts"
+        self.ensure_artifact_categories(
+            artifact, [ArtifactCategory.SOURCE_PACKAGE]
+        )
+        return lookup
+
 
 def orchestrate_workflow(work_request: WorkRequest) -> None:
     """
