@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -44,7 +44,7 @@ class LintianWorkflow(Workflow[LintianWorkflowData, BaseDynamicTaskData]):
         environment = f"{self.data.vendor}/match:codename={self.data.codename}"
 
         for arch in architectures:
-            filtered_binary_artifacts = self.filter_artifact_lookup(
+            filtered_binary_artifacts = self.filter_artifact_lookup_by_arch(
                 self.data.binary_artifacts, (arch, "all")
             )
 
