@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -56,17 +56,17 @@ class MakeSignedSourceWorkflow(
         architectures = architectures.intersection(self.data.architectures)
 
         for arch in architectures:
-            signing_templates_lookup = self.filter_artifact_lookup(
+            signing_templates_lookup = self.filter_artifact_lookup_by_arch(
                 self.data.signing_template_artifacts, (arch,)
             )
 
-            binary_artifacts_lookups = self.filter_artifact_lookup(
+            binary_artifacts_lookups = self.filter_artifact_lookup_by_arch(
                 self.data.binary_artifacts,
                 (arch, "all"),
             )
 
             for signing_template_lookup in signing_templates_lookup:
-                # filter_artifact_lookup always returns a parsed list of
+                # filter_artifact_lookup_by_arch always returns a parsed list of
                 # single lookups.
                 assert not isinstance(signing_template_lookup, LookupDict)
 
