@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -214,12 +214,7 @@ class ReverseDependenciesAutopkgtestWorkflow(
             workflow_root=self.work_request.get_workflow_root(),
             expect_type=LookupChildType.ARTIFACT,
         ).artifact
-        if source_artifact.category != ArtifactCategory.SOURCE_PACKAGE:
-            raise ValueError(
-                f"Expected source_artifact to be of category "
-                f"{ArtifactCategory.SOURCE_PACKAGE}; got "
-                f"{source_artifact.category}"
-            )
+        source_artifact = self.locate_debian_source_package(source_artifact)
         source_artifact_data = source_artifact.create_data()
         assert isinstance(source_artifact_data, DebianSourcePackage)
 
@@ -406,6 +401,7 @@ class ReverseDependenciesAutopkgtestWorkflow(
                 backend=self.data.backend,
                 architectures=self.data.architectures,
                 arch_all_host_architecture=self.data.arch_all_host_architecture,
+                extra_repositories=self.data.extra_repositories,
                 debug_level=self.data.debug_level,
             ),
             workflow_data=WorkRequestWorkflowData(
