@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -61,12 +61,7 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
             user=self.work_request.created_by,
             expect_type=LookupChildType.ARTIFACT,
         ).artifact
-        if artifact.category != ArtifactCategory.SOURCE_PACKAGE:
-            raise ValueError(
-                f"source artifact is type {artifact.category!r}"
-                f" instead of {ArtifactCategory.SOURCE_PACKAGE}"
-            )
-        return artifact
+        return self.locate_debian_source_package(artifact)
 
     @cached_property
     def source_package_data(self) -> DebianSourcePackage:
@@ -269,8 +264,12 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
         """Create a single sbuild WorkRequest."""
         assert self.work_request is not None
         host_architecture = self._get_host_architecture(architecture)
+        input_ = self.data.input.copy()
+        input_.source_artifact = self.locate_debian_source_package_lookup(
+            self.data.input.source_artifact
+        )
         task_data = SbuildData(
-            input=self.data.input,
+            input=input_,
             host_architecture=host_architecture,
             environment=environment,
             backend=self.data.backend,
@@ -312,27 +311,16 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
         except KeyError:
             build_logs_collection_id = None
 
-        source_artifact = lookup_single(
-            self.data.input.source_artifact,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.get_workflow_root(),
-            expect_type=LookupChildType.ARTIFACT,
-        ).artifact
-
         if build_logs_collection_id is not None:
             vendor, codename = self.data.target_distribution.split(":", 1)
-            source_package_data = SourcePackage.create_data(
-                source_artifact.data
-            )
 
             variables = {
                 "work_request_id": wr.id,
                 "vendor": vendor,
                 "codename": codename,
                 "architecture": host_architecture,
-                "srcpkg_name": source_package_data.name,
-                "srcpkg_version": source_package_data.version,
+                "srcpkg_name": self.source_package_data.name,
+                "srcpkg_version": self.source_package_data.version,
             }
             event_reactions = wr.event_reactions
             event_reactions.on_creation.append(
