@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -12,13 +12,16 @@
 import logging
 from unittest import mock
 
-from debusine.db.models import WorkRequest
+from debusine.artifacts.models import ArtifactCategory
+from debusine.db.models import ArtifactRelation, WorkRequest
 from debusine.server.workflows.base import (
     Workflow,
     WorkflowRunError,
     orchestrate_workflow,
 )
 from debusine.server.workflows.models import BaseWorkflowData
+from debusine.server.workflows.noop import NoopWorkflow
+from debusine.tasks import TaskConfigError
 from debusine.tasks.models import BaseDynamicTaskData, TaskTypes
 from debusine.tasks.server import TaskDatabaseInterface
 from debusine.tasks.tests.helper_mixin import TestTaskMixin
@@ -26,6 +29,151 @@ from debusine.test.django import TestCase
 from debusine.test.utils import preserve_task_registry
 
 
+class WorkflowClassTests(TestCase):
+    """Test the Workflow base class."""
+
+    def test_ensure_artifact_categories_matches(self) -> None:
+        """Test with a matching artifact."""
+        artifact, _ = self.playground.create_artifact(
+            category=ArtifactCategory.TEST
+        )
+        Workflow.ensure_artifact_categories(
+            artifact, [ArtifactCategory.TEST, ArtifactCategory.UPLOAD]
+        )
+
+    def test_ensure_artifact_categories_mismatches(self) -> None:
+        """Test with a mis-matching artifact."""
+        artifact, _ = self.playground.create_artifact(
+            category=ArtifactCategory.TEST
+        )
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r'Category of ".* debusine:test .*" is not one of '
+            r"\['debian:system-tarball', 'debian:system-image'\]",
+        ):
+            Workflow.ensure_artifact_categories(
+                artifact,
+                [
+                    ArtifactCategory.SYSTEM_TARBALL,
+                    ArtifactCategory.SYSTEM_IMAGE,
+                ],
+            )
+
+    def test_follow_artifact_relation_upload(self) -> None:
+        """Test finding a source package from an upload."""
+        upload_artifacts = self.playground.create_upload_artifacts()
+        test_artifact, _ = self.playground.create_artifact()
+
+        # Create some extra unrelated relations
+        self.playground.create_artifact_relation(
+            artifact=upload_artifacts.upload,
+            target=upload_artifacts.binaries[0],
+            relation_type=ArtifactRelation.Relations.BUILT_USING,
+        )
+        self.playground.create_artifact_relation(
+            artifact=upload_artifacts.upload,
+            target=test_artifact,
+            relation_type=ArtifactRelation.Relations.EXTENDS,
+        )
+
+        artifact = Workflow.follow_artifact_relation(
+            upload_artifacts.upload,
+            ArtifactRelation.Relations.EXTENDS,
+            ArtifactCategory.SOURCE_PACKAGE,
+        )
+        self.assertEqual(artifact, upload_artifacts.source)
+
+    def test_follow_artifact_relation_not_found(self) -> None:
+        """Test with a lone debian:upload (no relations)."""
+        upload_artifacts = self.playground.create_upload_artifacts(source=False)
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r"Unable to find an artifact of category debian:source-package "
+            r'with a relationship of type extends from ".* debian:upload .*"',
+        ):
+            Workflow.follow_artifact_relation(
+                upload_artifacts.upload,
+                ArtifactRelation.Relations.EXTENDS,
+                ArtifactCategory.SOURCE_PACKAGE,
+            )
+
+    def test_follow_artifact_relation_multiple_found(self) -> None:
+        """Test with a debian:upload extending multiple srcpkgs."""
+        upload_artifacts = self.playground.create_upload_artifacts(source=True)
+        source2 = self.playground.create_source_artifact()
+        self.playground.create_artifact_relation(
+            artifact=upload_artifacts.upload,
+            target=source2,
+            relation_type=ArtifactRelation.Relations.EXTENDS,
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r"Multiple artifacts of category debian:source-package with "
+            r'a relationship of type extends from ".* debian:upload .*" found',
+        ):
+            Workflow.follow_artifact_relation(
+                upload_artifacts.upload,
+                ArtifactRelation.Relations.EXTENDS,
+                ArtifactCategory.SOURCE_PACKAGE,
+            )
+
+    def test_locate_debian_source_package_with_srcpkg(self) -> None:
+        """Test with a debian:source-package."""
+        source_artifact = self.playground.create_source_artifact()
+        result = Workflow.locate_debian_source_package(source_artifact)
+        self.assertEqual(result, source_artifact)
+
+    def test_locate_debian_source_package_with_upload(self) -> None:
+        """Test with a debian:source-package."""
+        upload_artifacts = self.playground.create_upload_artifacts()
+        result = Workflow.locate_debian_source_package(upload_artifacts.upload)
+        self.assertEqual(result, upload_artifacts.source)
+
+    def test_locate_debian_source_package_with_junk(self) -> None:
+        """Test with a debian:source-package."""
+        artifact, _ = self.playground.create_artifact(
+            category=ArtifactCategory.TEST
+        )
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r'Category of ".* debusine:test .*" is not one of '
+            r"\['debian:source-package', 'debian:upload'\]",
+        ):
+            Workflow.locate_debian_source_package(artifact)
+
+
+class WorkflowInstanceTests(TestCase):
+    """Test instances of the Workflow class."""
+
+    def setUp(self) -> None:
+        """Create a NoopWorkflow for testing."""
+        super().setUp()
+        w = self.playground.create_workflow(task_name="noop")
+        self.workflow = NoopWorkflow(w)
+
+    def test_locate_debian_source_package_lookup_srcpkg(self) -> None:
+        """Test locate_debian_source_package_lookup() with a source package."""
+        source_artifact = self.playground.create_source_artifact()
+        self.assertEqual(
+            self.workflow.locate_debian_source_package_lookup(
+                source_artifact.id
+            ),
+            source_artifact.id,
+        )
+
+    def test_locate_debian_source_package_lookup_upload(self) -> None:
+        """Test locate_debian_source_package_lookup() with a debian:upload."""
+        artifacts = self.playground.create_upload_artifacts()
+        self.assertEqual(
+            self.workflow.locate_debian_source_package_lookup(
+                artifacts.upload.id
+            ),
+            f"{artifacts.source.id}@artifacts",
+        )
+
+
 class OrchestrateWorkflowTests(TestCase):
     """Test orchestrate_workflow()."""
 
