@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -27,6 +27,7 @@ from debusine.server.workflows.models import (
     QAWorkflowData,
     SbuildWorkflowData,
 )
+from debusine.tasks import TaskConfigError
 from debusine.tasks.models import (
     BackendType,
     BaseDynamicTaskData,
@@ -171,8 +172,10 @@ class QAWorkflowTests(TestCase):
         )
 
     def test_compute_dynamic_data_source_artifact_wrong_category(self) -> None:
-        """`source_artifact` must be a `debian:source-package`."""
-        artifact, _ = self.create_artifact(category=ArtifactCategory.TEST)
+        """`source_artifact` must be a source-package or upload."""
+        artifact, _ = self.playground.create_artifact(
+            category=ArtifactCategory.TEST
+        )
         wr = self.playground.create_workflow(
             task_name="qa",
             task_data=QAWorkflowData(
@@ -187,12 +190,29 @@ class QAWorkflowTests(TestCase):
         workflow = QAWorkflow(wr)
 
         with self.assertRaisesRegex(
-            ValueError,
-            f"source artifact is type '{ArtifactCategory.TEST}' instead of "
-            f"{ArtifactCategory.SOURCE_PACKAGE}",
+            TaskConfigError,
+            r'Category of ".* debusine:test .*" is not one of '
+            r"\['debian:source-package', 'debian:upload'\]",
         ):
             workflow.compute_dynamic_data(TaskDatabase(wr))
 
+    def test_source_package_upload(self) -> None:
+        """`source_artifact` can be an upload."""
+        upload_artifacts = self.playground.create_upload_artifacts()
+        wr = self.playground.create_workflow(
+            task_name="qa",
+            task_data=QAWorkflowData(
+                source_artifact=upload_artifacts.upload.id,
+                binary_artifacts=LookupMultiple.parse_obj(
+                    ["internal@collections/name:build-amd64"]
+                ),
+                vendor="debian",
+                codename="sid",
+            ),
+        )
+        workflow = QAWorkflow(wr)
+        self.assertEqual(workflow.source_package, upload_artifacts.source)
+
     @preserve_task_registry()
     def test_populate(self) -> None:
         """Test populate."""
@@ -207,6 +227,13 @@ class QAWorkflowTests(TestCase):
                 "source_artifact": source_artifact.id,
                 "binary_artifacts": ["internal@collections/name:build-amd64"],
                 "architectures": ["amd64"],
+                "extra_repositories": [
+                    {
+                        "url": "http://example.com/",
+                        "suite": "bookworm",
+                        "components": ["main"],
+                    }
+                ],
             },
             architectures=architectures,
         )
@@ -230,6 +257,13 @@ class QAWorkflowTests(TestCase):
                 "codename": "bookworm",
                 "source_artifact": source_artifact.id,
                 "vendor": "debian",
+                "extra_repositories": [
+                    {
+                        "url": "http://example.com/",
+                        "suite": "bookworm",
+                        "components": ["main"],
+                    }
+                ],
             },
         )
         self.assertEqual(
@@ -315,6 +349,13 @@ class QAWorkflowTests(TestCase):
                 ],
                 "codename": "bookworm",
                 "vendor": "debian",
+                "extra_repositories": [
+                    {
+                        "url": "http://example.com/",
+                        "suite": "bookworm",
+                        "components": ["main"],
+                    }
+                ],
             },
         )
 
@@ -353,7 +394,7 @@ class QAWorkflowTests(TestCase):
                 workspace=self.playground.get_default_workspace(),
             )
             for arch in ["amd64", "i386"]:
-                artifact, _ = self.create_artifact(
+                artifact, _ = self.playground.create_artifact(
                     category=ArtifactCategory.SYSTEM_TARBALL,
                     data={"codename": "bookworm", "architecture": arch},
                 )
@@ -511,6 +552,7 @@ class QAWorkflowTests(TestCase):
                 "backend": BackendType.AUTO,
                 "architectures": ["amd64"],
                 "arch_all_host_architecture": "amd64",
+                "extra_repositories": None,
             },
         )
         self.assertEqual(
@@ -584,6 +626,7 @@ class QAWorkflowTests(TestCase):
                 "codename": "bookworm",
                 "source_artifact": source_artifact.id,
                 "vendor": "debian",
+                "extra_repositories": None,
             },
         )
 
@@ -632,6 +675,7 @@ class QAWorkflowTests(TestCase):
                 "codename": "bookworm",
                 "source_artifact": source_artifact.id,
                 "vendor": "debian",
+                "extra_repositories": None,
             },
         )
 
