@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -28,6 +28,7 @@ from debusine.server.collections.tests.utils import CollectionTestMixin
 from debusine.server.workflows import SbuildWorkflow, WorkflowValidationError
 from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import SbuildWorkflowData
+from debusine.tasks import TaskConfigError
 from debusine.tasks.models import (
     ActionTypes,
     BackendType,
@@ -152,7 +153,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             "architecture": "amd64",
             "variant": "buildd",
         }
-        tarball, _ = self.create_artifact(
+        tarball, _ = self.playground.create_artifact(
             category=ArtifactCategory.SYSTEM_TARBALL,
             data={**data, "with_dev": True},
         )
@@ -366,7 +367,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                 self.playground.create_debian_environments_collection()
             )
             tarballs = {
-                arch: self.create_artifact(
+                arch: self.playground.create_artifact(
                     category=ArtifactCategory.SYSTEM_TARBALL,
                     data={
                         "codename": "bookworm",
@@ -516,7 +517,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                 self.playground.create_debian_environments_collection()
             )
             tarballs = {
-                arch: self.create_artifact(
+                arch: self.playground.create_artifact(
                     category=ArtifactCategory.SYSTEM_TARBALL,
                     data={
                         "codename": "bookworm",
@@ -590,7 +591,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             environments = (
                 self.playground.create_debian_environments_collection()
             )
-            tarball, _ = self.create_artifact(
+            tarball, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 data={
                     "codename": "bookworm",
@@ -729,7 +730,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             environments = (
                 self.playground.create_debian_environments_collection()
             )
-            tarball, _ = self.create_artifact(
+            tarball, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 data={
                     "codename": "bookworm",
@@ -812,7 +813,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             environments = (
                 self.playground.create_debian_environments_collection()
             )
-            tarball, _ = self.create_artifact(
+            tarball, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 data={
                     "codename": "bookworm",
@@ -909,12 +910,14 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             orchestrator = SbuildWorkflow(workflow)
             return orchestrator
 
-        invalid_artifact, _ = self.create_artifact(
+        invalid_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST
         )
         orchestrator = create_orchestrator(invalid_artifact)
         with self.assertRaisesRegex(
-            ValueError, f"source artifact is type '{ArtifactCategory.TEST}'"
+            TaskConfigError,
+            r'Category of ".* debusine:test .*" is not one of '
+            r"\['debian:source-package', 'debian:upload'\]",
         ):
             orchestrator.source_package
 
@@ -974,6 +977,31 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
         ):
             orchestrator.validate_input()
 
+    def test_populate_upload(self) -> None:
+        """The workflow accepts debian:upload artifacts."""
+        upload_artifacts = self.playground.create_upload_artifacts()
+        self.playground.create_debian_environment(codename="bookworm")
+
+        wf = self.playground.create_workflow(
+            task_name="sbuild",
+            task_data=SbuildWorkflowData(
+                input=SbuildInput(
+                    source_artifact=upload_artifacts.upload.id,
+                ),
+                target_distribution="debian:bookworm",
+                architectures=["amd64"],
+            ),
+        )
+        SbuildWorkflow(wf).populate()
+
+        children = list(WorkRequest.objects.filter(parent=wf))
+        self.assertEqual(len(children), 1)
+        child = children[0]
+        self.assertEqual(
+            child.task_data["input"]["source_artifact"],
+            f"{upload_artifacts.source.id}@artifacts",
+        )
+
     def test_label(self) -> None:
         """Test get_label."""
         wr = WorkRequest(
