@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -273,7 +273,7 @@ class UpdateEnvironmentsWorkflowTests(CollectionTestMixin, TestCase):
             name="debian", category=CollectionCategory.ENVIRONMENTS
         )
         with context.disable_permission_checks():
-            tarball, _ = self.create_artifact(
+            tarball, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 data={"architecture": "amd64"},
                 work_request=child,
