@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -367,6 +367,29 @@ class WorkflowTests(TestCase):
 
         self.assertEqual(Workflow.lookup_result_architecture(result), "i386")
 
+    def test_lookup_result_binary_packages_artifact_item(self) -> None:
+        """lookup_result_architecture with a binary-packages artifact item."""
+        collection = self.playground.create_collection(
+            "test", CollectionCategory.TEST
+        )
+        artifact = self.playground.create_minimal_binary_packages_artifact(
+            "hello", "1.0-1", "1.0-1", "i386"
+        )
+        item = CollectionItem.objects.create_from_artifact(
+            artifact,
+            parent_collection=collection,
+            name="hello",
+            data={},
+            created_by_user=self.playground.get_default_user(),
+        )
+        result = LookupResult(
+            result_type=CollectionItem.Types.ARTIFACT,
+            collection_item=item,
+            artifact=artifact,
+        )
+
+        self.assertEqual(Workflow.lookup_result_architecture(result), "i386")
+
     def test_lookup_result_debian_binary_package(self) -> None:
         """lookup_result_architecture return arch from a binary package."""
         with context.disable_permission_checks():
@@ -389,6 +412,38 @@ class WorkflowTests(TestCase):
             "amd64",
         )
 
+    def test_lookup_result_debian_binary_package_item(self) -> None:
+        """lookup_result_architecture return arch from a binary package item."""
+        collection = self.playground.create_collection(
+            "test", CollectionCategory.TEST
+        )
+        artifact, _ = self.playground.create_artifact(
+            category=ArtifactCategory.BINARY_PACKAGE,
+            data={
+                "srcpkg_name": "hello",
+                "srcpkg_version": "1.0.0",
+                "deb_fields": {"Architecture": "amd64"},
+                "deb_control_files": [],
+            },
+        )
+        item = CollectionItem.objects.create_from_artifact(
+            artifact,
+            parent_collection=collection,
+            name="hello",
+            data={},
+            created_by_user=self.playground.get_default_user(),
+        )
+        result = LookupResult(
+            result_type=CollectionItem.Types.ARTIFACT,
+            collection_item=item,
+            artifact=artifact,
+        )
+
+        self.assertEqual(
+            Workflow.lookup_result_architecture(result),
+            "amd64",
+        )
+
     def test_lookup_result_debian_upload(self) -> None:
         """lookup_result_architecture return arch from an upload artifact."""
         with context.disable_permission_checks():
@@ -412,10 +467,43 @@ class WorkflowTests(TestCase):
             "amd64",
         )
 
+    def test_lookup_result_debian_upload_item(self) -> None:
+        """lookup_result_architecture return arch from an upload item."""
+        collection = self.playground.create_collection(
+            "test", CollectionCategory.TEST
+        )
+        artifact, _ = self.playground.create_artifact(
+            category=ArtifactCategory.UPLOAD,
+            data={
+                "type": "dpkg",
+                "changes_fields": {
+                    "Architecture": "amd64",
+                    "Files": [{"name": "test.deb"}],
+                },
+            },
+        )
+        item = CollectionItem.objects.create_from_artifact(
+            artifact,
+            parent_collection=collection,
+            name="hello",
+            data={},
+            created_by_user=self.playground.get_default_user(),
+        )
+        result = LookupResult(
+            result_type=CollectionItem.Types.ARTIFACT,
+            collection_item=item,
+            artifact=artifact,
+        )
+
+        self.assertEqual(
+            Workflow.lookup_result_architecture(result),
+            "amd64",
+        )
+
     def test_lookup_result_other_artifacts(self) -> None:
         """lookup_result_architecture with source package artifact."""
         with context.disable_permission_checks():
-            artifact, _ = self.create_artifact(
+            artifact, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SOURCE_PACKAGE,
                 data={
                     "name": "hello",
