@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -40,7 +40,7 @@ class GenerateKey(BaseSigningTask[GenerateKeyData, BaseDynamicTaskData]):
 
     def upload_artifacts(
         self, execute_directory: Path, *, execution_success: bool  # noqa: U100
-    ):
+    ) -> None:
         """Upload signing key artifact."""
         assert self.debusine is not None
         # run() always either returns True or raises an exception; in the
@@ -57,8 +57,6 @@ class GenerateKey(BaseSigningTask[GenerateKeyData, BaseDynamicTaskData]):
             work_request=self.work_request_id,
         )
 
-        return True
-
     def get_label(self) -> str:
         """Return the task label."""
         return f"generate {self.data.purpose} key"
