@@ -1,4 +1,4 @@
-# Copyright 2021-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -40,6 +40,7 @@ from typing import (
     ClassVar,
     Generic,
     IO,
+    Literal,
     TYPE_CHECKING,
     TextIO,
     TypeVar,
@@ -66,6 +67,8 @@ from debusine.tasks.models import (
     BaseDynamicTaskDataWithExecutor,
     BaseTaskData,
     BaseTaskDataWithExecutor,
+    BaseTaskDataWithExtraRepositories,
+    EventReaction,
     LookupSingle,
     TaskTypes,
     WorkerType,
@@ -584,6 +587,20 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         :return: None if no label could be computed from task data
         """
 
+    def get_event_reactions(
+        self,
+        event_name: Literal[  # noqa: U100
+            "on_creation", "on_unblock", "on_success", "on_failure"
+        ],
+    ) -> list[EventReaction]:
+        """
+        Return event reactions for this task.
+
+        This allows tasks to provide actions that are processed by the
+        server at various points in the lifecycle of the work request.
+        """
+        return []
+
 
 class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
     r"""
@@ -1384,3 +1401,87 @@ class RunCommandTask(
             os.killpg(process_group, signal)
         except ProcessLookupError:
             pass
+
+
+TDER = TypeVar("TDER", bound=BaseTaskDataWithExtraRepositories)
+
+
+class ExtraRepositoryMixin(
+    BaseExternalTask[TDER, DTD], Generic[TDER, DTD], metaclass=ABCMeta
+):
+    """Methods for configuring external APT repositories."""
+
+    data: TDER
+
+    # Deb822 style sources:
+    extra_repository_sources: list[Path]
+    # GPG keys (for apt < 2.3.10):
+    extra_repository_keys: list[Path]
+
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        """Initialize class variables."""
+        super().__init__(*args, **kwargs)
+        self.extra_repository_sources = []
+        self.extra_repository_keys = []
+
+    def supports_deb822_sources(self, codename: str) -> bool:
+        """Determine if codename supports deb822 sources."""
+        return codename not in ("jessie", "trusty")
+
+    def supports_inline_signed_by(self, codename: str) -> bool:
+        """Determine if codename supports inline signatures."""
+        # apt < 2.3.10 has no support for keys inline in Signed-By
+        return codename not in (
+            "jessie",
+            "stretch",
+            "buster",
+            "bullseye",
+            "trusty",
+            "xenial",
+            "bionic",
+            "focal",
+        )
+
+    def write_extra_repository_config(
+        self, codename: str, destination: Path
+    ) -> None:
+        """
+        Write extra_repositories config to files in destination.
+
+        extra_repository_keys will be populated with keys to install into
+        /etc/apt/keyrings, if needed.
+
+        extra_repository_sources will be populated with deb822 sources files,
+        if supported.
+        """
+        inline_signed_by = self.supports_inline_signed_by(codename)
+        for i, extra_repo in enumerate(self.data.extra_repositories or []):
+            if not inline_signed_by:
+                if extra_repo.signing_key:
+                    path = destination / f"extra_apt_key_{i}.asc"
+                    path.write_text(extra_repo.signing_key + "\n")
+                    self.extra_repository_keys.append(path)
+
+            # apt < 1.1 has no support for deb822 sources
+            if self.supports_deb822_sources(codename):
+                path = destination / f"extra_repository_{i}.sources"
+                signed_by = None
+                if not inline_signed_by:
+                    signed_by = f"/etc/apt/keyrings/extra_apt_key_{i}.asc"
+                path.write_text(
+                    extra_repo.as_deb822_source(signed_by_filename=signed_by)
+                )
+                self.extra_repository_sources.append(path)
+
+    def iter_oneline_sources(self) -> Generator[str]:
+        """
+        Generate apt one-line sources.list entries.
+
+        Use this for releases that don't supports_deb822_sources.
+        """
+        for i, extra_repository in enumerate(
+            self.data.extra_repositories or []
+        ):
+            yield extra_repository.as_oneline_source(
+                signed_by_filename=f"/etc/apt/keyrings/extra_apt_key_{i}.asc"
+            )
