@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -12,7 +12,7 @@ import logging
 import re
 import shlex
 from pathlib import Path
-from typing import Any, TypeAlias
+from typing import Any, TypeAlias, cast
 
 try:
     import pydantic.v1 as pydantic
@@ -28,7 +28,11 @@ from debusine.artifacts.models import (
     DebianAutopkgtestResultStatus,
     DebianAutopkgtestSource,
 )
-from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
+from debusine.tasks import (
+    BaseTaskWithExecutor,
+    ExtraRepositoryMixin,
+    RunCommandTask,
+)
 from debusine.tasks.models import AutopkgtestData, AutopkgtestDynamicData
 from debusine.tasks.server import TaskDatabaseInterface
 
@@ -38,6 +42,7 @@ ParsedSummaryFile: TypeAlias = dict[str, DebianAutopkgtestResult]
 
 
 class Autopkgtest(
+    ExtraRepositoryMixin[AutopkgtestData, AutopkgtestDynamicData],
     RunCommandTask[AutopkgtestData, AutopkgtestDynamicData],
     BaseTaskWithExecutor[AutopkgtestData, AutopkgtestDynamicData],
 ):
@@ -176,6 +181,14 @@ class Autopkgtest(
 
         return parsed
 
+    def _environment_codename(self) -> str:
+        """Return the codename of the environment we're testing on."""
+        if self.executor is None:
+            raise ValueError(
+                "self.executor not set, call _prepare_executor() first"
+            )
+        return cast(str, self.executor.system_image.data["codename"])
+
     def fetch_input(self, destination: Path) -> bool:
         """Download the required artifacts."""
         assert self.dynamic_data
@@ -196,6 +209,12 @@ class Autopkgtest(
         for artifact_id in self.dynamic_data.input_context_artifacts_ids:
             self.fetch_artifact(artifact_id, destination)
 
+        self._prepare_executor()
+        self.write_extra_repository_config(
+            codename=self._environment_codename(),
+            destination=destination,
+        )
+
         return True
 
     def _cmdline(self) -> list[str]:
@@ -227,11 +246,10 @@ class Autopkgtest(
         if debug_level := self.data.debug_level:
             cmd.append("-" + "d" * debug_level)
 
-        for extra_apt_source in self.data.extra_apt_sources:
-            cmd.append(f"--add-apt-source={extra_apt_source}")
+        cmd += self._cmdline_extra_repository()
 
         if self.data.use_packages_from_base_repository:
-            release = self.executor.system_image.data["codename"]
+            release = self._environment_codename()
             cmd.append(f"--apt-default-release={release}")
 
         for variable, value in self.data.extra_environment.items():
@@ -258,6 +276,24 @@ class Autopkgtest(
 
         return cmd
 
+    def _cmdline_extra_repository(self) -> list[str]:
+        """Generate command line arguments for extra_repositories."""
+        args: list[str] = []
+        if self.supports_deb822_sources(self._environment_codename()):
+            # autopkgtest has native deb822 support coming soon: #1089736
+            for path in self.extra_repository_sources:
+                args.append(
+                    f"--copy={path}:/etc/apt/sources.list.d/{path.name}"
+                )
+        else:
+            for source in self.iter_oneline_sources():
+                args.append(f"--add-apt-source={source}")
+
+        # apt < 2.3.10 has no support for keys embedded in Signed-By
+        for path in self.extra_repository_keys:
+            args.append(f"--copy={path}:/etc/apt/keyrings/{path.name}")
+        return args
+
     def configure_for_execution(
         self, download_directory: Path  # noqa: U100
     ) -> bool:
@@ -287,8 +323,6 @@ class Autopkgtest(
         )
         self._autopkgtest_targets.append(dsc_file)
 
-        self._prepare_executor()
-
         return True
 
     def task_succeeded(
