@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -9,7 +9,6 @@
 
 """MMDebstrap task, extending the SystemBootstrap ontology."""
 import shlex
-import subprocess
 from pathlib import Path
 from typing import Any
 
@@ -23,7 +22,6 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
     """Implement MmDebstrap: extends the ontology SystemBootstrap."""
 
     _OUTPUT_SYSTEM_FILE = "system.tar.xz"
-    _OS_RELEASE_FILE = "os-release"
     _VAR_LIB_DPKG = "var_lib_dpkg"
     _TEST_SBIN_INIT_RETURN_CODE_FILE = "test-sbin-init"
 
@@ -35,18 +33,6 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
         """Initialize MmDebstrap."""
         super().__init__(task_data, dynamic_task_data)
 
-        self._host_sources_file: Path | None = None
-        self._chroot_sources_file: Path | None = None
-
-        # keyrings that will be uploaded into the chroot
-        self._upload_keyrings: list[Path] = []
-
-        # all the keyrings that have been downloaded
-        self._keyrings: list[Path] = []
-
-        # customization_script file
-        self._customization_script: Path | None = None
-
     @classmethod
     def analyze_worker(cls) -> dict[str, Any]:
         """Report metadata for this task on this worker."""
@@ -95,7 +81,7 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
             # case find fails with
             # "Failed to restore initial working directory"
             (
-                '--customize-hook=cd "$1"; '
+                '--customize-hook=cd "$1" && '
                 "find etc/apt/sources.list.d -type f -delete"
             ),
             (
@@ -189,79 +175,6 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
         """Do nothing: no artifacts need to be downloaded."""
         return True
 
-    def configure_for_execution(self, download_dir: Path) -> bool:
-        """Create file.sources and add it into the log."""
-        # Create file.sources (keyrings are downloaded, if needed)
-        keyrings_dir = download_dir / "keyrings"
-
-        # mmdebstrap uses the subuid for using the keyring files. The directory
-        # where the keyring files are saved must be accessible for the
-        # mmdebstrap subuid user (and not only by the debusine-worker user).
-        download_dir.chmod(0o755)
-        keyrings_dir.mkdir(mode=0o755)
-
-        host_sources = self._generate_deb822_sources(
-            self.data.bootstrap_repositories,
-            keyrings_dir=keyrings_dir,
-            use_signed_by=self.data.bootstrap_options.use_signed_by,
-        )
-
-        # Make the files readable by any user (same reason as
-        # above for the directory) and add them in self._keyrings
-        # so cmdline use them
-        for keyring in keyrings_dir.iterdir():
-            keyring.chmod(0o644)
-            self._keyrings.append(keyring)
-
-        chroot_sources = [host_source.copy() for host_source in host_sources]
-
-        # Change the path of the keyrings from the host paths to the
-        # chroot path
-        for chroot_source, task_repository in zip(
-            chroot_sources, self.data.bootstrap_repositories
-        ):
-            if (signed_by := chroot_source.get("Signed-By")) is None:
-                # Nothing needs to be done
-                continue
-
-            # If Signed-By has been set by _generate_deb822_sources, it means
-            # that the repository specified a keyring
-            assert task_repository.keyring is not None
-
-            if task_repository.keyring.install:
-                signed_by_path = Path(signed_by)
-                filename = signed_by_path.name
-                chroot_source["Signed-By"] = (
-                    "/etc/apt/keyrings-debusine/" + filename
-                )
-                self._upload_keyrings.append(signed_by_path)
-            else:
-                # The key is not installed in the chroot, no "Signed-By"
-                del chroot_source["Signed-By"]
-
-        self._host_sources_file = download_dir / "host.sources"
-        self._chroot_sources_file = download_dir / "chroot.sources"
-
-        self._write_deb822s(host_sources, self._host_sources_file)
-        self._write_deb822s(chroot_sources, self._chroot_sources_file)
-
-        # Add the host and chroot source files for debugging purposes
-        self.append_to_log_file(
-            self._host_sources_file.name,
-            self._host_sources_file.read_text().splitlines(),
-        )
-
-        self.append_to_log_file(
-            self._chroot_sources_file.name,
-            self._chroot_sources_file.read_text().splitlines(),
-        )
-
-        if script := self.data.customization_script:
-            self._customization_script = download_dir / "customization_script"
-            self._customization_script.write_text(script)
-
-        return True
-
     def upload_artifacts(
         self, execute_dir: Path, *, execution_success: bool
     ) -> None:
@@ -269,98 +182,58 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
         if not self.debusine:
             raise AssertionError("self.debusine not set")
 
-        if execution_success:
-            system_file = execute_dir / self._OUTPUT_SYSTEM_FILE
+        if not execution_success:
+            return
+        system_file = execute_dir / self._OUTPUT_SYSTEM_FILE
 
-            bootstrap_options = self.data.bootstrap_options
-            vendor = self._get_value_os_release(
-                execute_dir / self._OS_RELEASE_FILE, "ID"
+        bootstrap_options = self.data.bootstrap_options
+        vendor = self._get_value_os_release(
+            execute_dir / self._OS_RELEASE_FILE, "ID"
+        )
+
+        main_bootstrap_repository = self.data.bootstrap_repositories[0]
+        try:
+            codename = self._get_value_os_release(
+                execute_dir / self._OS_RELEASE_FILE, "VERSION_CODENAME"
             )
+        except KeyError:
+            # jessie doesn't provide VERSION_CODENAME
+            codename = main_bootstrap_repository.suite
 
-            main_bootstrap_repository = self.data.bootstrap_repositories[0]
-            try:
-                codename = self._get_value_os_release(
-                    execute_dir / self._OS_RELEASE_FILE, "VERSION_CODENAME"
-                )
-            except KeyError:
-                # jessie doesn't provide VERSION_CODENAME
-                codename = main_bootstrap_repository.suite
+        # /etc/os-release reports the testing release for unstable (#341)
+        if main_bootstrap_repository.suite in ("unstable", "sid"):
+            codename = "sid"
 
-            # /etc/os-release reports the testing release for unstable (#341)
-            if main_bootstrap_repository.suite in ("unstable", "sid"):
-                codename = "sid"
+        pkglist = self._get_pkglist(execute_dir / self._VAR_LIB_DPKG)
+        with_init = self._get_with_init(
+            execute_dir / self._TEST_SBIN_INIT_RETURN_CODE_FILE
+        )
 
-            pkglist = self._get_pkglist(execute_dir / self._VAR_LIB_DPKG)
-            with_init = self._get_with_init(
-                execute_dir / self._TEST_SBIN_INIT_RETURN_CODE_FILE
-            )
+        artifact = DebianSystemTarballArtifact.create(
+            system_file,
+            data={
+                "variant": bootstrap_options.variant,
+                "architecture": bootstrap_options.architecture,
+                "vendor": vendor,
+                "codename": codename,
+                "pkglist": pkglist,
+                # with_dev: with the current setup (in mmdebstrap),
+                # the devices in /dev are always created
+                "with_dev": True,
+                "with_init": with_init,
+                # TODO / XXX: is "mirror" meant to be the first repository?
+                # or "mirrors" and list all of them?
+                # Or we could duplicate all the bootstrap_repositories...
+                "mirror": main_bootstrap_repository.mirror,
+            },
+        )
 
-            artifact = DebianSystemTarballArtifact.create(
-                system_file,
-                data={
-                    "variant": bootstrap_options.variant,
-                    "architecture": bootstrap_options.architecture,
-                    "vendor": vendor,
-                    "codename": codename,
-                    "pkglist": pkglist,
-                    # with_dev: with the current setup (in mmdebstrap),
-                    # the devices in /dev are always created
-                    "with_dev": True,
-                    "with_init": with_init,
-                    # TODO / XXX: is "mirror" meant to be the first repository?
-                    # or "mirrors" and list all of them?
-                    # Or we could duplicate all the bootstrap_repositories...
-                    "mirror": main_bootstrap_repository.mirror,
-                },
-            )
-
-            self.debusine.upload_artifact(
-                artifact,
-                workspace=self.workspace_name,
-                work_request=self.work_request_id,
-            )
+        self.debusine.upload_artifact(
+            artifact,
+            workspace=self.workspace_name,
+            work_request=self.work_request_id,
+        )
 
     @staticmethod
     def _get_with_init(test_init_return_code_file: Path) -> bool:
         return test_init_return_code_file.read_text().rstrip() == "0"
-
-    @staticmethod
-    def _get_value_os_release(os_release: Path, key: str) -> str:
-        """Parse file with the format /etc/os-release, return value for key."""
-        # In https://www.freedesktop.org/software/systemd/man/latest/os-release.html  # noqa: E501
-        # it specifies that the file is a "newline-separated list of
-        # environment-like shell-compatible variable assignments".
-        # Thus, shlex.split() is used it removes the quotes if they are in
-        # any value
-
-        for key_value in shlex.split(os_release.read_text()):
-            k, v = key_value.split("=", 1)
-
-            if key == k:
-                return v
-
-        raise KeyError(key)
-
-    @staticmethod
-    def _get_pkglist(var_lib_dpkg: Path) -> dict[str, str]:
-        """
-        Execute dpkg-query --admindir=var_lib_dpkg/dpkg --show.
-
-        :return: dictionary with package names and versions.
-        """
-        cmd = [
-            "dpkg-query",
-            f"--admindir={var_lib_dpkg}/dpkg",
-            "--show",
-        ]
-
-        process = subprocess.run(
-            cmd, check=True, text=True, capture_output=True
-        )
-
-        result = {}
-        for line in process.stdout.splitlines():
-            name, version = line.split("\t")
-            result[name] = version
-
-        return result
