@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -17,6 +17,8 @@ from itertools import groupby
 from pathlib import Path
 from typing import Annotated, Any, Generic, Literal, TypeAlias, TypeVar, Union
 
+import debian.deb822 as deb822
+
 try:
     import pydantic.v1 as pydantic
 except ImportError:
@@ -182,6 +184,85 @@ class CollectionItemMatcher(BaseTaskDataModel):
     value: Any
 
 
+class ExtraRepository(BaseTaskDataModel):
+    """extra_repositories for Debian tasks."""
+
+    url: pydantic.AnyUrl
+    suite: str
+    components: list[str] | None = None
+    signing_key: str | None = None
+
+    @pydantic.validator("suite")
+    @classmethod
+    def suite_regex(cls, suite: str) -> str:
+        """Ensure that suite matches a regex."""
+        # Replace with an Annotated[str] in pydantic 2
+        if not re.match(r"^(\w|[./ -])+$", suite):
+            raise ValueError(f"Invalid suite {suite}, must match (\\w|[./ -])+")
+        return suite
+
+    @pydantic.validator("components")
+    @classmethod
+    def components_regex(cls, components: list[str] | None) -> list[str] | None:
+        """Ensure that suite matches a regex."""
+        if not components:
+            return None
+        # Replace with an Annotated[str] in pydantic 2
+        for component in components:
+            if not re.match(r"^(\w|-)+$", component):
+                raise ValueError(
+                    f"Invalid component {component}, must match (\\w|-)+"
+                )
+        return components
+
+    @pydantic.root_validator
+    @classmethod
+    def flat_repository(cls, values: dict[str, Any]) -> dict[str, Any]:
+        """Check if suite is a flat repository."""
+        suite = values.get("suite")
+        components = values.get("components")
+        if suite and suite.endswith("/"):
+            if components is not None:
+                raise ValueError(
+                    "Components cannot be specified for a flat "
+                    "repository (where the suite ends with /)"
+                )
+        elif not components:
+            raise ValueError("Components must be specified")
+        return values
+
+    def as_oneline_source(self, signed_by_filename: str | None = None) -> str:
+        """Render a one-line sources.list entry."""
+        parts = ["deb"]
+        if self.signing_key:
+            if not signed_by_filename:
+                raise ValueError("No signed_by_filename specified")
+            parts.append(f"[signed-by={signed_by_filename}]")
+        parts.append(self.url)
+        parts.append(self.suite)
+        if self.components:
+            parts += self.components
+        return ' '.join(parts)
+
+    def as_deb822_source(self, signed_by_filename: str | None = None) -> str:
+        """Render a Deb822 sources.list.d entry."""
+        source = deb822.Deb822()
+        source["Types"] = "deb"
+        source["URIs"] = self.url
+        source["Suites"] = self.suite
+        if self.components:
+            source["Components"] = " ".join(self.components)
+        if self.signing_key:
+            if signed_by_filename:
+                source["Signed-By"] = signed_by_filename
+            else:
+                source["Signed-By"] = "\n" + "\n".join(
+                    " " + (line or ".")
+                    for line in self.signing_key.splitlines()
+                )
+        return source.dump()
+
+
 class LookupDict(BaseTaskDataModel):
     """Dictionary lookups for collection items."""
 
@@ -494,6 +575,12 @@ class BaseTaskDataWithExecutor(BaseTaskData):
     environment: LookupSingle | None = None
 
 
+class BaseTaskDataWithExtraRepositories(BaseTaskData):
+    """Base task data with fields used to configure extra_repositories."""
+
+    extra_repositories: list[ExtraRepository] | None = None
+
+
 class BaseDynamicTaskData(BaseTaskDataModel):
     """
     Base class for dynamic task data.
@@ -513,6 +600,45 @@ class BaseDynamicTaskDataWithExecutor(BaseDynamicTaskData):
     environment_id: int | None = None
 
 
+class RuntimeStatistics(BaseTaskDataModel):
+    """Runtime statistics about a completed task."""
+
+    #: Runtime duration of task, in seconds.
+    duration: int | None = None
+
+    #: Amount of CPU time used (both user and system), in seconds.
+    cpu_time: int | None = None
+
+    #: Maximum disk space used, in bytes.
+    disk_space: int | None = None
+
+    #: Maximum RAM used, in bytes.
+    memory: int | None = None
+
+    #: Available disk space when the task started, in bytes; may be rounded.
+    available_disk_space: int | None = None
+
+    #: Available RAM when the task started, in bytes; may be rounded.
+    available_memory: int | None = None
+
+    #: Number of CPU cores on the worker that ran the task.
+    cpu_count: int | None = None
+
+
+class OutputDataError(BaseTaskDataModel):
+    """An error encountered when running a task."""
+
+    message: str
+    code: str
+
+
+class OutputData(BaseTaskDataModel):
+    """Data produced when a task is completed."""
+
+    runtime_statistics: RuntimeStatistics | None = None
+    errors: list[OutputDataError] | None = None
+
+
 class NoopData(BaseTaskData):
     """In memory task data for the Noop task."""
 
@@ -553,7 +679,9 @@ class AutopkgtestTimeout(BaseTaskDataModel):
     copy_: int | None = pydantic.Field(alias="copy", ge=0)
 
 
-class AutopkgtestData(BaseTaskDataWithExecutor):
+class AutopkgtestData(
+    BaseTaskDataWithExecutor, BaseTaskDataWithExtraRepositories
+):
     """In memory task data for the Autopkgtest task."""
 
     input: AutopkgtestInput
@@ -561,7 +689,6 @@ class AutopkgtestData(BaseTaskDataWithExecutor):
     include_tests: list[str] = pydantic.Field(default_factory=list)
     exclude_tests: list[str] = pydantic.Field(default_factory=list)
     debug_level: int = pydantic.Field(default=0, ge=0, le=3)
-    extra_apt_sources: list[str] = pydantic.Field(default_factory=list)
     use_packages_from_base_repository: bool = False
     extra_environment: dict[str, str] = pydantic.Field(default_factory=dict)
     needs_internet: AutopkgtestNeedsInternet = AutopkgtestNeedsInternet.RUN
@@ -769,6 +896,7 @@ class SystemBootstrapOptions(BaseTaskDataModel):
     # this to particular values.
     variant: str | None = None
     extra_packages: list[str] = pydantic.Field(default_factory=list)
+    use_signed_by: bool = True
 
 
 class SystemBootstrapRepositoryType(StrEnum):
@@ -918,7 +1046,7 @@ class PiupartsDataInput(BaseTaskDataModel):
     binary_artifacts: LookupMultiple
 
 
-class PiupartsData(BaseTaskDataWithExecutor):
+class PiupartsData(BaseTaskDataWithExecutor, BaseTaskDataWithExtraRepositories):
     """In memory task data for the Piuparts task."""
 
     input: PiupartsDataInput
@@ -970,59 +1098,12 @@ class SbuildBuildComponent(StrEnum):
     SOURCE = "source"
 
 
-class SbuildExtraRepository(BaseTaskDataModel):
-    """extra_repositories for sbuild task."""
-
-    url: pydantic.AnyUrl
-    suite: str
-    components: list[str] | None = None
-    signing_key: str | None = None
-
-    @pydantic.validator("suite")
-    @classmethod
-    def suite_regex(cls, suite: str) -> str:
-        """Ensure that suite matches a regex."""
-        # Replace with an Annotated[str] in pydantic 2
-        if not re.match(r"^(\w|[./ -])+$", suite):
-            raise ValueError(f"Invalid suite {suite}, must match (\\w|[./ -])+")
-        return suite
-
-    @pydantic.validator("components")
-    @classmethod
-    def components_regex(cls, components: list[str]) -> list[str]:
-        """Ensure that suite matches a regex."""
-        # Replace with an Annotated[str] in pydantic 2
-        for component in components:
-            if not re.match(r"^(\w|-)+$", component):
-                raise ValueError(
-                    f"Invalid component {component}, must match (\\w|-)+"
-                )
-        return components
-
-    @pydantic.root_validator
-    @classmethod
-    def flat_repository(cls, values: dict[str, Any]) -> dict[str, Any]:
-        """Check if suite is a flat repository."""
-        suite = values.get("suite")
-        components = values.get("components")
-        if suite and suite.endswith("/"):
-            if components is not None:
-                raise ValueError(
-                    "Components cannot be specified for a flat "
-                    "repository (where the suite ends with /)"
-                )
-        elif not components:
-            raise ValueError("Components must be specified")
-        return values
-
-
-class SbuildData(BaseTaskDataWithExecutor):
+class SbuildData(BaseTaskDataWithExecutor, BaseTaskDataWithExtraRepositories):
     """In memory task data for the Sbuild task."""
 
     input: SbuildInput
     host_architecture: str
     distribution: str | None = None
-    extra_repositories: list[SbuildExtraRepository] | None = None
     build_components: list[SbuildBuildComponent] = pydantic.Field(
         default_factory=lambda: [SbuildBuildComponent.ANY],
     )
