@@ -1,4 +1,4 @@
-# Copyright 2021 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -42,7 +42,12 @@ from debusine.artifacts.models import (
     DoseDistCheck,
 )
 from debusine.client.models import RelationType, RemoteArtifact
-from debusine.tasks import BaseTaskWithExecutor, RunCommandTask, TaskConfigError
+from debusine.tasks import (
+    BaseTaskWithExecutor,
+    ExtraRepositoryMixin,
+    RunCommandTask,
+    TaskConfigError,
+)
 from debusine.tasks.models import SbuildData, SbuildDynamicData
 from debusine.tasks.sbuild_validator_mixin import SbuildValidatorMixin
 from debusine.tasks.server import TaskDatabaseInterface
@@ -51,6 +56,7 @@ from debusine.utils import read_dsc
 
 class Sbuild(
     SbuildValidatorMixin,
+    ExtraRepositoryMixin[SbuildData, SbuildDynamicData],
     RunCommandTask[SbuildData, SbuildDynamicData],
     BaseTaskWithExecutor[SbuildData, SbuildDynamicData],
 ):
@@ -72,11 +78,6 @@ class Sbuild(
         self._dsc_file: Path | None = None
         self._extra_packages: list[Path] = []
 
-        # Deb822 style sources:
-        self._extra_repository_sources: list[Path] = []
-        # GPG keys (for apt < 2.3.10):
-        self._extra_repository_keys: list[Path] = []
-
     def compute_dynamic_data(
         self, task_database: TaskDatabaseInterface
     ) -> SbuildDynamicData:
@@ -271,7 +272,9 @@ class Sbuild(
         if self.backend != "schroot":
             self._prepare_executor()
 
-        self._write_extra_repository_config(destination)
+        self.write_extra_repository_config(
+            codename=self._environment_codename(), destination=destination
+        )
 
         return True
 
@@ -291,49 +294,6 @@ class Sbuild(
 
         return cast(str, self.executor.system_image.data["codename"])
 
-    def _write_extra_repository_config(self, destination: Path) -> None:
-        """Write extra_repositories config to files in destination."""
-        codename = self._environment_codename()
-        # apt < 2.3.10 has no support for keys inline in Signed-By
-        inline_signed_by = codename not in (
-            "jessie",
-            "stretch",
-            "buster",
-            "bullseye",
-            "trusty",
-            "xenial",
-            "bionic",
-            "focal",
-        )
-        for i, extra_repo in enumerate(self.data.extra_repositories or []):
-            if not inline_signed_by:
-                if extra_repo.signing_key:
-                    path = destination / f"extra_apt_key_{i}.asc"
-                    path.write_text(extra_repo.signing_key + "\n")
-                    self._extra_repository_keys.append(path)
-
-            # apt < 1.1 has no support for deb822 sources
-            if codename not in ("jessie", "trusty"):
-                path = destination / f"extra_repository_{i}.sources"
-                source = deb822.Deb822()
-                source["Types"] = "deb"
-                source["URIs"] = extra_repo.url
-                source["Suites"] = extra_repo.suite
-                if extra_repo.components:
-                    source["Components"] = " ".join(extra_repo.components)
-                if extra_repo.signing_key:
-                    if inline_signed_by:
-                        source["Signed-By"] = "\n" + "\n".join(
-                            " " + (line or ".")
-                            for line in extra_repo.signing_key.splitlines()
-                        )
-                    else:
-                        source["Signed-By"] = (
-                            f"/etc/apt/keyrings/extra_apt_key_{i}.asc"
-                        )
-                path.write_text(source.dump())
-                self._extra_repository_sources.append(path)
-
     def _cmdline(self) -> list[str]:
         """
         Build the sbuild command line.
@@ -409,33 +369,26 @@ class Sbuild(
         """Generate the command line arguments for extra_repositories."""
         args: list[str] = []
 
-        # apt < 1.1 has no support for deb822 sources
-        if self._environment_codename() in ("jessie", "trusty"):
-            for extra_repository in self.data.extra_repositories or []:
-                source = f"deb {extra_repository.url} {extra_repository.suite}"
-                if extra_repository.components:
-                    source += f" {' '.join(extra_repository.components)}"
+        if self.supports_deb822_sources(self._environment_codename()):
+            # sbuild has no native deb822 support, yet: #1089735
+            for path in self.extra_repository_sources:
+                args.append(
+                    f"--pre-build-commands=cat {path} | "
+                    f"%SBUILD_CHROOT_EXEC sh -c 'cat > "
+                    f"/etc/apt/sources.list.d/{path.name}'"
+                )
+        else:
+            for source in self.iter_oneline_sources():
                 args.append(f"--extra-repository={source}")
-            for key_file in self._extra_repository_keys:
-                args.append(f"--extra-repository-key={key_file}")
-            return args
 
         # apt < 2.3.10 has no support for keys embedded in Signed-By
-        for path in self._extra_repository_keys:
+        for path in self.extra_repository_keys:
             args.append(
                 f"--pre-build-commands=cat {path} | "
                 f"%SBUILD_CHROOT_EXEC sh -c 'mkdir -p /etc/apt/keyrings && "
                 f"cat > /etc/apt/keyrings/{path.name}'"
             )
 
-        # sbuild has no native deb822 support, yet: #1089735
-        for path in self._extra_repository_sources:
-            args.append(
-                f"--pre-build-commands=cat {path} | "
-                f"%SBUILD_CHROOT_EXEC sh -c 'cat > "
-                f"/etc/apt/sources.list.d/{path.name}'"
-            )
-
         return args
 
     def _cmdline_binnmu(self) -> list[str]:
