@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -20,3 +20,7 @@ class SystemImageBuild(
     """Implement ontology SystemImageBuild."""
 
     TASK_VERSION = 1
+
+    def get_label(self) -> str:
+        """Return the task label."""
+        return "bootstrap a system image"
