@@ -1,4 +1,4 @@
-# Copyright 2022-2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -10,6 +10,7 @@
 """Common test-helper code involving Tasks."""
 import datetime
 import hashlib
+import shlex
 import tarfile
 from collections.abc import Mapping
 from io import BytesIO
@@ -181,6 +182,34 @@ class TaskHelperMixin(Generic[TaskClass]):
 
         self.task = self.task_class(task_data)
 
+    def write_os_release(
+        self, filename: Path, codename: str = "bookworm", version: int = 12
+    ) -> dict[str, str]:
+        """
+        Write in directory / self.task._OS_RELEASE_FILE a release file.
+
+        :return: dictionary with keys to values.
+        """
+        data = {
+            "ID": "debian",
+            "NAME": "Debian GNU/Linux",
+            "PRETTY_NAME": f"Debian GNU/Linux {version} ({codename})",
+            "VERSION": f"{version} ({codename})",
+            "VERSION_CODENAME": codename,
+            "VERSION_ID": str(version),
+        }
+
+        if codename == "jessie":
+            del data["VERSION_CODENAME"]
+        if version >= 13:
+            del data["VERSION"]
+            del data["VERSION_ID"]
+
+        filename.write_text(
+            "\n".join(f"{k}={shlex.quote(v)}" for k, v in data.items()) + "\n"
+        )
+        return data
+
 
 class ExternalTaskHelperMixin(
     TaskHelperMixin[ExternalTaskClass], Generic[ExternalTaskClass]
