@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -329,8 +329,18 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
         self.assertEqual(self.task.data.debug_level, 0)
 
     def test_configure_extra_apt_sources(self) -> None:
-        """Field "extra_apt_sources" is allowed."""
-        self.configure_task(override={"extra_apt_sources": ["A", "B"]})
+        """Field "extra_repositories" is allowed."""
+        self.configure_task(
+            override={
+                "extra_repositories": [
+                    {
+                        "url": "http://example.net",
+                        "suite": "bookworm",
+                        "components": ["main"],
+                    }
+                ]
+            }
+        )
 
     def test_configure_use_packages_from_base_repository(self) -> None:
         """Field "use_package_from_base_repository" is allowed."""
@@ -617,21 +627,54 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
         self.assertIn("-dd", cmdline)
 
-    def test_cmdline_extra_apt_sources(self) -> None:
-        """Command line include "--add-apt-source=XXX"."""
+    def test_cmdline_extra_repositories_jessie(self) -> None:
+        """Ensure the --add-apt-source= parameter is passed to autopkgtest."""
+        self.configure_task(
+            override={
+                "extra_repositories": [
+                    {
+                        "url": "http://example.com/",
+                        "suite": "unstable",
+                        "components": ["main"],
+                    },
+                    {
+                        "url": "http://example.net/",
+                        "suite": "flat/",
+                    },
+                ],
+            }
+        )
         self.mock_executor()
-        extra_apt_sources = [
-            "deb https://deb.debian.org/debian bookworm main non-free "
-            "non-free-firmware contrib",
-            "deb https://deb.debian.org/debian-security bookworm-security "
-            "main non-free contrib",
-        ]
-        self.task.data.extra_apt_sources = extra_apt_sources
-
+        assert self.task.executor
+        self.task.executor.system_image.data["codename"] = "jessie"
         cmdline = self.task._cmdline()
 
-        for extra_apt_source in extra_apt_sources:
-            self.assertIn(f"--add-apt-source={extra_apt_source}", cmdline)
+        self.assertIn(
+            "--add-apt-source=deb http://example.com/ unstable main", cmdline
+        )
+        self.assertIn("--add-apt-source=deb http://example.net/ flat/", cmdline)
+
+    def test_cmdline_extra_repositories_trixie(self) -> None:
+        """Ensure the --copy= parameter is passed to autopkgtest."""
+        debusine_mock = self.mock_debusine()
+        self.mock_image_download(debusine_mock)
+        self.task._prepare_executor()
+        self.task.extra_repository_sources = [
+            Path("/tmp/extra_repository_0.sources")
+        ]
+        self.task.extra_repository_keys = [
+            Path("/tmp/key.asc"),
+        ]
+        cmdline = self.task._cmdline()
+
+        self.assertIn(
+            (
+                "--copy=/tmp/extra_repository_0.sources:"
+                "/etc/apt/sources.list.d/extra_repository_0.sources"
+            ),
+            cmdline,
+        )
+        self.assertIn("--copy=/tmp/key.asc:/etc/apt/keyrings/key.asc", cmdline)
 
     def test_cmdline_apt_default_release_not_included(self) -> None:
         """Command line NOT include "--apt-default-release={distribution}."""
@@ -783,9 +826,23 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
         self.assertEqual(self.task.executor.backend_name, "unshare")
 
+    def test_environment_codename_executor(self) -> None:
+        """Test _environment_codename gets the executor distribution."""
+        self.mock_executor()
+        self.assertEqual(self.task._environment_codename(), "bookworm")
+
+    def test_environment_codename_executor_unconfigured(self) -> None:
+        """Test _environment_codename raises ValueError without executor."""
+        with self.assertRaisesRegex(
+            ValueError,
+            r"self\.executor not set, call _prepare_executor\(\) first",
+        ):
+            self.task._environment_codename()
+
     def test_fetch_input_no_context_artifacts(self) -> None:
         """Download "source_artifact" and "binary_artifacts"."""
         debusine_mock = self.mock_debusine()
+        self.mock_image_download(debusine_mock)
         workspace = "Testing"
         debusine_mock.download_artifact.return_value = ArtifactResponse(
             id=10,
@@ -853,6 +910,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
     def test_fetch_input_context_artifacts(self) -> None:
         """Download the "context_artifacts"."""
         debusine_mock = self.mock_debusine()
+        self.mock_image_download(debusine_mock)
 
         destination = self.create_temporary_directory()
 
@@ -871,8 +929,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
     def test_configure_for_execution(self) -> None:
         """Autopkgtest configure_for_execution() return True."""
-        debusine_mock = self.mock_debusine()
-        self.mock_image_download(debusine_mock)
+        self.mock_executor()
 
         directory = self.create_temporary_directory()
 
