@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -11,13 +11,10 @@
 import copy
 import shlex
 from pathlib import Path
-from typing import Any, cast
+from typing import cast
 from unittest import mock
 from unittest.mock import call
 
-import responses
-from debian.deb822 import Deb822
-
 from debusine.artifacts.local_artifact import DebianSystemTarballArtifact
 from debusine.tasks import MmDebstrap
 from debusine.tasks.models import (
@@ -60,11 +57,6 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
         """Initialize test."""
         self.configure_task()
 
-    def tearDown(self) -> None:
-        """Delete temporary directory if it exists."""
-        if self.task._debug_log_files_directory:
-            self.task._debug_log_files_directory.cleanup()
-
     def test_configure_task(self) -> None:
         """self.configure_task() does not raise any exception."""
         self.configure_task(self.SAMPLE_TASK_DATA)
@@ -192,7 +184,7 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
             "--verbose",
             "--hook-dir=/usr/share/mmdebstrap/hooks/maybe-jessie-or-older",
             (
-                '--customize-hook=cd "$1"; '
+                '--customize-hook=cd "$1" && '
                 "find etc/apt/sources.list.d -type f -delete"
             ),
             (
@@ -302,266 +294,6 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
         directory = Path()
         self.assertTrue(self.task.fetch_input(directory))
 
-    @responses.activate
-    def test_configure_for_execution(self) -> None:
-        """Test configure_for_execution(): call _generate_deb822_sources."""
-        download_dir = self.create_temporary_directory()
-
-        # Assume that, in production code, the directory was created
-        # with restrictive permissions
-        download_dir.chmod(0o700)
-
-        url_0 = "https://example.com/keyring1.asc"
-        url_1 = "https://example.com/keyring2.asc"
-
-        repositories: list[dict[str, Any]] = [
-            {
-                "mirror": "https://deb.debian.org/deb",
-                "suite": "bookworm",
-                "components": ["main", "contrib"],
-                "check_signature_with": "system",
-            },
-            {
-                "mirror": "https://deb.debian.org/deb",
-                "suite": "bookworm",
-                "components": ["main", "contrib"],
-                "check_signature_with": "no-check",
-            },
-            {
-                "mirror": "https://deb.debian.org/deb",
-                "suite": "bookworm",
-                "components": ["main", "contrib"],
-                "check_signature_with": "external",
-                "keyring": {"url": url_0, "install": True},
-            },
-            {
-                "mirror": "https://deb.debian.org/deb",
-                "suite": "bullseye",
-                "components": ["main"],
-                "check_signature_with": "external",
-                "keyring": {"url": url_1, "install": False},
-            },
-        ]
-
-        keyring_contents_0 = (
-            b"-----BEGIN PGP PUBLIC KEY BLOCK-----\nASCII-armored keyring"
-        )
-        keyring_contents_1 = b"Binary keyring"
-
-        responses.add(
-            responses.GET,
-            url_0,
-            body=keyring_contents_0,
-        )
-        responses.add(
-            responses.GET,
-            url_1,
-            body=keyring_contents_1,
-        )
-
-        self.configure_task(override={"bootstrap_repositories": repositories})
-
-        with mock.patch.object(
-            self.task, "append_to_log_file", autospec=True
-        ) as append_to_log_file:
-            self.assertTrue(self.task.configure_for_execution(download_dir))
-
-        # Files were generated
-        sources_files = list(download_dir.glob("*.sources"))
-        self.assertEqual(len(sources_files), 2)
-
-        assert self.task._chroot_sources_file
-        # Source file Signed-By changed to the correct chroot path
-        with self.task._chroot_sources_file.open() as f:
-            for actual, task_repository in zip(
-                Deb822.iter_paragraphs(f), repositories
-            ):
-                if task_repository["check_signature_with"] in (
-                    "system",
-                    "no-check",
-                ):
-                    # For a "system" or "no-check": the chroot sources list
-                    # does not have any "Signed-By"
-                    self.assertNotIn("Signed-By", actual)
-                else:
-                    # It must be "external"
-                    self.assertEqual(
-                        task_repository["check_signature_with"], "external"
-                    )
-
-                    if task_repository["keyring"]["install"]:
-                        # The path changed to one that will make sense in chroot
-                        self.assertRegex(
-                            actual["Signed-By"],
-                            r"^/etc/apt/keyrings-debusine/.*\.asc",
-                        )
-                    else:
-                        # No Signed-By in the chroot: the keyring is not
-                        # uploaded (it is used for the initial setup only)
-                        self.assertNotIn("Signed-By", actual)
-
-        # check files were added in the log artifact
-        expected_calls = []
-        for source_file in [
-            download_dir / "host.sources",
-            download_dir / "chroot.sources",
-        ]:
-            expected_calls.append(
-                call(source_file.name, source_file.read_text().splitlines())
-            )
-        append_to_log_file.assert_has_calls(expected_calls)
-
-        assert self.task._host_sources_file
-        host_repositories = list(
-            Deb822.iter_paragraphs(self.task._host_sources_file.read_text())
-        )
-
-        # Both repositories with check_signature_external have Signed-By
-        # in the host.sources
-        self.assertEqual(
-            Path(host_repositories[2]["Signed-By"]).read_bytes(),
-            keyring_contents_0,
-        )
-        self.assertEqual(
-            Path(host_repositories[3]["Signed-By"]).read_bytes(),
-            keyring_contents_1,
-        )
-
-        # And the files were added in self.task._keyrings
-        self.assertIn(
-            Path(host_repositories[2]["Signed-By"]), self.task._keyrings
-        )
-        self.assertIn(
-            Path(host_repositories[3]["Signed-By"]), self.task._keyrings
-        )
-
-        # With appropriate extensions
-        self.assertEqual(Path(host_repositories[2]["Signed-By"]).suffix, ".asc")
-        self.assertEqual(Path(host_repositories[3]["Signed-By"]).suffix, ".gpg")
-
-        # Two keyrings added
-        self.assertEqual(len(self.task._keyrings), 2)
-
-        # Verify that self.task._upload_keyrings contains only one, correct
-        # suffix, correct content
-        upload_keyrings = self.task._upload_keyrings
-        self.assertEqual(len(upload_keyrings), 1)
-        self.assertEqual(upload_keyrings[0].suffix, ".asc")
-        self.assertEqual(upload_keyrings[0].read_bytes(), keyring_contents_0)
-
-        # The chroot sources file have one Signed-By and the other on
-        # no Signed-By
-        host_repositories = list(
-            Deb822.iter_paragraphs(self.task._chroot_sources_file.read_text())
-        )
-        self.assertRegex(
-            host_repositories[2]["Signed-By"],
-            r"^/etc/apt/keyrings-debusine/keyring-repo-.*\.asc$",
-        )
-        self.assertNotIn("Signed-By", host_repositories[3])
-
-        # Download directory can be read by anyone (needed for mmdebstrap to
-        # use the keys from the subuid, in mode unshare as it is used)
-        self.assertEqual(download_dir.stat().st_mode & 0o755, 0o755)
-
-        keyrings_dir = download_dir / "keyrings"
-        self.assertEqual(keyrings_dir.stat().st_mode & 0o755, 0o755)
-
-    @responses.activate
-    def test_configure_for_execution_no_signed_by_in_repositories(self) -> None:
-        """Test configure_for_execution(): use_signed_by is False."""
-        download_dir = self.create_temporary_directory()
-
-        bootstrap_options = {"architecture": "amd64", "use_signed_by": False}
-
-        url = "https://example.com/keyring1.asc"
-
-        repositories = [
-            {
-                "mirror": "https://deb.debian.org/deb",
-                "suite": "bookworm",
-                "components": ["main", "contrib"],
-                "check_signature_with": "external",
-                "keyring": {"url": url},
-            },
-        ]
-
-        responses.add(
-            responses.GET,
-            url,
-        )
-
-        self.configure_task(
-            override={
-                "bootstrap_repositories": repositories,
-                "bootstrap_options": bootstrap_options,
-            }
-        )
-        self.task.configure_for_execution(download_dir)
-
-        assert self.task._chroot_sources_file
-        assert self.task._host_sources_file
-        # Source file Signed-By changed to the correct chroot path
-        with self.task._chroot_sources_file.open() as chroot_sources:
-            with self.task._host_sources_file.open() as host_sources:
-                count = 0
-                for repository in [chroot_sources, host_sources]:
-                    self.assertNotIn("Signed-By", repository)
-                    count += 1
-
-                # Expect two: one from chroot, one from host sources
-                self.assertEqual(count, 2)
-
-        self.assertEqual(len(self.task._keyrings), 1)
-        self.assertEqual(len(self.task._upload_keyrings), 0)
-
-    def test_configure_for_execution_customization_script(self) -> None:
-        """Test configure_for_execution() write customization_script."""
-        download_dir = self.create_temporary_directory()
-
-        script = "#!/bin/sh\n\necho 'something'\n"
-        self.configure_task(
-            override={
-                "customization_script": script,
-            }
-        )
-
-        self.task.configure_for_execution(download_dir)
-
-        assert self.task._customization_script
-        self.assertEqual(
-            self.task._customization_script.name, "customization_script"
-        )
-        self.assertEqual(self.task._customization_script.read_text(), script)
-
-    def write_os_release(
-        self, directory: Path, codename: str = "bookworm", version: int = 12
-    ) -> dict[str, str]:
-        """
-        Write in directory / self.task._OS_RELEASE_FILE a release file.
-
-        :return: dictionary with keys to values.
-        """
-        data = {
-            "ID": "debian",
-            "NAME": "Debian GNU/Linux",
-            "PRETTY_NAME": f"Debian GNU/Linux {version} ({codename})",
-            "VERSION": f"{version} ({codename})",
-            "VERSION_CODENAME": codename,
-            "VERSION_ID": str(version),
-        }
-
-        if codename == "jessie":
-            del data["VERSION_CODENAME"]
-        if version >= 13:
-            del data["VERSION"]
-            del data["VERSION_ID"]
-
-        (directory / self.task._OS_RELEASE_FILE).write_text(
-            "\n".join(f"{k}={shlex.quote(v)}" for k, v in data.items()) + "\n"
-        )
-        return data
-
     def test_upload_artifacts(self) -> None:
         """Test upload_artifacts()."""
         directory = self.create_temporary_directory()
@@ -572,7 +304,9 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
         # Debusine.upload_artifact is mocked to verify the call only
         debusine_mock = self.mock_debusine()
 
-        os_release_data = self.write_os_release(directory)
+        os_release_data = self.write_os_release(
+            directory / self.task._OS_RELEASE_FILE
+        )
         (directory / self.task._TEST_SBIN_INIT_RETURN_CODE_FILE).write_text("0")
 
         packages = self.patch_subprocess_run_pkglist()
@@ -625,7 +359,7 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
         debusine_mock = self.mock_debusine()
 
         os_release_data = self.write_os_release(
-            directory, codename="jessie", version=8
+            directory / self.task._OS_RELEASE_FILE, codename="jessie", version=8
         )
         (directory / self.task._TEST_SBIN_INIT_RETURN_CODE_FILE).write_text("0")
 
@@ -680,7 +414,9 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
         debusine_mock = self.mock_debusine()
 
         os_release_data = self.write_os_release(
-            directory, codename="trixie", version=13
+            directory / self.task._OS_RELEASE_FILE,
+            codename="trixie",
+            version=13,
         )
         (directory / self.task._TEST_SBIN_INIT_RETURN_CODE_FILE).write_text("0")
 
@@ -724,24 +460,6 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
 
         debusine_mock.upload_artifact.assert_has_calls(calls)
 
-    def test_get_value_os_release(self) -> None:
-        """get_value_os_release() get the correct value."""
-        directory = self.create_temporary_directory()
-        os_release_data = self.write_os_release(directory)
-
-        actual = MmDebstrap._get_value_os_release(
-            directory / self.task._OS_RELEASE_FILE, "PRETTY_NAME"
-        )
-        self.assertEqual(actual, os_release_data["PRETTY_NAME"])
-
-    def test_get_value_os_release_key_error(self) -> None:
-        """get_value_os_release() raise KeyError."""
-        os_release = self.create_temporary_file()
-
-        key = "PRETTY_NAME"
-        with self.assertRaisesRegex(KeyError, key):
-            MmDebstrap._get_value_os_release(os_release, key)
-
     def test_upload_artifacts_do_nothing(self) -> None:
         """Test upload_artifacts() doing nothing: execution_success=False."""
         self.mock_debusine()
