@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -10,6 +10,7 @@
 """Unit tests for task models."""
 
 import re
+import textwrap
 from datetime import datetime, timezone
 from typing import get_args
 from unittest import TestCase
@@ -561,12 +562,12 @@ class AutopkgtestDataTests(TestCase):
                     )
 
 
-class SbuildExtraRepositoryTests(TestCase):
-    """Tests for SbuildExtraRepository."""
+class ExtraRepositoryTests(TestCase):
+    """Tests for ExtraRepository."""
 
     def test_valid_sources(self) -> None:
         """Accept a typical valid source."""
-        task_models.SbuildExtraRepository.parse_obj(
+        task_models.ExtraRepository.parse_obj(
             {
                 "url": "http://deb.debian.org/debian",
                 "suite": "bookworm",
@@ -576,7 +577,7 @@ class SbuildExtraRepositoryTests(TestCase):
 
     def test_source_with_key(self) -> None:
         """Accept a valid flat source with a key."""
-        task_models.SbuildExtraRepository.parse_obj(
+        task_models.ExtraRepository.parse_obj(
             {
                 "url": "http://example.com/repo",
                 "suite": "./",
@@ -584,12 +585,23 @@ class SbuildExtraRepositoryTests(TestCase):
             }
         )
 
+    def test_flat_source_with_empty_components_list(self) -> None:
+        """Rewrite an empty components list to None."""
+        repo = task_models.ExtraRepository.parse_obj(
+            {
+                "url": "http://example.com/repo",
+                "suite": "./",
+                "components": [],
+            }
+        )
+        self.assertIsNone(repo.components)
+
     def test_file_url(self) -> None:
         """Reject a file:// URL."""
         with self.assertRaisesRegex(
             ValueError, "invalid or missing URL scheme"
         ):
-            task_models.SbuildExtraRepository.parse_obj(
+            task_models.ExtraRepository.parse_obj(
                 {
                     "url": "file:/etc/passwd",
                     "suite": "bookworm",
@@ -600,7 +612,7 @@ class SbuildExtraRepositoryTests(TestCase):
     def test_crazy_suite(self) -> None:
         """Reject an unreasonable suite."""
         with self.assertRaisesRegex(ValueError, "Invalid suite b@d_suite"):
-            task_models.SbuildExtraRepository.parse_obj(
+            task_models.ExtraRepository.parse_obj(
                 {
                     "url": "http://example.com/",
                     "suite": "b@d_suite",
@@ -613,7 +625,7 @@ class SbuildExtraRepositoryTests(TestCase):
         with self.assertRaisesRegex(
             ValueError, "Components cannot be specified for a flat repository"
         ):
-            task_models.SbuildExtraRepository.parse_obj(
+            task_models.ExtraRepository.parse_obj(
                 {
                     "url": "http://example.com/",
                     "suite": "flat/",
@@ -624,7 +636,7 @@ class SbuildExtraRepositoryTests(TestCase):
     def test_crazy_component(self) -> None:
         """Reject an unreasonable component."""
         with self.assertRaisesRegex(ValueError, "Invalid component foo/bar"):
-            task_models.SbuildExtraRepository.parse_obj(
+            task_models.ExtraRepository.parse_obj(
                 {
                     "url": "http://example.com/",
                     "suite": "bookworm",
@@ -632,6 +644,141 @@ class SbuildExtraRepositoryTests(TestCase):
                 }
             )
 
+    def test_as_oneline_source_flat(self) -> None:
+        """Render a flat sources.list entry to a string."""
+        repo = task_models.ExtraRepository.parse_obj(
+            {
+                "url": "http://example.com/",
+                "suite": "flat/",
+            }
+        )
+        self.assertEqual(
+            repo.as_oneline_source(), "deb http://example.com/ flat/"
+        )
+
+    def test_as_oneline_source_components(self) -> None:
+        """Render a regular sources.list entry to a string."""
+        repo = task_models.ExtraRepository.parse_obj(
+            {
+                "url": "http://example.com/",
+                "suite": "bookworm",
+                "components": ["foo", "bar"],
+            }
+        )
+        self.assertEqual(
+            repo.as_oneline_source(), "deb http://example.com/ bookworm foo bar"
+        )
+
+    def test_as_oneline_source_signed(self) -> None:
+        """Render a signed-by sources.list entry to a string."""
+        repo = task_models.ExtraRepository.parse_obj(
+            {
+                "url": "http://example.com/",
+                "suite": "bookworm",
+                "components": ["foo", "bar"],
+                "signing_key": "PUBLIC KEY",
+            }
+        )
+
+        with self.assertRaisesRegex(
+            ValueError, r"No signed_by_filename specified"
+        ):
+            repo.as_oneline_source()
+
+        self.assertEqual(
+            repo.as_oneline_source(signed_by_filename="/signature.asc"),
+            (
+                "deb [signed-by=/signature.asc] http://example.com/ bookworm "
+                "foo bar"
+            ),
+        )
+
+    def test_as_deb822_source_flat(self) -> None:
+        """Render a flat sources.list entry to a string."""
+        repo = task_models.ExtraRepository.parse_obj(
+            {
+                "url": "http://example.com/",
+                "suite": "flat/",
+            }
+        )
+        self.assertEqual(
+            repo.as_deb822_source(),
+            textwrap.dedent(
+                """\
+                Types: deb
+                URIs: http://example.com/
+                Suites: flat/
+                """
+            ),
+        )
+
+    def test_as_deb822_source_components(self) -> None:
+        """Render a regular sources.list entry to a string."""
+        repo = task_models.ExtraRepository.parse_obj(
+            {
+                "url": "http://example.com/",
+                "suite": "bookworm",
+                "components": ["foo", "bar"],
+            }
+        )
+        self.assertEqual(
+            repo.as_deb822_source(),
+            textwrap.dedent(
+                """\
+                Types: deb
+                URIs: http://example.com/
+                Suites: bookworm
+                Components: foo bar
+                """
+            ),
+        )
+
+    def test_as_deb822_source_signed(self) -> None:
+        """Render a signed-by sources.list entry to a string."""
+        repo = task_models.ExtraRepository.parse_obj(
+            {
+                "url": "http://example.com/",
+                "suite": "bookworm",
+                "components": ["foo", "bar"],
+                "signing_key": "\n".join(
+                    (
+                        "-----BEGIN PGP PUBLIC KEY BLOCK-----",
+                        "",
+                        "ABCDEFGHI",
+                        "-----END PGP PUBLIC KEY BLOCK-----",
+                    )
+                ),
+            }
+        )
+        self.assertEqual(
+            repo.as_deb822_source(),
+            textwrap.dedent(
+                """\
+                Types: deb
+                URIs: http://example.com/
+                Suites: bookworm
+                Components: foo bar
+                Signed-By:
+                 -----BEGIN PGP PUBLIC KEY BLOCK-----
+                 .
+                 ABCDEFGHI
+                 -----END PGP PUBLIC KEY BLOCK-----
+                """
+            ),
+        )
+        self.assertEqual(
+            repo.as_deb822_source(signed_by_filename="/signature.asc"),
+            textwrap.dedent(
+                """\
+                Types: deb
+                URIs: http://example.com/
+                Suites: bookworm
+                Components: foo bar
+                Signed-By: /signature.asc
+                """
+            ),
+        )
+
 
 class SbuildDynamicDataTests(TestCase):
     """Tests for SbuildDynamicData."""
