@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -11,7 +11,9 @@
 import datetime
 import lzma
 import os
+import shlex
 import tarfile
+import textwrap
 from io import BytesIO
 from pathlib import Path
 from typing import BinaryIO
@@ -23,7 +25,11 @@ try:
 except ImportError:
     import pydantic as pydantic  # type: ignore
 
-from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianSystemTarball,
+)
 from debusine.client.models import (
     ArtifactResponse,
     FileResponse,
@@ -201,11 +207,21 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                     tarinfo.linkname = "../run/systemd/resolve/stub-resolv.conf"
                     tar.addfile(tarinfo)
 
-            data = {
-                "with_dev": with_dev,
-                "codename": codename,
-                "filename": filename,
-            }
+            fake_url = pydantic.parse_obj_as(
+                pydantic.AnyUrl, "https://not-used"
+            )
+
+            data = DebianSystemTarball(
+                filename=filename,
+                vendor="debian",
+                codename=codename,
+                mirror=fake_url,
+                variant=None,
+                pkglist={},
+                architecture="amd64",
+                with_dev=with_dev,
+                with_init=False,
+            )
 
             file_model = FileResponse(
                 size=100,
@@ -213,7 +229,7 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                     "sha256": pydantic.parse_obj_as(StrMaxLength255, "not-used")
                 },
                 type="file",
-                url=pydantic.parse_obj_as(pydantic.AnyUrl, "https://not-used"),
+                url=fake_url,
             )
 
             files = FilesResponseType({"system.tar.xz": file_model})
@@ -223,7 +239,7 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                 workspace="System",
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 created_at=datetime.datetime.now(),
-                data=data,
+                data=data.dict(),
                 download_tar_gz_url=pydantic.parse_obj_as(
                     pydantic.AnyUrl,
                     f"http://localhost/artifact/{artifact_id}?archive=tar.gz",
@@ -346,7 +362,7 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
             with_dev=True
         )
 
-        system_tar = destination_dir / "system.tar"
+        system_tar = destination_dir / "system.tar.gz"
 
         self.task._prepare_base_tgz(download_directory)
 
@@ -354,7 +370,8 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
             self.task.data.base_tgz, destination_dir
         )
 
-        # Downloaded system.tar.xz, then processed it and renamed to system.tar
+        # Downloaded system.tar.xz, then processed it and wrote the result
+        # to system.tar.gz
         self.assertEqual(self.task._base_tar, system_tar)
 
         assert isinstance(self.task.executor_instance, MagicMock)
@@ -382,7 +399,7 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
         )
 
         self.assertEqual(
-            list(destination_dir.iterdir()), [destination_dir / "system.tar"]
+            list(destination_dir.iterdir()), [destination_dir / "system.tar.gz"]
         )
 
     def test_prepare_base_tgz_download_base_tgz_resolveconf(self) -> None:
@@ -421,38 +438,155 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
             ],
         )
 
-        # Downloaded system.tar.xz, then processed it and renamed to system.tar
-        system_tar = destination_dir / "system.tar"
+        # Downloaded system.tar.xz, then processed it and wrote the result
+        # to system.tar.gz
+        system_tar = destination_dir / "system.tar.gz"
         self.assertEqual(self.task._base_tar, system_tar)
 
-        with tarfile.open(system_tar, "r") as tar:
+        with tarfile.open(system_tar, "r:gz") as tar:
             self.assertTrue(tar.getmember("./etc/resolv.conf").isreg())
 
+    def prepare_scripts(self, codename: str) -> str:
+        """Build and read the post_chroot_unpack_debusine script."""
+        directory = self.create_temporary_directory()
+        with mock.patch.object(self.task, "_base_tar_data", autospec=True):
+            assert self.task._base_tar_data
+            self.task._base_tar_data.codename = codename
+            self.task._prepare_scripts(directory)
+        script = directory / "post_chroot_unpack_debusine"
+        self.assertTrue(script.exists())
+        return script.read_text()
+
+    def test_prepare_scripts_jessie(self) -> None:
+        """_prepare_scripts with oneline apt sources."""
+        self.configure_task(
+            override={
+                "extra_repositories": [
+                    {
+                        "url": "http://example.net",
+                        "suite": "bookworm",
+                        "components": ["main"],
+                        "signing_key": "KEY A",
+                    },
+                ]
+            }
+        )
+        script = self.prepare_scripts("jessie")
+        self.assertIn("mkdir -p /etc/apt/keyrings\n", script)
+        key = shlex.quote("KEY A\n")
+        self.assertIn(
+            f"printf %s {key} > /etc/apt/keyrings/extra_apt_key_0.asc\n",
+            script,
+        )
+        source = shlex.quote(
+            "deb [signed-by=/etc/apt/keyrings/extra_apt_key_0.asc] "
+            "http://example.net bookworm main\n"
+        )
+        self.assertIn(
+            (
+                f"printf %s {source} > "
+                f"/etc/apt/sources.list.d/extra_repository_0.list\n"
+            ),
+            script,
+        )
+
+    def test_prepare_scripts_buster(self) -> None:
+        """_prepare_scripts with deb822 signed-by apt sources."""
+        self.configure_task(
+            override={
+                "extra_repositories": [
+                    {
+                        "url": "http://example.net",
+                        "suite": "bookworm",
+                        "components": ["main"],
+                        "signing_key": "KEY A",
+                    },
+                ]
+            }
+        )
+        script = self.prepare_scripts("buster")
+        self.assertIn("mkdir -p /etc/apt/keyrings\n", script)
+        key = shlex.quote("KEY A\n")
+        self.assertIn(
+            f"printf %s {key} > /etc/apt/keyrings/extra_apt_key_0.asc\n",
+            script,
+        )
+        source = shlex.quote(
+            textwrap.dedent(
+                """\
+                Types: deb
+                URIs: http://example.net
+                Suites: bookworm
+                Components: main
+                Signed-By: /etc/apt/keyrings/extra_apt_key_0.asc
+                """
+            )
+        )
+        self.assertIn(
+            (
+                f"printf %s {source} > "
+                f"/etc/apt/sources.list.d/extra_repository_0.sources\n"
+            ),
+            script,
+        )
+
+    def test_prepare_scripts_bookworm(self) -> None:
+        """_prepare_scripts with deb822 signed-by apt sources."""
+        self.configure_task(
+            override={
+                "extra_repositories": [
+                    {
+                        "url": "http://example.net",
+                        "suite": "bookworm",
+                        "components": ["main"],
+                        "signing_key": "KEY A",
+                    },
+                ]
+            }
+        )
+        script = self.prepare_scripts("bookworm")
+        self.assertNotIn("/etc/apt/keyrings", script)
+        source = shlex.quote(
+            textwrap.dedent(
+                """\
+                Types: deb
+                URIs: http://example.net
+                Suites: bookworm
+                Components: main
+                Signed-By:
+                 KEY A
+                """
+            )
+        )
+        self.assertIn(
+            (
+                f"printf %s {source} > "
+                f"/etc/apt/sources.list.d/extra_repository_0.sources\n"
+            ),
+            script,
+        )
+
     def test_execute(self) -> None:
         """Test full (mocked) execution."""
         self.configure_task(override={"input": {'binary_artifacts': [1, 2]}})
         self.task.dynamic_data = PiupartsDynamicData(
-            environment_id=1, input_binary_artifacts_ids=[1, 2], base_tgz_id=44
+            environment_id=7, input_binary_artifacts_ids=[1, 2], base_tgz_id=42
         )
         self.patch_prepare_executor_instance()
         download_directory = self.create_temporary_directory()
 
-        with mock.patch.object(
-            self.task,
-            "fetch_artifact",
-            autospec=True,
-            return_value=True,
-        ) as mocked:
-            self.assertTrue(self.task.fetch_input(download_directory))
-            mocked.assert_any_call(1, download_directory)
-            mocked.assert_any_call(2, download_directory)
+        fetch_artifact_mocked = self.patch_fetch_artifact_for_basetgz(
+            with_dev=True, codename="trixie"
+        )
+        self.assertTrue(self.task.fetch_input(download_directory))
+        fetch_artifact_mocked.assert_any_call(1, download_directory)
+        fetch_artifact_mocked.assert_any_call(2, download_directory)
 
         (file2 := download_directory / "file2.deb").write_text("")
         (file1 := download_directory / "file1.deb").write_text("")
         (file3 := download_directory / "makedev_2.3.1-97_all.deb").write_text(
             ""
         )
-        self.patch_fetch_artifact_for_basetgz(with_dev=True, codename="trixie")
 
         self.assertTrue(self.task.configure_for_execution(download_directory))
 
@@ -468,6 +602,7 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                     "--allow-database",
                     "--warn-on-leftovers-after-purge",
                     f"--basetgz={self.task._base_tar}",
+                    f"--scriptsdir={self.task._scripts_dir}",
                     str(file1),
                     str(file2),
                     str(file3),
