@@ -1,4 +1,4 @@
-# Copyright 2021-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -41,6 +41,7 @@ from debusine.tasks import (
     BaseExternalTask,
     BaseTask,
     BaseTaskWithExecutor,
+    ExtraRepositoryMixin,
     RunCommandTask,
     Sbuild,
     TaskConfigError,
@@ -56,6 +57,8 @@ from debusine.tasks.models import (
     BaseDynamicTaskDataWithExecutor,
     BaseTaskData,
     BaseTaskDataWithExecutor,
+    BaseTaskDataWithExtraRepositories,
+    ExtraRepository,
     TaskTypes,
     WorkerType,
 )
@@ -147,6 +150,19 @@ class TestBaseTaskWithExecutorAndArchitecture(
         raise NotImplementedError()
 
 
+class TestExtraRepositoryMixin(
+    TestTaskMixin,
+    ExtraRepositoryMixin[
+        BaseTaskDataWithExtraRepositories, BaseDynamicTaskData
+    ],
+):
+    """Test ExtraRepositoryMixin."""
+
+    def run(self, execute_directory: Path) -> bool:
+        """Unused abstract method from BaseExternalTask."""
+        raise NotImplementedError()
+
+
 class BaseTaskTests(TestCase):
     """Unit tests for BaseTask class."""
 
@@ -2063,3 +2079,207 @@ class TestTestTaskMixin(TestCase):
         """Test get_label."""
         task = TestTaskMixin()
         self.assertEqual(task.get_label(), "test")
+
+
+class ExtraRepositoryMixinTests(TestCase):
+    """Unit tests for BaseTaskWithExecutor class."""
+
+    def setUp(self) -> None:
+        """Create the shared attributes."""
+        super().setUp()
+        self.task = TestExtraRepositoryMixin({})
+        self.directory = self.create_temporary_directory()
+
+    def configure_extra_repositories(
+        self, *extra_repositories: dict[str, Any]
+    ) -> None:
+        """Parse a list of dictionaries into ExtraRepository objects."""
+        self.task.data.extra_repositories = []
+        for repo in extra_repositories:
+            self.task.data.extra_repositories.append(
+                ExtraRepository.parse_obj(repo)
+            )
+
+    def test_supports_deb822_sources(self) -> None:
+        """Test supports_deb822_sources with 4 sample distributions."""
+        self.assertFalse(self.task.supports_deb822_sources("jessie"))
+        self.assertFalse(self.task.supports_deb822_sources("trusty"))
+        self.assertTrue(self.task.supports_deb822_sources("stretch"))
+        self.assertTrue(self.task.supports_deb822_sources("xenial"))
+
+    def test_supports_inline_signed_by(self) -> None:
+        """Test supports_inline_signed_by with 4 sample distributions."""
+        self.assertFalse(self.task.supports_deb822_sources("jessie"))
+        self.assertFalse(self.task.supports_deb822_sources("trusty"))
+        self.assertTrue(self.task.supports_deb822_sources("bookworm"))
+        self.assertTrue(self.task.supports_deb822_sources("jammy"))
+
+    def test_write_extra_repository_config_jessie_no_keys(self) -> None:
+        """Test repositories without deb822 or keys."""
+        self.configure_extra_repositories(
+            {
+                "url": "http://example.net",
+                "suite": "bookworm",
+                "components": ["main"],
+            },
+        )
+
+        self.task.write_extra_repository_config("jessie", self.directory)
+
+        self.assertEqual(self.task.extra_repository_keys, [])
+        self.assertEqual(self.task.extra_repository_sources, [])
+
+    def test_write_extra_repository_config_two_keys(self) -> None:
+        """Test repositories with two keys."""
+        self.configure_extra_repositories(
+            {
+                "url": "http://example.net/",
+                "suite": "a",
+                "components": ["a"],
+                "signing_key": "KEY A",
+            },
+            {
+                "url": "http://example.net/",
+                "suite": "b/",
+                "signing_key": "KEY B",
+            },
+        )
+
+        self.task.write_extra_repository_config("jessie", self.directory)
+
+        self.assertEqual(len(self.task.extra_repository_keys), 2)
+        expected_keys = ["KEY A\n", "KEY B\n"]
+        for key_file, expected_key in zip(
+            self.task.extra_repository_keys, expected_keys
+        ):
+            self.assertEqual(key_file.read_text(), expected_key)
+
+        self.assertEqual(self.task.extra_repository_sources, [])
+
+    def test_write_extra_repository_config_deb822_signed_keyring(self) -> None:
+        """Test repositories in deb822 format with keys in apt < 2.3.10."""
+        self.configure_extra_repositories(
+            {
+                "url": "http://example.net",
+                "suite": "bullseye",
+                "components": ["main", "non-free"],
+                "signing_key": "\n".join(
+                    (
+                        "-----BEGIN PGP PUBLIC KEY BLOCK-----",
+                        "",
+                        "ABCDEFGHI",
+                        "-----END PGP PUBLIC KEY BLOCK-----",
+                    )
+                ),
+            },
+        )
+
+        self.task.write_extra_repository_config("bullseye", self.directory)
+
+        self.assertEqual(len(self.task.extra_repository_keys), 1)
+        self.assertEqual(
+            self.task.extra_repository_keys[0].read_text(),
+            textwrap.dedent(
+                """\
+                -----BEGIN PGP PUBLIC KEY BLOCK-----
+
+                ABCDEFGHI
+                -----END PGP PUBLIC KEY BLOCK-----
+                """
+            ),
+        )
+        self.assertEqual(
+            self.task.extra_repository_keys[0].name,
+            "extra_apt_key_0.asc",
+        )
+        self.assertEqual(len(self.task.extra_repository_sources), 1)
+        self.assertEqual(
+            self.task.extra_repository_sources[0].read_text(),
+            textwrap.dedent(
+                """\
+                Types: deb
+                URIs: http://example.net
+                Suites: bullseye
+                Components: main non-free
+                Signed-By: /etc/apt/keyrings/extra_apt_key_0.asc
+                """
+            ),
+        )
+
+    def test_write_extra_repository_config_deb822_signed_inline(self) -> None:
+        """Test repositories in deb822 format with keys inline."""
+        self.configure_extra_repositories(
+            {
+                "url": "http://example.net",
+                "suite": "bookworm",
+                "components": ["main", "non-free"],
+                "signing_key": "\n".join(
+                    (
+                        "-----BEGIN PGP PUBLIC KEY BLOCK-----",
+                        "",
+                        "ABCDEFGHI",
+                        "-----END PGP PUBLIC KEY BLOCK-----",
+                    )
+                ),
+            },
+        )
+
+        self.task.write_extra_repository_config("bookworm", self.directory)
+
+        self.assertEqual(self.task.extra_repository_keys, [])
+        self.assertEqual(len(self.task.extra_repository_sources), 1)
+        self.assertEqual(
+            self.task.extra_repository_sources[0].read_text(),
+            textwrap.dedent(
+                """\
+                Types: deb
+                URIs: http://example.net
+                Suites: bookworm
+                Components: main non-free
+                Signed-By:
+                 -----BEGIN PGP PUBLIC KEY BLOCK-----
+                 .
+                 ABCDEFGHI
+                 -----END PGP PUBLIC KEY BLOCK-----
+                """
+            ),
+        )
+
+    def test_iter_oneline_sources(self) -> None:
+        """Test iter_oneline_sources with a variety of sources."""
+        self.configure_extra_repositories(
+            {
+                "url": "http://example.net",
+                "suite": "flat/",
+            },
+            {
+                "url": "http://example.com",
+                "suite": "bookworm",
+                "components": ["main", "non-free"],
+            },
+            {
+                "url": "http://example.org",
+                "suite": "buster",
+                "components": ["main", "non-free"],
+                "signing_key": "\n".join(
+                    (
+                        "-----BEGIN PGP PUBLIC KEY BLOCK-----",
+                        "",
+                        "ABCDEFGHI",
+                        "-----END PGP PUBLIC KEY BLOCK-----",
+                    )
+                ),
+            },
+        )
+        sources = list(self.task.iter_oneline_sources())
+        self.assertEqual(
+            sources,
+            [
+                "deb http://example.net flat/",
+                "deb http://example.com bookworm main non-free",
+                (
+                    "deb [signed-by=/etc/apt/keyrings/extra_apt_key_2.asc] "
+                    "http://example.org buster main non-free"
+                ),
+            ],
+        )
