@@ -1,4 +1,4 @@
-# Copyright 2022-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -34,24 +34,14 @@ from unittest.util import safe_repr
 
 import django.test
 import requests
+from asgiref.sync import async_to_sync
 from channels.layers import get_channel_layer
 from django.db import transaction
 from django.db.models import Model, QuerySet
 from django.http.response import HttpResponseBase
 
-from debusine.artifacts.models import ArtifactCategory
 from debusine.db.context import ContextConsistencyError, context
-from debusine.db.models import (
-    Artifact,
-    ArtifactRelation,
-    File,
-    FileUpload,
-    Scope,
-    Token,
-    User,
-    WorkRequest,
-    Workspace,
-)
+from debusine.db.models import Scope, Token, User, WorkRequest, Workspace
 from debusine.db.models.permissions import (
     PermissionCheckPredicate,
     PermissionUser,
@@ -61,7 +51,6 @@ from debusine.db.models.permissions import (
     resolve_roles_list,
 )
 from debusine.db.playground.scenarios import Scenario
-from debusine.server.file_backend.interface import FileBackendInterface
 from debusine.test.playground import Playground
 
 # Relative import needed to break an import loop
@@ -70,6 +59,10 @@ from .base import TestCase as BaseTestCase  # noqa: ABS101
 if TYPE_CHECKING:
     from django.test.client import _MonkeyPatchedWSGIResponse
 
+    TestMixinBase = unittest.TestCase
+else:
+    TestMixinBase = object
+
 M = TypeVar("M", bound=Model)
 R = TypeVar("R", bound=Roles)
 
@@ -740,99 +733,6 @@ class TestCase(
                 if scenario.needs_set_current:
                     scenario.set_current()
 
-    @classmethod
-    def create_file_in_backend(
-        cls,
-        backend: FileBackendInterface[Any],
-        contents: bytes = b"test",
-    ) -> File:
-        """
-        Create a temporary file and adds it in the backend.
-
-        :param backend: file backend to add the file in
-        :param contents: contents of the file
-        """
-        return cls.playground.create_file_in_backend(backend, contents)
-
-    @classmethod
-    def create_artifact_relation(
-        cls,
-        artifact: Artifact,
-        target: Artifact,
-        relation_type: ArtifactRelation.Relations = (
-            ArtifactRelation.Relations.RELATES_TO
-        ),
-    ) -> ArtifactRelation:
-        """Create an ArtifactRelation."""
-        return cls.playground.create_artifact_relation(
-            artifact, target, relation_type
-        )
-
-    @classmethod
-    def create_artifact(
-        cls,
-        paths: list[str] | None = None,
-        files_size: int = 100,
-        *,
-        category: ArtifactCategory = ArtifactCategory.TEST,
-        workspace: Workspace | None = None,
-        data: dict[str, Any] | None = None,
-        expiration_delay: int | None = None,
-        work_request: WorkRequest | None = None,
-        created_by: User | None = None,
-        create_files: bool = False,
-        skip_add_files_in_store: bool = False,
-    ) -> tuple[Artifact, dict[str, bytes]]:
-        """
-        Create an artifact and return tuple with the artifact and files data.
-
-        :param paths: list of paths to create (will contain random data)
-        :param files_size: size of the test data
-        :param category: this artifact's category (see :ref:`artifacts`)
-        :param data: key-value data for this artifact (see :ref:`artifacts`)
-        :param expiration_delay: set expiration_delay field (in days)
-        :param work_request: work request that created this artifact
-        :param created_by: set Artifact.created_by to it
-        :param create_files: create a file and add it into the LocalFileBackend
-        :param skip_add_files_in_store: do not add the files in the store
-          (only create the File object in the database)
-
-        This method return a tuple:
-        - artifact: Artifact
-        - files_contents: Dict[str, bytes] (paths and test data)
-        """
-        return cls.playground.create_artifact(
-            paths,
-            files_size,
-            category=category,
-            workspace=workspace,
-            data=data,
-            expiration_delay=expiration_delay,
-            work_request=work_request,
-            created_by=created_by,
-            create_files=create_files,
-            skip_add_files_in_store=skip_add_files_in_store,
-        )
-
-    @classmethod
-    def create_file_upload(cls) -> FileUpload:
-        """
-        Create a new FileUpload object.
-
-        Create the workspace, artifact, file and file_in_artifact associated
-        to the file_upload object.
-        """
-        return cls.playground.create_file_upload()
-
-    @classmethod
-    def create_file(cls, contents: bytes = b"test") -> File:
-        """
-        Create a File model and return the saved fileobj.
-
-        :param contents: used to compute hash digest and size
-        """
-        return cls.playground.create_file(contents)
-
     def assert_work_request_event_reactions(
         self,
         work_request: WorkRequest,
@@ -867,13 +767,19 @@ class TestCase(
         )
 
 
-class ChannelsHelpersMixin:
+class ChannelsHelpersMixin(TestMixinBase):
     """
     Channel-related methods to help writing unit tests.
 
     Provides methods to setup a channel and assert messages or lack of messages.
     """
 
+    def tearDown(self) -> None:
+        """Flush the channel layer."""
+        super().tearDown()
+        channel_layer = get_channel_layer()
+        async_to_sync(channel_layer.flush)()
+
     # TODO: coverage is confused by something here, possibly
     # https://github.com/python/cpython/issues/106749
     async def create_channel(
