@@ -1,4 +1,4 @@
-# Copyright 2022-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -24,13 +24,7 @@ from rest_framework import status
 from rest_framework.response import Response
 
 from debusine.db.context import context
-from debusine.db.models import (
-    ArtifactRelation,
-    FileStore,
-    Scope,
-    User,
-    Workspace,
-)
+from debusine.db.models import ArtifactRelation, Scope, User, Workspace
 from debusine.server.views import ProblemResponse
 from debusine.test.django import (
     AllowAll,
@@ -169,9 +163,9 @@ class TestCaseTests(TestCase):
     @context.disable_permission_checks()
     def test_create_artifact_relation_default_type(self) -> None:
         """create_artifact_relation() create artifact with type=RELATED_TO."""
-        artifact, _ = self.create_artifact()
-        target, _ = self.create_artifact()
-        created_artifact_relation = self.create_artifact_relation(
+        artifact, _ = self.playground.create_artifact()
+        target, _ = self.playground.create_artifact()
+        created_artifact_relation = self.playground.create_artifact_relation(
             artifact, target
         )
         created_artifact_relation.refresh_from_db
@@ -186,11 +180,11 @@ class TestCaseTests(TestCase):
     @context.disable_permission_checks()
     def test_create_artifact_relation_specific_type(self) -> None:
         """create_artifact_relation() create artifact with given type."""
-        artifact, _ = self.create_artifact()
-        target, _ = self.create_artifact()
+        artifact, _ = self.playground.create_artifact()
+        target, _ = self.playground.create_artifact()
         relation_type = ArtifactRelation.Relations.BUILT_USING
 
-        created_artifact_relation = self.create_artifact_relation(
+        created_artifact_relation = self.playground.create_artifact_relation(
             artifact, target, relation_type
         )
         created_artifact_relation.refresh_from_db()
@@ -199,25 +193,10 @@ class TestCaseTests(TestCase):
         self.assertEqual(created_artifact_relation.target, target)
         self.assertEqual(created_artifact_relation.type, relation_type)
 
-    def test_create_file_in_backend(self) -> None:
-        """create_file_in_backend() returns a File and write contents to it."""
-        file_backend = FileStore.default().get_backend_object()
-        contents = b"some-test-data"
-
-        fileobj = self.create_file_in_backend(
-            file_backend,
-            contents,
-        )
-
-        self.assertIsInstance(fileobj.id, int)
-        local_path = file_backend.get_local_path(fileobj)
-        assert local_path is not None
-        self.assertEqual(local_path.read_bytes(), contents)
-
     @context.disable_permission_checks()
     def test_create_artifact_default_expire_date(self) -> None:
         """create_artifact() set expire_at to None by default."""
-        artifact, _ = self.create_artifact()
+        artifact, _ = self.playground.create_artifact()
         artifact.refresh_from_db()
         self.assertIsNone(artifact.expire_at)
 
@@ -225,7 +204,7 @@ class TestCaseTests(TestCase):
     def test_create_artifact_expire_date(self) -> None:
         """create_artifact() set expire_at use correct expire_at.."""
         now = timezone.now()
-        artifact, _ = self.create_artifact(expiration_delay=1)
+        artifact, _ = self.playground.create_artifact(expiration_delay=1)
         artifact.created_at = now - timedelta(days=1)
         artifact.save()
         artifact.refresh_from_db()
@@ -237,7 +216,7 @@ class TestCaseTests(TestCase):
         paths = ["src/a", "b"]
         files_size = 12
 
-        artifact, files_contents = self.create_artifact(
+        artifact, files_contents = self.playground.create_artifact(
             paths, files_size, create_files=True
         )
 
@@ -259,7 +238,9 @@ class TestCaseTests(TestCase):
     @context.disable_permission_checks()
     def test_create_artifact_do_not_create_files(self) -> None:
         """create_artifact() returns an Artifact and does not create files."""
-        artifact, _ = self.create_artifact(["README"], create_files=False)
+        artifact, _ = self.playground.create_artifact(
+            ["README"], create_files=False
+        )
 
         self.assertEqual(artifact.fileinartifact_set.all().count(), 0)
 
@@ -269,7 +250,7 @@ class TestCaseTests(TestCase):
             ValueError,
             "^skip_add_files_in_store must be False if create_files is False$",
         ):
-            self.create_artifact(
+            self.playground.create_artifact(
                 create_files=False, skip_add_files_in_store=True
             )
 
