@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -35,6 +35,8 @@ from django.forms import (
     MultipleChoiceField,
     Textarea,
 )
+from django.http import QueryDict
+from django.utils.safestring import mark_safe
 
 from debusine.db.context import context
 
@@ -56,6 +58,7 @@ from debusine.db.models import (
     Token,
     User,
     WorkRequest,
+    WorkflowTemplate,
     Workspace,
 )
 from debusine.tasks import BaseTask
@@ -478,15 +481,11 @@ class WorkflowFilterForm(Form):
         super().__init__(*args, **kwargs)
 
         workflow_templates_choices = [
-            (task_name, task_name)
-            for task_name in WorkRequest.objects.in_current_workspace()
+            (template_name, template_name)
+            for template_name in WorkflowTemplate.objects.in_current_workspace()
             .can_display(context.user)
-            .filter(workflow_data_json__workflow_template_name__isnull=False)
-            .order_by("workflow_data_json__workflow_template_name")
-            .distinct()
-            .values_list(
-                "workflow_data_json__workflow_template_name", flat=True
-            )
+            .order_by("name")
+            .values_list("name", flat=True)
         ]
 
         self.fields["workflow_templates"] = MultipleChoiceField(
@@ -494,8 +493,23 @@ class WorkflowFilterForm(Form):
             required=False,
         )
 
+        statuses = (
+            sorted(
+                [
+                    (value, label)
+                    for value, label in WorkRequest.Statuses.choices
+                    if value != WorkRequest.Statuses.RUNNING
+                ]
+            )
+            + [
+                (f"running__{value}", f"running__{label}")
+                for value, label in WorkRequest.RuntimeStatuses.choices
+            ]
+            + [("running__any", "running__Any")]
+        )
+
         self.fields["statuses"] = MultipleChoiceField(
-            choices=sorted(WorkRequest.Statuses.choices),
+            choices=statuses,
             required=False,
         )
 
@@ -533,3 +547,51 @@ class WorkflowFilterForm(Form):
         self.fields["with_failed_work_requests"] = BooleanField(
             required=False,
         )
+
+    def statuses_runtime_statuses(self) -> str:
+        """Render the statuses field as a <select> element."""
+        options_html = ""
+
+        statuses = cast(QueryDict, self.data).getlist("statuses")
+
+        for status_value, status_label in WorkRequest.Statuses.choices:
+            if status_value == WorkRequest.Statuses.RUNNING:
+                running_options = '<optgroup label="Running">'
+
+                for (
+                    runtime_value,
+                    runtime_label,
+                ) in WorkRequest.RuntimeStatuses.choices:
+                    value = f"running__{runtime_value}"
+                    running_options += f'<option value="{value}"'
+
+                    if value in statuses:
+                        running_options += ' selected'
+
+                    running_options += f'>{runtime_label}</option>\n'
+
+                # Add "Any" for any Running status.
+
+                running_options += '<option value="running__any"'
+
+                if "running__any" in statuses:
+                    running_options += ' selected'
+
+                running_options += '>Any</option>\n'
+
+                running_options += "</optgroup>\n"
+
+                options_html += running_options
+            else:
+                options_html += f'<option value="{status_value}"'
+                if str(status_value) in statuses:
+                    options_html += ' selected'
+                options_html += f'>{status_label}</option>\n'
+
+        return mark_safe(
+            f"""
+        <select name="statuses" id="id_statuses" multiple>\n
+            {options_html}
+        </select>
+        """
+        )
