@@ -1,4 +1,4 @@
-# Copyright 2023-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -17,7 +17,7 @@ app_name = "workspaces"
 
 workspace_urlpatterns: list[URLPattern | URLResolver] = [
     path(
-        "view/",
+        "",
         views.WorkspaceDetailView.as_view(),
         name="detail",
     ),
@@ -35,6 +35,13 @@ workspace_urlpatterns: list[URLPattern | URLResolver] = [
             namespace="workflows",
         ),
     ),
+    path(
+        "workflow-template/",
+        include(
+            "debusine.web.urls.workflow_templates",
+            namespace="workflow_templates",
+        ),
+    ),
     path(
         "artifact/",
         include(
