@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -68,7 +68,7 @@ class FileUI(NamedTuple):
         """Get a FileUI for a file."""
         # TODO: this is currently rather simple minded.
         # If it will need more complexity in the future,
-        # debusine.artfacts.models.ArtifactData can be extended with a
+        # debusine.artifacts.models.ArtifactData can be extended with a
         # get_content_type or get_file_ui method.
         # Another thing we may want to consider is adding a content_type
         # argument to File.
@@ -191,32 +191,35 @@ class TextFileWidget(FileWidget):
     def get_context_data(self) -> dict[str, Any]:
         """Add context data for text files."""
         context = super().get_context_data()
-        # Generate CSS with:
-        # python3 -m pygments -S github-dark -f html \
-        #     > debusine/web/static/web/css/debusine-code-highlight.css
-        # See https://pygments.org/styles/ for a list of styles
-        if self.file_ui.pygments_lexer:
-            lexer = pygments.lexers.get_lexer_by_name(
-                self.file_ui.pygments_lexer
-            )
-        else:
-            try:
-                lexer = pygments.lexers.get_lexer_for_mimetype(
-                    self.file_ui.content_type
+
+        if self.file_in_artifact.complete:
+            # Generate CSS with:
+            # python3 -m pygments -S github-dark -f html \
+            #     > debusine/web/static/web/css/debusine-code-highlight.css
+            # See https://pygments.org/styles/ for a list of styles
+            if self.file_ui.pygments_lexer:
+                lexer = pygments.lexers.get_lexer_by_name(
+                    self.file_ui.pygments_lexer
                 )
-            except pygments.util.ClassNotFound:
-                lexer = pygments.lexers.get_lexer_for_mimetype("text/plain")
+            else:
+                try:
+                    lexer = pygments.lexers.get_lexer_for_mimetype(
+                        self.file_ui.content_type
+                    )
+                except pygments.util.ClassNotFound:
+                    lexer = pygments.lexers.get_lexer_for_mimetype("text/plain")
 
-        formatter = LinenoHtmlFormatter(
-            cssclass="file_highlighted",
-            linenos=False,
-        )
+            formatter = LinenoHtmlFormatter(
+                cssclass="file_highlighted",
+                linenos=False,
+            )
 
-        with self._open_file() as stream:
-            formatted = pygments.highlight(stream.read(), lexer, formatter)
+            with self._open_file() as stream:
+                formatted = pygments.highlight(stream.read(), lexer, formatter)
+
+            context["file_linenumbers"] = SafeString(formatter.render_linenos())
+            context["file_content"] = SafeString(formatted)
 
-        context["file_linenumbers"] = SafeString(formatter.render_linenos())
-        context["file_content"] = SafeString(formatted)
         return context
 
 
@@ -248,6 +251,12 @@ class FileDownloadMixin(View):
         download: bool = True,
     ) -> HttpResponseBase:
         """Return a response that streams the_given file."""
+        if not file_in_artifact.complete:
+            return ProblemResponse(
+                "Cannot download incomplete file",
+                status_code=status.HTTP_404_NOT_FOUND,
+            )
+
         try:
             content_range = parse_range_header(self.request.headers)
         except ValueError as exc:
