@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -153,7 +153,7 @@ class LintianView(WorkRequestPlugin):
         for artifact in artifacts:
             try:
                 analysis_in_artifact = artifact.fileinartifact_set.get(
-                    path="analysis.json"
+                    path="analysis.json", complete=True
                 )
             except FileInArtifact.DoesNotExist:
                 continue
@@ -176,7 +176,9 @@ class LintianView(WorkRequestPlugin):
         The path is for any of the lintian.txt found in artifacts.
         """
         for artifact in artifacts:
-            for file_in_artifact in artifact.fileinartifact_set.order_by("id"):
+            for file_in_artifact in artifact.fileinartifact_set.filter(
+                complete=True
+            ).order_by("id"):
                 if file_in_artifact.path == Lintian.CAPTURE_OUTPUT_FILENAME:
                     return (
                         artifact.get_absolute_url_download()
@@ -205,7 +207,9 @@ class LintianView(WorkRequestPlugin):
         if artifact is not None:
             return [
                 file.path
-                for file in artifact.fileinartifact_set.order_by("path")
+                for file in artifact.fileinartifact_set.filter(
+                    complete=True
+                ).order_by("path")
             ]
         else:
             return []
