@@ -1,4 +1,4 @@
-# Copyright 2022-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -209,6 +209,26 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
         else:
             self.assertEqual(sidebar[5].value, "never")
 
+    def assertFile(
+        self, tree: lxml.objectify.ObjectifiedElement, artifact: Artifact
+    ) -> None:
+        """Ensure that the single file in the artifact is shown."""
+        file = artifact.fileinartifact_set.select_related("file").get()
+        file_contents = self.assertHasElement(
+            tree, "//div[@id='file-contents']"
+        )
+
+        if file.complete:
+            self.assertTextContentEqual(file_contents.div[0], file.path)
+            card_body_class = file_contents.div[1].get("class")
+            assert card_body_class is not None
+            self.assertIn("card-body", card_body_class.split())
+        else:
+            self.assertTextContentEqual(
+                file_contents.div[0], f"{file.path} (incomplete)"
+            )
+            self.assertEqual(len(file_contents.div), 1)
+
     def assertFileList(
         self, tree: lxml.objectify.ObjectifiedElement, artifact: Artifact
     ) -> None:
@@ -220,12 +240,17 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
 
         for tr, file in zip(table.tbody.tr, files):
             with self.subTest(file.path):
-                self.assertEqual(
-                    tr.td[0].a.get("href"), file.get_absolute_url()
-                )
-                self.assertTextContentEqual(
-                    tr.td[0].a, os.path.basename(file.path)
-                )
+                basename = os.path.basename(file.path)
+                if file.complete:
+                    self.assertEqual(
+                        tr.td[0].a.get("href"), file.get_absolute_url()
+                    )
+                    self.assertTextContentEqual(tr.td[0].a, basename)
+                else:
+                    self.assertEqual(tr.td[0].countchildren(), 0)
+                    self.assertTextContentEqual(
+                        tr.td[0], f"{basename} (incomplete)"
+                    )
                 self.assertEqual(tr.td[1].get("title"), str(file.file.size))
 
     def assertRelations(
@@ -250,12 +275,30 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
                 self.assertEqual(
                     tr.td[1].i.attrib["class"], "bi bi-arrow-right"
                 )
-                self.assertEqual(tr.td[3].a.text, relation.target.get_label())
+                label = relation.target.get_label()
+                self.assertEqual(tr.td[3].a.text, label)
+                if relation.target.fileinartifact_set.filter(
+                    complete=False
+                ).exists():
+                    self.assertTextContentEqual(
+                        tr.td[3], f"{label} (incomplete)"
+                    )
+                else:
+                    self.assertTextContentEqual(tr.td[3], label)
         for tr, relation in zip(table.tbody.tr[len(relations) :], targeted_by):
             with self.subTest(str(tr)):
                 self.assertEqual(tr.td[0].text, relation.type)
                 self.assertEqual(tr.td[1].i.attrib["class"], "bi bi-arrow-left")
-                self.assertEqual(tr.td[3].a.text, relation.artifact.get_label())
+                label = relation.artifact.get_label()
+                self.assertEqual(tr.td[3].a.text, label)
+                if relation.artifact.fileinartifact_set.filter(
+                    complete=False
+                ).exists():
+                    self.assertTextContentEqual(
+                        tr.td[3], f"{label} (incomplete)"
+                    )
+                else:
+                    self.assertTextContentEqual(tr.td[3], label)
 
     def test_permissions(self) -> None:
         """Test basic permission enforcement."""
@@ -293,6 +336,19 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
         self.assertRelations(tree, self.artifact)
         self.assertFalse(tree.xpath("//div[@id='file-contents']"))
 
+    def test_get_success_html_list_incomplete(self) -> None:
+        """View shows incomplete files in a list differently."""
+        self.artifact.fileinartifact_set.filter(path="AUTHORS").update(
+            complete=False
+        )
+        response = self._get()
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertArtifactSidebarMatches(response, self.artifact)
+        tree = self.assertHTMLValid(response)
+        self.assertFileList(tree, self.artifact)
+        self.assertRelations(tree, self.artifact)
+        self.assertFalse(tree.xpath("//div[@id='file-contents']"))
+
     def test_get_success_html_singlefile(self) -> None:
         """View show the content of the only file in the artifact."""
         with context.disable_permission_checks():
@@ -302,12 +358,23 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
         self.assertArtifactSidebarMatches(response, self.artifact)
         tree = self.assertHTMLValid(response)
         self.assertFalse(tree.xpath("//table[@id='file-list']"))
-        self.assertHasElement(tree, "//div[@id='file-contents']")
+        self.assertFile(tree, self.artifact)
+
+    def test_get_success_html_singlefile_incomplete(self) -> None:
+        """View shows a single incomplete file in the artifact differently."""
+        self.artifact.fileinartifact_set.filter(path="AUTHORS").delete()
+        self.artifact.fileinartifact_set.update(complete=False)
+        response = self._get()
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertArtifactSidebarMatches(response, self.artifact)
+        tree = self.assertHTMLValid(response)
+        self.assertFalse(tree.xpath("//table[@id='file-list']"))
+        self.assertFile(tree, self.artifact)
 
     def test_get_success_html_empty_artifact(self) -> None:
         """Test HTML output if there are no files in the artifact."""
         with context.disable_permission_checks():
-            artifact, _ = self.create_artifact([])
+            artifact, _ = self.playground.create_artifact([])
         response = self._get(artifact.id)
         tree = self.assertHTMLValid(response)
         self.assertFalse(tree.xpath("//table[@id='file-list']"))
@@ -338,6 +405,16 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
         tree = self.assertHTMLValid(response)
         self.assertRelations(tree, self.artifact)
 
+    def test_get_success_html_fwd_relations_incomplete(self) -> None:
+        """View shows whether forward-related artifacts are incomplete."""
+        other_artifact = self.playground.create_build_log_artifact(
+            skip_add_files_in_store=True
+        )
+        self.playground.create_artifact_relation(self.artifact, other_artifact)
+        response = self._get()
+        tree = self.assertHTMLValid(response)
+        self.assertRelations(tree, self.artifact)
+
     def test_get_success_html_rev_relations(self) -> None:
         """View shows a list of reverse relations."""
         with context.disable_permission_checks():
@@ -354,6 +431,21 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
         tree = self.assertHTMLValid(response)
         self.assertRelations(tree, self.artifact)
 
+    def test_get_success_html_rev_relations_incomplete(self) -> None:
+        """View shows whether reverse-related artifacts are incomplete."""
+        other_artifact = self.playground.create_build_log_artifact(
+            skip_add_files_in_store=True
+        )
+        self.playground.create_artifact_relation(other_artifact, self.artifact)
+        self.playground.create_artifact_relation(
+            other_artifact,
+            self.artifact,
+            relation_type=ArtifactRelation.Relations.EXTENDS,
+        )
+        response = self._get()
+        tree = self.assertHTMLValid(response)
+        self.assertRelations(tree, self.artifact)
+
     def test_ui_shortcuts_source(self) -> None:
         """Check that UI shortcuts for source packages are as expected."""
         artifact = self.playground.create_source_artifact()
@@ -596,6 +688,30 @@ class ArtifactDetailFileViewTests(ViewTestMixin, PermissionTests):
             ],
         )
         self.assertIn("file", response.context)
+        tree = self.assertHTMLValid(response)
+        title = self.assertHasElement(tree, "head/title")
+        self.assertTextContentEqual(title, f"Debusine - {self.file.path}")
+
+    def test_get_incomplete(self) -> None:
+        """Test a get of an incomplete file."""
+        self.file.complete = False
+        self.file.save()
+        response = self.permission_tests_get(include_token=True)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertEqual(
+            response.context["main_ui_shortcuts"],
+            [
+                ui_shortcuts.create_file_view_raw(self.file),
+                ui_shortcuts.create_file_download(self.file),
+                ui_shortcuts.create_artifact_view(self.artifact),
+            ],
+        )
+        self.assertIn("file", response.context)
+        tree = self.assertHTMLValid(response)
+        title = self.assertHasElement(tree, "head/title")
+        self.assertTextContentEqual(
+            title, f"Debusine - {self.file.path} (incomplete)"
+        )
 
 
 class ArtifactDetailFileViewRawTests(ViewTestMixin, PermissionTests):
@@ -975,6 +1091,20 @@ class DownloadPathViewTests(ViewTestMixin, PermissionTests):
             html=True,
         )
 
+    def test_get_file_incomplete(self) -> None:
+        """Get returns 404 for an incomplete file."""
+        self.artifact.fileinartifact_set.update(complete=False)
+
+        response = self.get_file()
+
+        self.assertContains(
+            response,
+            f'Artifact {self.artifact.id} does not have any file or directory '
+            f'for "{self.path_in_artifact}"',
+            status_code=status.HTTP_404_NOT_FOUND,
+            html=True,
+        )
+
     def test_get_subdirectory_does_not_exist_404(self) -> None:
         """View return HTTP 404 Not Found: no files in the subdirectory."""
         subdirectory = "does-not-exist"
@@ -988,6 +1118,23 @@ class DownloadPathViewTests(ViewTestMixin, PermissionTests):
             html=True,
         )
 
+    def test_get_subdirectory_entirely_incomplete(self) -> None:
+        """View returns 404 if all files in the subdirectory are incomplete."""
+        self.tree.fileinartifact_set.filter(path__startswith="src/lib/").update(
+            complete=False
+        )
+
+        subdirectory = "src/lib"
+        response = self.get_artifact(self.tree, "tar.gz", subdirectory)
+
+        self.assertContains(
+            response,
+            f'Artifact {self.tree.id} does not have any file or '
+            f'directory for "{subdirectory}"',
+            status_code=status.HTTP_404_NOT_FOUND,
+            html=True,
+        )
+
     def test_get_subdirectory_only_tar_gz(self) -> None:
         """View return tar.gz file with the files from a subdirectory."""
         subdirectory = "src/lib"
@@ -1008,6 +1155,35 @@ class DownloadPathViewTests(ViewTestMixin, PermissionTests):
         )
         self.assertEqual(tar.getnames(), expected_files)
 
+    def test_get_subdirectory_only_excludes_incomplete(self) -> None:
+        """Downloading a subdirectory excludes incomplete files."""
+        excluded_file = self.tree.fileinartifact_set.get(path="src/lib/utils.c")
+        excluded_file.complete = False
+        excluded_file.save()
+
+        subdirectory = "src/lib"
+        response = self.get_artifact(self.tree, "tar.gz", subdirectory)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertEqual(
+            response.headers["Content-Disposition"],
+            f'attachment; filename="artifact-{self.tree.id}-src_lib.tar.gz"',
+        )
+        response_content = io.BytesIO(
+            b"".join(getattr(response, "streaming_content"))
+        )
+
+        tar = tarfile.open(fileobj=response_content, mode="r:gz")
+
+        expected_files = [
+            path
+            for path in self.tree_paths
+            if (
+                path.startswith(subdirectory + "/")
+                and path != excluded_file.path
+            )
+        ]
+        self.assertEqual(tar.getnames(), expected_files)
+
     def test_get_unsupported_archive_parameter(self) -> None:
         """View return HTTP 400 Bad Request: unsupported archive parameter."""
         archive_format = "tar.xz"
@@ -1035,6 +1211,20 @@ class DownloadPathViewTests(ViewTestMixin, PermissionTests):
             response.headers["content-type"], "text/markdown; charset=utf-8"
         )
 
+    def test_get_artifact_auto_only_one_complete_file(self) -> None:
+        """Auto download format returns file if only one file is complete."""
+        FileInArtifact.objects.create(
+            artifact=self.artifact,
+            path="incomplete",
+            file=self.playground.create_file(b"incomplete"),
+            complete=False,
+        )
+        response = self.get_artifact(self.artifact, archive=None)
+        self.assertFileResponse(response, status.HTTP_200_OK)
+        self.assertEqual(
+            response.headers["content-type"], "text/markdown; charset=utf-8"
+        )
+
     def test_get_artifact_auto_tree(self) -> None:
         """Check downloading whole artifact with auto download format."""
         response = self.get_artifact(self.tree, archive=None)
