@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -84,7 +84,7 @@ class AutopkgtestViewTests(TestCase):
         self, work_request: WorkRequest
     ) -> Artifact:
         """Create an AutopkgtestArtifact for a work request."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             paths=["log", "summary"],
             work_request=work_request,
             create_files=True,
@@ -102,7 +102,7 @@ class AutopkgtestViewTests(TestCase):
     def test_get_context_data_simple(self) -> None:
         """get_context_data returns the expected data for a simple case."""
         with context.disable_permission_checks():
-            environment_artifact, _ = self.create_artifact(
+            environment_artifact, _ = self.playground.create_artifact(
                 paths=["system.tar.xz"],
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 data={
@@ -113,16 +113,16 @@ class AutopkgtestViewTests(TestCase):
                 },
                 create_files=True,
             )
-            source_artifact, _ = self.create_artifact(
+            source_artifact, _ = self.playground.create_artifact(
                 paths=["hello.dsc"],
                 category=ArtifactCategory.SOURCE_PACKAGE,
                 data={"name": "hello", "version": "1.0", "type": "dpkg"},
                 create_files=True,
             )
-            binary_artifact_1, _ = self.create_artifact(
+            binary_artifact_1, _ = self.playground.create_artifact(
                 paths=["hello-1.deb"], create_files=True
             )
-            binary_artifact_2, _ = self.create_artifact(
+            binary_artifact_2, _ = self.playground.create_artifact(
                 paths=["hello-2.deb"], create_files=True
             )
             work_request: WorkRequest = self.create_work_request_for_artifacts(
@@ -167,7 +167,7 @@ class AutopkgtestViewTests(TestCase):
                     "context_artifacts": {"lookup": [], "artifacts": []},
                     "debug_level": 0,
                     "exclude_tests": [],
-                    "extra_apt_sources": [],
+                    "extra_repositories": None,
                     "extra_environment": {},
                     "fail_on_scenarios": ["failed"],
                     "host_architecture": "amd64",
@@ -198,7 +198,7 @@ class AutopkgtestViewTests(TestCase):
     def test_get_context_data_with_context_artifacts(self) -> None:
         """Any context artifacts show up in the context data."""
         with context.disable_permission_checks():
-            environment_artifact, _ = self.create_artifact(
+            environment_artifact, _ = self.playground.create_artifact(
                 paths=["system.tar.xz"],
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 data={
@@ -209,16 +209,16 @@ class AutopkgtestViewTests(TestCase):
                 },
                 create_files=True,
             )
-            source_artifact, _ = self.create_artifact(
+            source_artifact, _ = self.playground.create_artifact(
                 paths=["hello.dsc"],
                 category=ArtifactCategory.SOURCE_PACKAGE,
                 data={"name": "hello", "version": "1.0", "type": "dpkg"},
                 create_files=True,
             )
-            context_artifact_1, _ = self.create_artifact(
+            context_artifact_1, _ = self.playground.create_artifact(
                 paths=["hello-rdep-1.deb"], create_files=True
             )
-            context_artifact_2, _ = self.create_artifact(
+            context_artifact_2, _ = self.playground.create_artifact(
                 paths=["hello-rdep-2.deb"], create_files=True
             )
             work_request: WorkRequest = self.create_work_request_for_artifacts(
@@ -260,7 +260,7 @@ class AutopkgtestViewTests(TestCase):
     def test_context_data_no_artifacts(self) -> None:
         """If there are no output artifacts, context data omits that info."""
         with context.disable_permission_checks():
-            environment_artifact, _ = self.create_artifact(
+            environment_artifact, _ = self.playground.create_artifact(
                 paths=["system.tar.xz"],
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 data={
@@ -271,7 +271,7 @@ class AutopkgtestViewTests(TestCase):
                 },
                 create_files=True,
             )
-            source_artifact, _ = self.create_artifact(
+            source_artifact, _ = self.playground.create_artifact(
                 paths=["hello.dsc"],
                 category=ArtifactCategory.SOURCE_PACKAGE,
                 data={"name": "hello", "version": "1.0", "type": "dpkg"},
@@ -287,10 +287,10 @@ class AutopkgtestViewTests(TestCase):
     def test_context_data_no_dynamic_task_data(self) -> None:
         """If there is no dynamic data, context data omits artifact info."""
         with context.disable_permission_checks():
-            environment_artifact, _ = self.create_artifact(
+            environment_artifact, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL
             )
-            source_artifact, _ = self.create_artifact(
+            source_artifact, _ = self.playground.create_artifact(
                 paths=["hello.dsc"],
                 category=ArtifactCategory.SOURCE_PACKAGE,
                 data={"name": "hello", "version": "1.0", "type": "dpkg"},
@@ -320,10 +320,10 @@ class AutopkgtestViewTests(TestCase):
     def test_context_data_missing_environment_artifact(self) -> None:
         """If the environment artifact is missing, context data omits it."""
         with context.disable_permission_checks():
-            environment_artifact, _ = self.create_artifact(
+            environment_artifact, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL
             )
-            source_artifact, _ = self.create_artifact(
+            source_artifact, _ = self.playground.create_artifact(
                 paths=["hello.dsc"],
                 category=ArtifactCategory.SOURCE_PACKAGE,
                 data={"name": "hello", "version": "1.0", "type": "dpkg"},
@@ -350,12 +350,12 @@ class AutopkgtestViewTests(TestCase):
     def test_context_data_environment_artifact_wrong_category(self) -> None:
         """We cope with the environment artifact having an odd category."""
         with context.disable_permission_checks():
-            environment_artifact, _ = self.create_artifact(
+            environment_artifact, _ = self.playground.create_artifact(
                 paths=["system.tar.xz"],
                 category=ArtifactCategory.TEST,
                 create_files=True,
             )
-            source_artifact, _ = self.create_artifact(
+            source_artifact, _ = self.playground.create_artifact(
                 paths=["hello.dsc"],
                 category=ArtifactCategory.SOURCE_PACKAGE,
                 data={"name": "hello", "version": "1.0", "type": "dpkg"},
@@ -381,10 +381,10 @@ class AutopkgtestViewTests(TestCase):
     def test_context_data_missing_source_artifact(self) -> None:
         """If the source artifact is missing, context data omits its files."""
         with context.disable_permission_checks():
-            environment_artifact, _ = self.create_artifact(
+            environment_artifact, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL
             )
-            source_artifact, _ = self.create_artifact(
+            source_artifact, _ = self.playground.create_artifact(
                 paths=["hello.dsc"],
                 category=ArtifactCategory.SOURCE_PACKAGE,
                 data={"name": "hello", "version": "1.0", "type": "dpkg"},
@@ -417,7 +417,7 @@ class AutopkgtestViewTests(TestCase):
     def test_context_data_source_artifact_wrong_category(self) -> None:
         """We cope with the source artifact having an odd category."""
         with context.disable_permission_checks():
-            environment_artifact, _ = self.create_artifact(
+            environment_artifact, _ = self.playground.create_artifact(
                 paths=["system.tar.xz"],
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 data={
@@ -428,7 +428,7 @@ class AutopkgtestViewTests(TestCase):
                 },
                 create_files=True,
             )
-            source_artifact, _ = self.create_artifact(
+            source_artifact, _ = self.playground.create_artifact(
                 paths=["hello.dsc"],
                 category=ArtifactCategory.TEST,
                 create_files=True,
@@ -454,19 +454,19 @@ class AutopkgtestViewTests(TestCase):
     def test_context_data_missing_binary_artifact(self) -> None:
         """If a binary artifact is missing, context data omits its files."""
         with context.disable_permission_checks():
-            environment_artifact, _ = self.create_artifact(
+            environment_artifact, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL
             )
-            source_artifact, _ = self.create_artifact(
+            source_artifact, _ = self.playground.create_artifact(
                 paths=["hello.dsc"],
                 category=ArtifactCategory.SOURCE_PACKAGE,
                 data={"name": "hello", "version": "1.0", "type": "dpkg"},
                 create_files=True,
             )
-            binary_artifact_1, _ = self.create_artifact(
+            binary_artifact_1, _ = self.playground.create_artifact(
                 paths=["hello-1.deb"], create_files=True
             )
-            binary_artifact_2, _ = self.create_artifact(
+            binary_artifact_2, _ = self.playground.create_artifact(
                 paths=["hello-2.deb"], create_files=True
             )
             work_request: WorkRequest = self.create_work_request_for_artifacts(
@@ -505,6 +505,35 @@ class AutopkgtestViewTests(TestCase):
             },
         )
 
+    def test_get_context_data_skips_incomplete_files(self) -> None:
+        """get_context_data() skips incomplete files."""
+        environment_artifact, _ = self.playground.create_artifact(
+            category=ArtifactCategory.SYSTEM_TARBALL
+        )
+        source_artifact, _ = self.playground.create_artifact(
+            paths=["hello.dsc"],
+            category=ArtifactCategory.SOURCE_PACKAGE,
+            data={"name": "hello", "version": "1.0", "type": "dpkg"},
+            create_files=True,
+            skip_add_files_in_store=True,
+        )
+        work_request: WorkRequest = self.create_work_request_for_artifacts(
+            environment_artifact, source_artifact, []
+        )
+        work_request.dynamic_task_data = AutopkgtestDynamicData(
+            environment_id=environment_artifact.id,
+            input_source_artifact_id=source_artifact.id,
+            input_binary_artifacts_ids=[],
+        ).dict()
+        work_request.save()
+
+        autopkgtest_view = AutopkgtestView(work_request)
+        context_data = autopkgtest_view.get_context_data()
+
+        self.assertEqual(
+            context_data["request_data"]["source_artifact"]["files"], []
+        )
+
     def assert_contains_artifact_information(
         self,
         response: HttpResponseBase,
@@ -557,7 +586,7 @@ class AutopkgtestViewTests(TestCase):
             workspace = self.playground.create_workspace(
                 name="Public", public=True
             )
-            environment_artifact, _ = self.create_artifact(
+            environment_artifact, _ = self.playground.create_artifact(
                 paths=["system.tar.xz"],
                 category=ArtifactCategory.SYSTEM_TARBALL,
                 data={
@@ -569,19 +598,19 @@ class AutopkgtestViewTests(TestCase):
                 create_files=True,
                 workspace=workspace,
             )
-            source_artifact, _ = self.create_artifact(
+            source_artifact, _ = self.playground.create_artifact(
                 paths=["hello.dsc"],
                 category=ArtifactCategory.SOURCE_PACKAGE,
                 data={"name": "hello", "version": "1.0", "type": "dpkg"},
                 create_files=True,
                 workspace=workspace,
             )
-            binary_artifact_1, _ = self.create_artifact(
+            binary_artifact_1, _ = self.playground.create_artifact(
                 paths=["hello-1.deb"],
                 create_files=True,
                 workspace=workspace,
             )
-            binary_artifact_2, _ = self.create_artifact(
+            binary_artifact_2, _ = self.playground.create_artifact(
                 paths=["hello-2.deb"],
                 create_files=True,
                 workspace=workspace,
