@@ -1,4 +1,4 @@
-# Copyright 2022-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -280,6 +280,17 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
             "application/octet-stream",
         )
 
+    def test_get_incomplete_file(self) -> None:
+        """Get returns 404 for an incomplete file."""
+        self.file.complete = False
+        self.file.save()
+        response = self.get_stream_response(self.file)
+        self.assertResponseProblem(
+            response,
+            "Cannot download incomplete file",
+            status_code=status.HTTP_404_NOT_FOUND,
+        )
+
     def test_get_file_range(self) -> None:
         """Get return part of the file (based on Range header)."""
         start, end = 10, 20
@@ -414,6 +425,24 @@ class FileWidgetTests(ViewTestMixin, TestCase):
             },
         )
 
+    def test_context_data_text_incomplete(self) -> None:
+        """Test context for incomplete text files."""
+        file = self.text
+        file.complete = False
+        file.save()
+        context = FileWidget.create(file).get_context_data()
+
+        self.assertNotIn("file_linenumbers", context)
+        self.assertNotIn("file_content", context)
+
+        self.assertEqual(
+            context,
+            {
+                "file_in_artifact": file,
+                "file_ui": FileUI.from_file_in_artifact(file),
+            },
+        )
+
     def test_context_data_dsc(self) -> None:
         """Test context for text files."""
         file = self.dsc
