@@ -1,4 +1,4 @@
-# Copyright 2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -54,17 +54,17 @@ class LintianViewTests(TestCase):
             name="Public", public=True
         )
 
-        source_artifact, _ = cls.create_artifact(
+        source_artifact, _ = cls.playground.create_artifact(
             paths=["hello.dsc"],
             create_files=True,
             workspace=cls.workspace,
         )
-        binary_artifact_1, _ = cls.create_artifact(
+        binary_artifact_1, _ = cls.playground.create_artifact(
             paths=["hello-1.deb"],
             create_files=True,
             workspace=cls.workspace,
         )
-        binary_artifact_2, _ = cls.create_artifact(
+        binary_artifact_2, _ = cls.playground.create_artifact(
             paths=["hello-2.deb"],
             create_files=True,
             workspace=cls.workspace,
@@ -97,7 +97,7 @@ class LintianViewTests(TestCase):
                 }
             }
         }
-        cls.binary_artifact, _ = cls.create_artifact(
+        cls.binary_artifact, _ = cls.playground.create_artifact(
             paths=["summary.json", "lintian.txt"],
             create_files=True,
             workspace=cls.workspace,
@@ -110,7 +110,7 @@ class LintianViewTests(TestCase):
         summary_source_package = {
             "summary": {"package_filename": {"hello": cls.package_source_file}}
         }
-        cls.source_artifact, _ = cls.create_artifact(
+        cls.source_artifact, _ = cls.playground.create_artifact(
             paths=["lintian.txt"],
             create_files=True,
             workspace=cls.workspace,
@@ -120,7 +120,7 @@ class LintianViewTests(TestCase):
         cls.source_artifact.created_by_work_request = cls.work_request
         cls.source_artifact.save()
 
-        build_log_artifact, _ = cls.create_artifact(
+        build_log_artifact, _ = cls.playground.create_artifact(
             paths=["some-file.log"],
             create_files=True,
             workspace=cls.workspace,
@@ -129,7 +129,11 @@ class LintianViewTests(TestCase):
         build_log_artifact.save()
 
     def add_file_in_artifact(
-        self, artifact: Artifact, path: str, contents: bytes
+        self,
+        artifact: Artifact,
+        path: str,
+        contents: bytes,
+        complete: bool = True,
     ) -> None:
         """
         Add file in the artifact.
@@ -142,9 +146,14 @@ class LintianViewTests(TestCase):
         file_backend = (
             artifact.workspace.default_file_store.get_backend_object()
         )
-        fileobj = self.create_file_in_backend(file_backend, contents)
+        if complete:
+            fileobj = self.playground.create_file_in_backend(
+                file_backend, contents
+            )
+        else:
+            fileobj = self.playground.create_file(contents)
         FileInArtifact.objects.create(
-            artifact=artifact, path=path, file=fileobj, complete=True
+            artifact=artifact, path=path, file=fileobj, complete=complete
         )
 
     def create_analysis(self) -> Path:
@@ -431,6 +440,23 @@ class LintianViewTests(TestCase):
             },
         )
 
+    def test_get_context_data_skips_incomplete_files(self) -> None:
+        """get_context_data() skips incomplete files."""
+        self.source_artifact.fileinartifact_set.update(complete=False)
+        self.binary_artifact.fileinartifact_set.update(complete=False)
+        self.add_file_in_artifact(
+            self.binary_artifact,
+            "analysis.json",
+            self.create_analysis().read_bytes(),
+            complete=False,
+        )
+
+        lintian_view = LintianView(self.work_request)
+        context_data = lintian_view.get_context_data()
+
+        self.assertIsNone(context_data["lintian_txt_path"])
+        self.assertEqual(context_data["tags"], {})
+
     def test_template_output(self) -> None:
         """Generic output of the template."""
         task_data_input = self.work_request.task_data["input"]
@@ -613,7 +639,7 @@ class LintianViewTests(TestCase):
     def test_find_lintian_txt_url_path_artifact_no_files(self) -> None:
         """_find_lintian_txt_url_path(), artifact no files: return None."""
         with context.disable_permission_checks():
-            artifact, _ = self.create_artifact()
+            artifact, _ = self.playground.create_artifact()
 
         self.assertIsNone(LintianView._find_lintian_txt_url_path([artifact]))
 
