@@ -1,4 +1,4 @@
-# Copyright 2022-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -39,10 +39,6 @@ class HomepageViewTests(ViewTestMixin, TestCase):
 
     scenario = scenarios.DefaultScopeUser()
 
-    list_your_tokens_url = reverse("user:token-list")
-    list_your_tokens_html = (
-        f'<a class="dropdown-item" href="{list_your_tokens_url}">Tokens</a>'
-    )
     create_work_request_url = reverse(
         "workspaces:work-requests:create", kwargs={"wname": "System"}
     )
@@ -68,6 +64,13 @@ class HomepageViewTests(ViewTestMixin, TestCase):
 
     def test_homepage(self) -> None:
         """Homepage view loads."""
+        list_your_tokens_url = reverse(
+            "user:token-list", kwargs={"username": self.scenario.user.username}
+        )
+        list_your_tokens_html = (
+            f'<a class="dropdown-item" href="{list_your_tokens_url}">Tokens</a>'
+        )
+
         response = self.client.get(reverse("homepage:homepage"))
         self.assertEqual(response.status_code, status.HTTP_200_OK)
         tree = self.assertHTMLValid(response)
@@ -77,7 +80,7 @@ class HomepageViewTests(ViewTestMixin, TestCase):
         self.assertNavCommonElements(tree, is_homepage=True)
         self.assertNavNoUser(tree)
 
-        self.assertNotContains(response, self.list_your_tokens_html, html=True)
+        self.assertNotContains(response, list_your_tokens_html, html=True)
         self.assertFalse(tree.xpath("//a[@id='nav-create-artifact']"))
         self.assertFalse(tree.xpath("//a[@id='nav-create-work-request']"))
 
