@@ -1,4 +1,4 @@
-# Copyright 2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -14,6 +14,7 @@ from datetime import timedelta
 from types import GeneratorType
 from typing import ClassVar
 
+import lxml
 import yaml
 from django.contrib.auth import get_user_model
 from django.contrib.messages import constants as messages_constants
@@ -52,6 +53,7 @@ from debusine.tasks.models import (
     TaskTypes,
 )
 from debusine.test.django import AllowAll, TestCase, override_permission
+from debusine.web.templatetags.artifacts import artifact_category_label
 from debusine.web.views import ui_shortcuts
 from debusine.web.views.lintian import LintianView
 from debusine.web.views.tests.test_views import _sort_table_handle
@@ -108,10 +110,42 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
             self.work_request.get_absolute_url(),
         )
 
+    def assertArtifacts(
+        self,
+        tree: lxml.objectify.ObjectifiedElement,
+        input_artifacts: list[Artifact],
+        output_artifacts: list[Artifact],
+    ) -> None:
+        """Ensure that the given artifacts are listed."""
+        table = tree.xpath("//table[@id='artifacts']")[0]
+        for tbody, title, artifacts in (
+            (table.tbody[0], "Input artifacts", input_artifacts),
+            (table.tbody[1], "Output artifacts", output_artifacts),
+        ):
+            self.assertEqual(tbody.tr.th.get("title"), title)
+            for tr, artifact in zip(tbody.tr[1:], artifacts):
+                with self.subTest(str(tr)):
+                    self.assertTextContentEqual(
+                        tr.td[0], artifact_category_label(artifact).capitalize()
+                    )
+                    self.assertEqual(
+                        tr.td[1].a.get("href"), artifact.get_absolute_url()
+                    )
+                    label = artifact.get_label()
+                    self.assertTextContentEqual(tr.td[1].a, label)
+                    if artifact.fileinartifact_set.filter(
+                        complete=False
+                    ).exists():
+                        self.assertTextContentEqual(
+                            tr.td[1], f"{label} (incomplete)"
+                        )
+                    else:
+                        self.assertTextContentEqual(tr.td[1], label)
+
     def test_get_work_request(self) -> None:
         """View detail return work request information."""
         with context.disable_permission_checks():
-            artifact, _ = self.create_artifact()
+            artifact, _ = self.playground.create_artifact()
             artifact.created_by_work_request = self.work_request
             artifact.save()
 
@@ -141,18 +175,29 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
         self.assertEqual(sidebar[9].content, "never")
 
         # Check artifact list
-        tbody = tree.xpath("//table[@id='artifacts']")[0].tbody[1]
-        self.assertEqual(len(tbody.tr), 2)
-        tr = tbody.tr[1]
-        self.assertTextContentEqual(tr.td[0], "Artifact")
-        self.assertEqual(tr.td[1].a.get("href"), artifact.get_absolute_url())
-        self.assertTextContentEqual(tr.td[1].a, "debusine:test")
+        self.assertArtifacts(tree, [], [artifact])
 
         # Not in a workflow
         self.assertNotContains(
             response, "<h2>Workflow information</h2>", html=True
         )
 
+    def test_get_work_request_incomplete(self) -> None:
+        """Artifacts with incomplete files are marked as incomplete."""
+        artifact, _ = self.playground.create_artifact(
+            paths=["incomplete"],
+            create_files=True,
+            skip_add_files_in_store=True,
+        )
+        artifact.created_by_work_request = self.work_request
+        artifact.save()
+
+        response = self.client.get(self.work_request.get_absolute_url())
+
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertHTMLValid(response)
+        self.assertArtifacts(tree, [], [artifact])
+
     def test_get_work_request_private_workspace_unauthenticated(self) -> None:
         """Work requests in private workspaces 403 to the unauthenticated."""
         private_workspace = self.playground.create_workspace(name="Private")
@@ -299,7 +344,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
 
         Contains lintian.txt file.
         """
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             paths=[Lintian.CAPTURE_OUTPUT_FILENAME],
             create_files=True,
             category=LintianArtifact._category,
