@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -15,7 +15,7 @@ from rest_framework import status
 
 from debusine.artifacts.models import CollectionCategory
 from debusine.db.context import context
-from debusine.db.models import Collection, Scope, Workspace
+from debusine.db.models import Collection, Scope, WorkRequest, Workspace
 from debusine.db.playground import scenarios
 from debusine.test.django import (
     AllowAll,
@@ -138,17 +138,55 @@ class WorkspaceViewsTests(ViewTestMixin, TestCase):
 
     def test_detail_workflowtemplate(self) -> None:
         """Test showing a workspace with one workflow templates."""
-        self.playground.create_workflow_template(
-            name="Build package",
-            task_name="sbuild",
-            task_data={"prefix": "test"},
+        template = self.playground.create_workflow_template(
+            name="noop-template",
+            task_name="noop",
             workspace=self.workspace_public,
         )
+
+        # 3 completed workflow
+        for i in range(3):
+            self.playground.create_workflow(
+                task_name="noop",
+                status=WorkRequest.Statuses.COMPLETED,
+            )
+
+        # 2 running workflows
+        for i in range(2):
+            self.playground.create_workflow(
+                task_name="noop",
+                status=WorkRequest.Statuses.RUNNING,
+            )
+
+        # 1 needs input workflow
+        workflow_needs_input = self.playground.create_workflow(
+            task_name="noop",
+            status=WorkRequest.Statuses.RUNNING,
+        )
+        workflow_needs_input.workflow_runtime_status = (
+            WorkRequest.RuntimeStatuses.NEEDS_INPUT
+        )
+        workflow_needs_input.save()
+
         response = self.client.get(self.workspace_public.get_absolute_url())
+
+        self.assertTrue(response.context["workflow_templates"].ordered)
+
         self.assertEqual(response.status_code, status.HTTP_200_OK)
         tree = self.assertHTMLValid(response)
         div = self.assertHasElement(tree, "//div[@id='workflow-templates']")
-        self.assertTextContentEqual(
-            div.div.div[0], "Build package (type sbuild, priority 0)"
+
+        workflow_templates = div.table
+
+        self.assertEqual(
+            workflow_templates.attrib["id"], "workflow-templates-list"
         )
-        self.assertTextContentEqual(div.div.div[1], "prefix: test")
+
+        self.assertEqual(
+            workflow_templates.tbody.tr[0].td[0].a, "noop-template"
+        )
+        self.assertEqual(
+            workflow_templates.tbody.tr[0].td[0].a.attrib["href"],
+            template.get_absolute_url(),
+        )
+        self.assertEqual(workflow_templates.tbody.tr[0].td[1], "noop")
