@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -194,7 +194,9 @@ class WorkRequestDetailView(
                 source_artifact_ids := task.get_source_artifacts_ids()
             ):
                 source_artifacts.extend(
-                    Artifact.objects.filter(pk__in=source_artifact_ids)
+                    Artifact.objects.filter(
+                        pk__in=source_artifact_ids
+                    ).annotate_complete()
                 )
                 context["source_artifacts"] = source_artifacts
         except NotImplementedError:
@@ -203,9 +205,9 @@ class WorkRequestDetailView(
             context["source_artifacts_not_implemented"] = True
 
         built_artifacts = list(
-            Artifact.objects.filter(
-                created_by_work_request=self.object
-            ).order_by("category", "id")
+            Artifact.objects.filter(created_by_work_request=self.object)
+            .annotate_complete()
+            .order_by("category", "id")
         )
 
         # Generate UI shortcuts for artifacts
