@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright 2021-2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -67,7 +67,12 @@ from debusine.client.exceptions import TokenDisabledError
 from debusine.client.models import WorkRequestResponse
 from debusine.tasks import BaseExternalTask, BaseTask, TaskConfigError
 from debusine.tasks.executors import analyze_worker_all_executors
-from debusine.tasks.models import TaskTypes, WorkerType
+from debusine.tasks.models import (
+    OutputData,
+    OutputDataError,
+    TaskTypes,
+    WorkerType,
+)
 from debusine.worker.config import ConfigHandler
 from debusine.worker.debusine_async_http_client import DebusineAsyncHttpClient
 from debusine.worker.system_information import (
@@ -487,15 +492,22 @@ class Worker:
                 expected_task_type = TaskTypes.SIGNING
             case _ as unreachable:
                 assert_never(unreachable)
+        error_args: tuple[Any, ...]
         if task_type != expected_task_type:
-            logging.error(
-                "Task: %s is of type %s, not %s",
-                work_request.task_name,
-                task_type,
-                expected_task_type,
-            )
+            error_message = "Task: %s is of type %s, not %s"
+            error_args = (work_request.task_name, task_type, expected_task_type)
+            logging.error(error_message, *error_args)
             self._debusine.work_request_completed_update(
-                work_request.id, "error"
+                work_request.id,
+                "error",
+                OutputData(
+                    errors=[
+                        OutputDataError(
+                            message=error_message % error_args,
+                            code="wrong-task-type",
+                        )
+                    ]
+                ),
             )
             self._reset_state()
             return
@@ -505,11 +517,20 @@ class Worker:
                 TaskTypes(work_request.task_type), work_request.task_name
             )
         except ValueError as exc:
-            logging.error(  # noqa: G200
-                'Task: %s Error setup: %s', work_request.task_name, exc
-            )
+            error_message = "Task: %s Error setup: %s"
+            error_args = (work_request.task_name, exc)
+            logging.error(error_message, *error_args)
             self._debusine.work_request_completed_update(
-                work_request.id, "error"
+                work_request.id,
+                "error",
+                OutputData(
+                    errors=[
+                        OutputDataError(
+                            message=error_message % error_args,
+                            code="setup-failed",
+                        )
+                    ]
+                ),
             )
             self._reset_state()
             return
@@ -519,11 +540,20 @@ class Worker:
                 work_request.task_data, work_request.dynamic_task_data
             )
         except TaskConfigError as exc:
-            logging.error(  # noqa: G200
-                'Task: %s Error configure: %s', work_request.task_name, exc
-            )
+            error_message = "Task: %s Error configure: %s"
+            error_args = (work_request.task_name, exc)
+            logging.error(error_message, *error_args)
             self._debusine.work_request_completed_update(
-                work_request.id, "error"
+                work_request.id,
+                "error",
+                OutputData(
+                    errors=[
+                        OutputDataError(
+                            message=error_message % error_args,
+                            code="configure-failed",
+                        )
+                    ]
+                ),
             )
             self._reset_state()
             return
@@ -584,23 +614,29 @@ class Worker:
         assert self._task_running is not None
 
         try:
+            output_data = OutputData()
+
             if self._task_running.aborted:
                 logging.info("Task: %s has been aborted", self._task_name)
                 # No need to notify debusine-server
                 return
             elif task_exec_future.exception():
-                logging.error(  # noqa: G200
-                    'Task: %s Error execute: %s',
-                    self._task_name,
-                    task_exec_future.exception(),
-                )
+                error_message = "Task: %s Error execute: %s"
+                error_args = (self._task_name, task_exec_future.exception())
+                logging.error(error_message, *error_args)
                 result = 'error'
+                output_data.errors = [
+                    OutputDataError(
+                        message=error_message % error_args,
+                        code="execute-failed",
+                    )
+                ]
             else:
                 result = 'success' if task_exec_future.result() else 'failure'
 
             try:
                 self._debusine.work_request_completed_update(
-                    work_request_id, result
+                    work_request_id, result, output_data
                 )
             except Exception:
                 # Don't leave the worker stuck in "Running" state on the server
