@@ -1,4 +1,4 @@
-# Copyright 2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -24,7 +24,9 @@ def total_physical_memory() -> int:
 
 def cpu_count() -> int:
     """Return number of CPUs in the system."""
-    return psutil.cpu_count()
+    if (count := psutil.cpu_count()) is None:
+        return 1
+    return count
 
 
 @cache
