@@ -1,4 +1,4 @@
-# Copyright 2021-2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -16,7 +16,8 @@ import logging
 import signal
 import socket
 import threading
-from collections.abc import Awaitable
+from collections.abc import AsyncGenerator, Awaitable
+from contextlib import asynccontextmanager
 from datetime import datetime
 from pathlib import Path
 from typing import Any, Literal
@@ -46,7 +47,12 @@ from debusine.tasks import (
     Sbuild,
     TaskConfigError,
 )
-from debusine.tasks.models import TaskTypes, WorkerType
+from debusine.tasks.models import (
+    OutputData,
+    OutputDataError,
+    TaskTypes,
+    WorkerType,
+)
 from debusine.tasks.tests.helper_mixin import TestTaskMixin
 from debusine.test import TestCase
 from debusine.worker import Worker
@@ -595,6 +601,34 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
         self.assertEqual(count, times)
 
+    @asynccontextmanager
+    async def completed_task(self) -> AsyncGenerator[None]:
+        """Context manager that waits for a task to complete."""
+        assert self.worker is not None
+        assert self.worker._main_event_loop is not None
+
+        completed = asyncio.Event()
+
+        async def report_completed(worker: Worker) -> None:
+            # Worker._reset_state would do this; we should clean up too.
+            if worker._task_lock.locked():  # pragma: no cover
+                worker._task_lock.release()
+
+            completed.set()
+
+        with mock.patch(
+            "debusine.worker._worker.Worker._reset_state",
+            side_effect=functools.partial(
+                asyncio.run_coroutine_threadsafe,
+                report_completed(self.worker),
+                self.worker._main_event_loop,
+            ),
+        ):
+            async with asyncio.timeout(10):
+                waiter_task = asyncio.create_task(completed.wait())
+                yield
+                await waiter_task
+
     async def receive_task_submit_result(
         self,
         task_type: TaskTypes,
@@ -636,20 +670,17 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             self.patch_debusine_work_request_completed_update()
         )
 
-        await self.worker.connect()
+        async with self.completed_task():
+            await self.worker.connect()
 
         # The WorkRequest is executed...
         self.assertEqual(mocked_execute.call_count, 1)
 
         # Check the workspace and worker host architecture assigned to the
         # task
-        self.assertEqual(
-            mocked_execute.call_args[0][0].workspace_name, "TestWorkspace"
-        )
-        self.assertEqual(
-            mocked_execute.call_args[0][0].worker_host_architecture,
-            host_architecture(),
-        )
+        task = mocked_execute.call_args[0][0]
+        self.assertEqual(task.workspace_name, "TestWorkspace")
+        self.assertEqual(task.worker_host_architecture, host_architecture())
 
         # BaseTask.configure_server_access was called
         debusine_server = mocked_configure_server_access.mock_calls[0][1][1]
@@ -657,17 +688,12 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         self.assertEqual(debusine_server.token, self.config.token)
         self.assertEqual(debusine_server.base_api_url, self.config.api_url)
 
-        work_request_completed_mocked.assert_called_once()
-
-        # The first argument of the call: work_request_id
-        self.assertRegex(
-            str(work_request_completed_mocked.mock_calls[0][1][0]), r"^\d+$"
+        work_request_completed_mocked.assert_called_once_with(
+            task.work_request_id, json_result, mock.ANY
         )
 
-        # Second argument of the call: result
-        self.assertEqual(
-            work_request_completed_mocked.mock_calls[0][1][1], json_result
-        )
+        output_data = work_request_completed_mocked.call_args[0][2]
+        self.assertIsInstance(output_data, OutputData)
 
     async def test_task_is_received_executed_submit_success(self) -> None:
         """Receive a task, execute it and submit success completion."""
@@ -740,28 +766,25 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
         log_message = "Worker is busy and can't execute a new work request"
 
-        # Log that a Work Request is available (coming from the server)
-        # but the Worker ignores it because the worker is already executing
-        # a task
-        with self.assertLogsContains(log_message):
-            await self.worker.connect()
+        async with self.completed_task():
+            # Log that a Work Request is available (coming from the server)
+            # but the Worker ignores it because the worker is already
+            # executing a task
+            with self.assertLogsContains(log_message):
+                await self.worker.connect()
 
-        # Sbuild.execute() is called only once
-        self.assertEqual(mocked_execute.call_count, 1)
+            # Sbuild.execute() is called only once
+            self.assertEqual(mocked_execute.call_count, 1)
 
-        # The server sent twice "work_request_available" and the worker
-        # called get-next-for-worker/ once only: because a Task is already
-        # being executed
-        self.assert_request_done_times(
-            "GET", "/api/1.0/work-request/get-next-for-worker/", times=1
-        )
+            # The server sent twice "work_request_available" and the worker
+            # called get-next-for-worker/ once only: because a Task is
+            # already being executed
+            self.assert_request_done_times(
+                "GET", "/api/1.0/work-request/get-next-for-worker/", times=1
+            )
 
-        # Allow the execute call to finish
-        lock.release()
-
-        # Let the task properly complete
-        while self.worker._task_exec_future is not None:
-            await asyncio.sleep(0)
+            # Allow the execute call to finish
+            lock.release()
 
     async def test_log_work_request_cannot_report_exit(self) -> None:
         """Worker can't report to the server, exits."""
@@ -838,6 +861,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
     async def assert_execute_work_request_send_and_log_error(
         self,
         error_msg: str,
+        error_code: str,
         worker_type: Literal[
             WorkerType.EXTERNAL, WorkerType.SIGNING
         ] = WorkerType.EXTERNAL,
@@ -856,22 +880,21 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         )
 
         with self.assertLogsContains(error_msg):
-            with mock.patch(
-                "debusine.worker._worker.Worker._reset_state"
-            ):  # ensure worker._task_exec_future is not cleared after execution
+            async with self.completed_task():
                 await self.worker.connect()
-                # Wait for task completion (if ran at all) before
-                # checking the logs
-                if self.worker._task_exec_future is not None:
-                    concurrent.futures.wait(
-                        [self.worker._task_exec_future], timeout=10
-                    )
 
-        self.assertEqual(work_request_completed_mocked.call_count, 1)
-        self.assertRegex(
-            str(work_request_completed_mocked.call_args[0][0]), r"^\d+$"
+        work_request_completed_mocked.assert_called_once_with(
+            mock.ANY, "error", mock.ANY
+        )
+        work_request_id, _, output_data = (
+            work_request_completed_mocked.call_args[0]
+        )
+        self.assertIsInstance(work_request_id, int)
+        self.assertIsInstance(output_data, OutputData)
+        self.assertEqual(
+            output_data.errors,
+            [OutputDataError(message=error_msg, code=error_code)],
         )
-        self.assertEqual(work_request_completed_mocked.call_args[0][1], "error")
 
     async def test_task_configure_error_submit_error(self) -> None:
         """Sbuild.configure fails, Worker submits error."""
@@ -881,7 +904,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         self.addCleanup(patcher.stop)
 
         await self.assert_execute_work_request_send_and_log_error(
-            'Task: sbuild Error configure: Invalid schema'
+            "Task: sbuild Error configure: Invalid schema", "configure-failed"
         )
 
     async def test_server_task_on_external_worker(self) -> None:
@@ -930,7 +953,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         )
 
         await self.assert_execute_work_request_send_and_log_error(
-            "Task: servertest is of type Server, not Worker"
+            "Task: servertest is of type Server, not Worker", "wrong-task-type"
         )
 
     async def test_signing_task_on_external_worker(self) -> None:
@@ -976,7 +999,8 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         )
 
         await self.assert_execute_work_request_send_and_log_error(
-            "Task: signingtest is of type Signing, not Worker"
+            "Task: signingtest is of type Signing, not Worker",
+            "wrong-task-type",
         )
 
     async def test_worker_task_on_signing_worker(self) -> None:
@@ -1005,6 +1029,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
         await self.assert_execute_work_request_send_and_log_error(
             "Task: noop is of type Worker, not Signing",
+            "wrong-task-type",
             worker_type=WorkerType.SIGNING,
         )
 
@@ -1084,7 +1109,8 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
         await self.assert_execute_work_request_send_and_log_error(
             "Task: nonexistent Error setup:"
-            " 'nonexistent' is not a registered Worker task_name"
+            " 'nonexistent' is not a registered Worker task_name",
+            "setup-failed",
         )
 
     def patch_debusine_work_request_completed_update(self) -> MagicMock:
@@ -1109,7 +1135,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         self.addCleanup(patcher_execute.stop)
 
         await self.assert_execute_work_request_send_and_log_error(
-            'Task: sbuild Error execute: No suitable schroot'
+            'Task: sbuild Error execute: No suitable schroot', "execute-failed"
         )
 
     async def test_process_message_invalid_msg(self) -> None:
